package rocketBugs;
/*
	The ARP2/3 object... very lightweight
*/

/*  <RocketBugs - for the simulation of actin-based Listeria motility>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.io.*; 
import java.awt.*;

public class A_Arp {
	double loc;		// location relative to end1 of filament
	boolean active;	// flag to indicate whether or not this ARP2/3 is in active conformation
	A_Rod branch;	// pointer to filament that comes off of this ARP2/3
	int branchId;		// position of branch in player array... only used when reading and writing state to file
	static Pt3D drawPt = new Pt3D(); // for drawing only
	static Pixel drawPix = new Pixel(); // for drawing only
	
	public A_Arp (double loc, boolean active, A_Rod branch) {
		this.loc = loc;
		this.active = active;
		this.branch = branch;
	}
	
	public A_Arp (double loc) {
		this.loc = loc;
		this.active = false;
		branch = null;
	}
	
	public A_Arp (double loc, boolean active, int branchId) {	// initialization used after reading from file... branch undefined
		this.loc = loc;
		this.active = active;
		branch = null;
		this.branchId = branchId;
	}

	public static A_Arp makeActiveArp (double loc, A_Rod host) {
		if (host.selected) {
			talkln ("side binding of an active ARP2/3 at " + String.valueOf(loc));
		}
		Pt3D arpPt = Pt3D.Add(host.end1, loc, host.uVec);	// calculate location of binding in fixed coordinates
		A_Rod newBranch = ActinBranch.addFActinBranch(host, arpPt);
		newBranch.minusCapped = true;
		newBranch.aBranch = true;
		//Player.impP.seedARPa(arpPt, -1);				// remove one active ARP2/3 from the pool at this point
		return new A_Arp(loc, true, newBranch);
	}
	
	public static A_Arp makeInactiveArp (double loc, A_Rod host) {
		//talkln ("side binding of an inactive ARP2/3");
		Pt3D arpPt = Pt3D.Add(host.end1, loc, host.uVec);	// calculate location of binding in fixed coordinates
		//Player.impP.seedARP(arpPt, -1);				// remove one active ARP2/3 from the pool at this point
		return new A_Arp(loc, false, null);
	}
	
	public void activateArp (Pt3D arpPt, A_Rod host) {
		//talkln ("activating a bound Arp2/3");
		A_Rod newBranch = ActinBranch.addFActinBranch(host, arpPt);
		newBranch.minusCapped = true;
		newBranch.aBranch = true;
		active = true;
		branch = newBranch;
	}
	
	public void showYourself (Graphics g, double scale, PinCam cam, Pt3D pt) {
		int arpSize = (int)(0.007*scale);
		int halfSize = (int)(arpSize/2);
		drawPt.add(pt,Env.lengthPerActin,branch.uVec);
		drawPix.ptToPix(drawPt, scale, cam.offset, cam.e);
		if (active) { g.setColor(Env.activeARPColor); } else { g.setColor(Env.inactiveARPColor); }
		g.fillOval(drawPix.x-halfSize, drawPix.y-halfSize, arpSize, arpSize);
	}
	
	public static void writeArp (DataOutputStream ds, A_Arp arp) {
		try {	
			ds.writeFloat((float)arp.loc);
			if (arp.active) { 
				ds.writeByte(1); 
				ds.writeInt(arp.branch.idnum);
			} else { ds.writeByte(0); }
		} catch (IOException ioe) { talkln ("error in writing Arp"); }
	}
	
	public static A_Arp readArp (DataInputStream ds) {
		try {
			float f1 = ds.readFloat();
			byte b1 = ds.readByte();
			if (b1 == 1) { 
				int i1 = ds.readInt();
				return new A_Arp(f1, true, i1); 
			} else { return new A_Arp(f1); }
		} catch (IOException ioe) { talkln ("error reading Arp"); }
		return null;
	}
	
	public void initializeArpAfterRead (Player[] thePlayers) {
		if (active) { branch = (A_Rod)thePlayers[branchId]; } 	// set branch link if this is an active Arp2/3
	}
	
	
	public static void talkln (String info) {
		System.out.println(info);
	}
	
	public static void talk (String info) {
		System.out.print(info);
	}
}