package rocketBugs;
/*
	The Blob Class ... basically a ball
*/

/*  <RocketBugs - for the simulation of actin-based Listeria motility>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.Math;
import java.awt.*;

public class A_Blob extends Player {
	static final double maxRad = .01;
	static final double minRad = .01;
	static double radius;
	static double gamBlob;	// viscous drag... from Stokes law for these spheroids
	static double diffBlob;		// diffusivity... related to drag through Einstein's relation
	// some variables related to thermal force calculations
	static int iset = 0;			// used as flag for giving out Gaussian deviates
	static double gset;		// used as holder for extra deviate
	// some declarations related to sphere painting
	static int numCircs = 3;
	static int numPts = 20;
	static Pt3D [][] spherePts = new Pt3D [numCircs][numPts+1];
	
	public A_Blob (Pt3D coord, double radius,int idnum) {
		super();
		this.idnum = (short)idnum;
		this.radius = radius;
		this.coord = coord;
		playerColor = Color.red;
		// define the Stokes law effective damping coefficient... note that our radius value
		// is given in micrometers, hence the multiplication by 1.0e-6
		gamBlob = 6*Math.PI*Env.aeta*radius*1.0e-6;
		diffBlob = Env.Boltz*Env.tempK/gamBlob;
		if (spherePts[0][0] == null) { makePaintPoints(); }
	}
	
	public void sumForces () {
		
		// add random forces and calculate forces for next time step
		forceSum.inc(randForces);
		randForces.x = oneRandomForce();
		randForces.y = oneRandomForce();
		randForces.z = oneRandomForce();
	}
	
	public void movePlayer () {
		sumForces();
		if (!structureMember) {
			// explicit Euler approximation to ODE solution
			veloc.scale(1.0e6/gamBlob, forceSum); //convert velocity back to um/s with 1.0e6 multiplier
			coord.inc(Env.deltaT, veloc);
		}
		
		// reset counters etc.
		resetCounters();
	}
	
	public static double oneRandomForce () {
		double v1, v2, rsq, fac;
		if (iset == 0) {
			rsq = 0; v1 = 0; v2 = 0;
			while ((rsq >= 1.0) | (rsq == 0)) {		// do the following till we get a point in the unit circle
				v1 = 2.0*rdm.uRdm() - 1.0;		// one coordinate on unit circle...
				v2 = 2.0*rdm.uRdm() - 1.0;		// .. and another.
				rsq = v1*v1 + v2*v2;				// the vector magnitude
			}
			fac = Math.sqrt(-4*diffBlob*Env.deltaT*Math.log(rsq)/rsq);
			gset = v1*fac*gamBlob/Env.deltaT;
			iset = 1;			// set the flag.... we have stored extra deviate to gset
			return v2*fac*gamBlob/Env.deltaT;
		} else {
			iset = 0;			// we're using extra deviate, reset flag
			return gset;
		}
	}			
	
	public void reportStats() {
		talkln ("Blob: Id #" + String.valueOf(idnum) + " with coordinates {" + coord.reportCoords() + "}");
	}
	
	public static void makePaintPoints () {
		// store circumferential sphere points for later painting
		double xRad, yRad, zRad;
		double theta, phi;
		for (int i = 0; i < numCircs; i++) {
			phi = i*Math.PI/numCircs;
			for (int j = 0; j <= numPts; j++) {
				theta = j*2*Math.PI/numPts;
				xRad = radius*Math.cos(phi)*Math.sin(theta);
				zRad = radius*Math.sin(phi)*Math.sin(theta);
				yRad = radius*Math.cos(theta);
				spherePts[i][j] = new Pt3D(xRad,yRad,zRad);
			}
		}
	}
	
	public void showYourself (Graphics g, double scale, PinCam cam) {
		g.setColor(playerColor);
		Pt3D pt = new Pt3D();
		Pixel [] pix = new Pixel [2];
		pix [0] = new Pixel();  pix[1] = new Pixel();
		
		for (int i = 0; i < numCircs ; i++) {
			for (int j = 0; j <= numPts; j++) {
				pt.add(coord, spherePts[i][j]);
				pix[1].ptToPix(pt, scale, cam.offset, cam.e);
				if (j != 0) { Pixel.drawLine(g,pix[0],pix[1]); }
				pix[0].copy(pix[1]);
			}
		}
	}
	
	public void stereoVision (Graphics g, double scale, PinCam camL, PinCam camR) {
		g.setColor(playerColor);
		Pt3D pt = new Pt3D();
		Pixel [][] pix = new Pixel [2][2];
		for (int i = 0; i < 2; i ++) { pix[i][0] = new Pixel(); pix[i][1] = new Pixel(); }
		
		double cenDist = Pt3D.ptDist(coord, camL.zoomCen);
		for (int i = 0; i < numCircs ; i++) {
			for (int j = 0; j <= numPts; j++) {
				pt.add(coord, spherePts[i][j]);
				pix [0][1] = camL.ptToPix (pt, scale);
				pix [1][1] = camR.ptToPix (pt, scale);
				if (j != 0) { 
					if (pix[0][1].viewDist > cenDist) { g.setColor(playerColor.darker()); } else { g.setColor(playerColor.brighter()); }
					if  ((pix[0][0].show) | (pix[0][1].show)) { Pixel.drawLine(g,pix[0][0],pix[0][1]); }
					if  ((pix[1][0].show) | (pix[1][1].show)) { Pixel.drawLine(g,pix[1][0],pix[1][1]); }
				}
				pix[0][0] = pix[0][1];
				pix[1][0] = pix[1][1];
			}
		}
	}
	
}
	