package rocketBugs;
/*
	A_Node.... the node class is used to assemble a cellular membrane.  It has the properties
	of location, lipid #, and neighbor knowledge, among other things.  Extends Player as to
	all explicit objects in this world.
	A node keeps track of the number of lipids between itself and the node at linker 2.
*/

/*  <RocketBugs - for the simulation of actin-based Listeria motility>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.Math;
import java.awt.*;

public class A_Node extends Player {
	// definition of membrane node properties....
	static final double areaPerLipid = 0.0001;	// how much 2D space a lipid occupies... lipids/umeter^2
	static final double pinoProbPerLipid = 0.8;  // probability that a lipid will be pinocytosed
	static final double intercalationRate =  50.0; 	// proportionality rate for strain induced intercalation
	static final double elasticity = 1.0e-14;			// elastic modulus of membrane
	static final double maxLength = .06;
	static final double minLength = .01;
	static final double maxStrain = 2.0;			// maximum allowable strain in membrane section
	static final double graceCircRad = .02;		// node grace circle to fudge side of rod and other node contact
	static double equilMembraneEnclosedArea = 0; 	// area enclosed when at equilibrium
	static double membraneEnclosedArea = 0;	// how much area is enclosed by membrane structure
	
	int memIdnum;
	double lipidCount;
	double relaxArea;
	double [] segStrain = new double [6];  	// store strains in direction of each linker node
	int [] linkerIds = new int [6];		// needed to save linker values when file writing/reading
	A_Node [] linkers = new A_Node [6];	// the other nodes to which this node is linked
	double [] linkLength = new double [6];	// the length to each other linked node... calculate once each step
	
	public A_Node (Pt3D coord, Pt3D angle, double lipidinit, int memIdnum, int idnum) {
		this.memIdnum = memIdnum;
		this.idnum = (short)idnum;
		this.coord = coord;
		this.angle = angle;
		lipidCount = lipidinit;
		playerColor = Color.white;
//		initialize();
//		calculateProperties ();
	}
/*	
	public void initialize() {
		// calculate cos and sin of Euler angles only once during each time step
		for (int j = 0; j < 3; j++) {
			cosAng[j] = Math.cos(coord[j+3]);
			sinAng[j] = Math.sin(coord[j+3]);
		}
		// find the transformation matrices at this time step
		transMat ();

		calculateProperties();
	}
	
	public void sumCollisionEffects () {
		// sum collision forces.... node is a torqueless body
		for (int i = 0; i < collidercount; i++) {
			for (int j = 0; j < 3; j++) {
				forceSum [j] += forces [i][j];
			}
		}
		
		// add pressure effects
		addPressureEffects ();
		
		// find the force between this and adjoining nodes
		addLinkerForces ();
		
		// add random forces and calculate forces for next time step
		for (int j = 0; j < 3; j++) {
			forceSum[j] += randForce[j];
			randForce[j] = addRandomForce();
		}
	}
	
	public void movePlayer () {
	
		// explicit Euler approximation to ODE solution
		// convert x-y coordinates to parallel-orthog to assumed node orientation
		double forcesumXprime = forcesumX*Cang + forcesumY*Sang;
		double forcesumYprime = forcesumY*Cang - forcesumX*Sang;
		double vxprime = 1.0e6*forcesumXprime/gamParallel;	//convert velocities back to um/s
		double vyprime = 1.0e6*forcesumYprime/gamOrthog;
		// ....then transform back into fixed coordinate frame
		vx = vxprime*Cang - vyprime*Sang;
		vy = vyprime*Cang + vxprime*Sang;
		xpos = xpos + vx*deltaT;
		ypos = ypos + vy*deltaT;
		zpos = zpos + vz*deltaT;
		
		//  change the lipid count to reflect pinocytosis and intercalation
		pinocytosisANDintercalation();
		
		// calculate important node features at this new updated position
		calculateProperties();
		
		// reset counters and forcesums
		collidercount = 0;
		maybecount = 0;
		forcesumX = 0;
		forcesumY = 0;
		forcesumZ = 0;
	} 
	
	private void calculateProperties() {
		double [] coordDiffs;
		relaxArea = lipidCount*areaPerLipid;
		// calculate the distance from the node center to each of its surrounding nodes
		for (int i = 0; i < 3; i++) {
			coordDiffs = new double [] {coord[0]-linkers[i].coord[0], coord[1]-linkers[i].coord[1], coord[2]-linkers[i].coord[2]};
			linkLength[i] = Util.vecMag (coordDiffs);
			segStrain [i]  = (linkLength[i]-relaxLength)/relaxLength;
		}
		gamOrthog = 6*Math.PI*aeta*5.0e-8;
		gamParallel = 6*Math.PI*aeta*5.0e-8;
	}
	
	public double localCurvature() {
		double adjNodeDistSqrd = Math.pow(linker1.xpos-linker2.xpos,2)+Math.pow(linker1.ypos-linker2.ypos,2);
		double numerator = adjNodeDistSqrd-Math.pow(curLength,2)-Math.pow(linker1.curLength,2);
		double denom = -2*curLength*linker1.curLength;
		return Math.acos(numerator/denom);
	}
	
	public void addLinkerForces () {
		double forceMag;
		double forceLinkX,forceLinkY;
		if (segStrain > 0) { forceMag = elasticity* segStrain; } else { forceMag = 0; }
		if (segStrain > maxStrain) { forceMag = elasticity*maxStrain; }
		forceLinkX = forceMag*(linker2.xpos-xpos)/curLength;
		forceLinkY = forceMag*(linker2.ypos-ypos)/curLength;
		
		// add forces to current force summations
		forcesumX += forceLinkX;
		forcesumY += forceLinkY;
		linker2.forcesumX += -forceLinkX;
		linker2.forcesumY += -forceLinkY;
	}
		
	private void pinocytosisANDintercalation() {
		// pinocytosis
		lipidCount += -pinoProbPerLipid*lipidCount;
		// intercalation
		if (segStrain > 0) { lipidCount += intercalationRate*segStrain; }
		// immediate flood of lipids
		if (segStrain > maxStrain) { lipidCount = curLength/(lengthPerLipid*(maxStrain + 1)); }
	}
	
	private void addPressureEffects() {
		double forceMag;
		double forceLinkX,forceLinkY;
		forceMag = 1.0e-12*(equilMembraneEnclosedArea-membraneEnclosedArea)*curLength;
		forceLinkX = forceMag*(linker2.ypos-ypos)/curLength;	// cos(th-PI/2) = +sin(th)
		forceLinkY = -forceMag*(linker2.xpos-xpos)/curLength;	// sin(th-PI/2) = -cos(th)
		
		// add forces to current force summations for this node and linker2
		forcesumX += forceLinkX/2;
		forcesumY += forceLinkY/2;
		linker2.forcesumX += forceLinkX/2;
		linker2.forcesumY += forceLinkY/2;
	}
		
	public void showYourself (Graphics g, double scale, double xoffset, double yoffset) {
		//note: xoffset,yoffset are non-zero only for zoom window
		int x1disp,x2disp,y1disp,y2disp;
		g.setColor(playerColor);
		x1disp = (int)((xpos-xoffset)*scale);   // get closest integers for painting
		x2disp = (int)((linker2.xpos-xoffset)*scale);
		y1disp = (int)((ypos-yoffset)*scale);
		y2disp = (int)((linker2.ypos-yoffset)*scale);
		g.drawLine (x1disp,y1disp,x2disp,y2disp);
	}
	
	public void prepareToSave () {
		if (linker1 != null) {linker1Id = linker1.idnum; } else { linker1Id = -1;}
		if (linker2 != null) {linker2Id = linker2.idnum; } else { linker2Id = -1;}
		linker1 = null;
		linker2 = null;
	}
	
	public void initializeFromFileAction (Player [] thePlayers) {
		if (linker1Id != -1) {linker1 = (A_Node)thePlayers[linker1Id]; } else { linker1 = null; }
		if (linker2Id != -1) {linker2 = (A_Node)thePlayers[linker2Id]; } else { linker2 = null; }
	} */
	
}
		
	