package rocketBugs;
/*
	The ActA object... not a extension of Player, just lightweight
*/

/*  <RocketBugs - for the simulation of actin-based Listeria motility>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.Math;
import java.awt.*;
import java.util.*;

public class ActA extends Linker {
	Listeria listeria;				// the listeria on which this ActA lives
	A_Rod tetherant = null;			// the rod to which this ActA is bound
	//Pt3D locOnLis = new Pt3D();		// MOVED to top of Linker file. location relative to listeria.rec1 in listeria's local (x) coordinate system
	Pt3D normalVec = new Pt3D();		// unit vector normal to listeria surface in local coordinates
	Pt3D normalVecInX = new Pt3D();	// as above but in X
	double tetherantAngle;			// angle between -normal vector and filament ... impingement/tether angle
	Pt3D tetherantLoc = new Pt3D();	// store the location in Listeria's local coordinates, of tetherant end2
	double stretchDist;	
	double maxStretchDistFromActA;
	double bindLoc;					// location from end1 of rod to binding location
	// variables describing state of ActA
	boolean arpBound = false;		// flag indicates if Arp2/3 is bound
	boolean vaspBound = false;		// flag indicates if Vasp is bound
	boolean replaceable = false;
	
	static Object syncTester = new Object();		// test for memory leak in mt code

	// probabilities that are static
	static double arpOnProb = Env.arpOnFudgeFactor*Env.kArpFluxIn*Env.deltaT*Env.ActA_ArpGrabAreaRatio;	// (/s) number Arp2/3 hitting surface * fractional surface area
	
	static double detachProb = Env.kActAFactinArp_Off*Env.deltaT;		// spontaneous dissociation rate for a bound actin filament
	static double actinOnProb = Env.kActinFluxIn*Env.deltaT*Env.ActA_ArpGrabAreaRatio;
	static double actinOffProb = Env.kActAActin_Off*Env.deltaT;
	
	static double chemTetherOffProb = Env.chemTetherOffRate*Env.deltaT;
	double stretchChemTetherOffProb;
	
	//for counting
	static int activeCounter = 0;		// counter that keeps track of # of active ActA/Arp that are nucleating filaments (should always be Env.numberActA's)
	
	// for painting.. only initialized if !remote
	Pixel locPix;			// a recycled pixel for drawing
	Pixel tPix;			// pixel marking end ActA or tether
	Pt3D tPt;			// pt marking end of ActA or tether
	
	// dummy vecs for calculations
	Pt3D vec1 = new Pt3D();
	
	public ActA (Listeria listeria, Pt3D location) {
		super();
		player1 = listeria;		// define player1 for Linker object
		this.listeria = listeria;
		locOnLis.copy(location);
		// calculate normalVec only once
		if (locOnLis.x > 0 ) { 			// on the cylinder
			normalVec.copy(0,locOnLis.y,locOnLis.z);
			normalVec.unitVec();
		} else {
			normalVec.unitVec(locOnLis);
		}
		if (!Env.remote) { initializeGraphics(); }
		updateState();		// initialize positions
		//talkln ("arpOnProb = " + arpOnProb);
		//talkln ("detachProb = " + detachProb);
	}
	
	public void sepaku () {
		// cut all links to objects
		listeria = null;		
		tetherant = null;
		locOnLis = null;
		normalVec = null;
		normalVecInX = null;
		tetherantLoc = null;

		forceMagTrack = null;
		stretchTrack = null;
		
		locPix = null;
		tPix = null;
		tPt = null;
		
		vec1 = null;
		
		super.sepaku();
	}
	
	private void initializeGraphics() {
		locPix = new Pixel();			// a recycled pixel for drawing
		tPix = new Pixel();			// pixel marking end ActA or tether
		tPt = new Pt3D();			// pt marking end of ActA or tether
	}
	
	public void updateState () {
		calculateLocationAndNormalInX ();
		updatePlayer2State();
		updateReactions ();
	}
	
	public void updateReactions () {
		// check for tether detachment
		if (linked) {
			checkTetherDetach(); 
		}
		
		// check for binding of Arp2/3 to ActA_Filament complex
		if ((linked) & (!arpBound)) { 
			checkArpOn (); 	// assumed equivalent to making new branch, for now
		}
		
		// check if a branch should be formed
		if ((arpBound) & (linked)) { 
			nucleateSideBranch();
		}
		// check for loss of Arp2/3 by this ActA and new filament nucleation
		if ((arpBound) & (!linked)) { // can only be arpBound and not linked if a nucleator
			if (Env.swapActiveActAs) { checkArpOff (); }
			if (Env.actANucleation) { checkForNucleation(); } 
		}
		// check for ActAs that need to be removed
		if ((removable) & (!linked)) {
			remove = true;	
		}
		// check for loss of Vasp by this ActA
		if (vaspBound) { checkVaspOff (); }
		
		// check if plus-end cap on actin should be removed
		if ((Env.actAProtectsCapping) & (linked)) {
			double distToEnd2 = tetherant.length - bindLoc;
			if (distToEnd2 < Env.actAProtectsCappingDist) {
				tetherant.plusCapOff();
			}
		}
		
		
		
		
	}
	
	private void updatePlayer2State () {
		checkState();
		if (linked) {
			player2 = tetherant;		// make sure player2 points to tetherant
			setLinkPt2();
			tetherant.registerActA(bindLoc);
			calculateTetherantAngle();
			calculateLinkForce ();
		}
	}
	
	public void checkState() {		// ActA only needs to check if the filament has gone missing
		if (player1 == null) { talkln ("Linker error: player1 is null"); }
		if (player2 == null) { 
			linked = false;
		} else {
			if (player2.remove) { 
				linked = false;
				player2 = null;
			}
		}
	}
	
	public void reportState () {
		talkln ("  ActA id# " + String.valueOf(idnum));
		talkln ("player1 is " + String.valueOf(player1));
		talkln ("player2 is " + String.valueOf(player2));
		talkln ("normalVecInX is " + normalVecInX.reportCoords());
		talkln ("linkPt1 is " + linkPt1.reportCoords());
		talkln ("linkPt2 is " + linkPt2.reportCoords());
		talkln (" linkForce is " + linkForce.reportCoords());

	}
		
	public void addTether (A_Rod newTetherant, double bindLoc) {
		tetherant = newTetherant;
		player2 = newTetherant;		// define player2 for Linker object
		newTetherant.linkerOn(this);
		linked = true;
		this.bindLoc = bindLoc;
		updatePlayer2State();
		if(Env.reportRegionInfo){
			registerRegionTether(this.locOnLis);
		}
		
		Env.registerNewTether(locOnLis);
	}
	
	public void registerRegionTether(Pt3D actAPtInx){
		double xPos = actAPtInx.x;
		double xInc = Env.listeriaCylLength/3.0;
		if ((xPos >= -Env.listeriaCapRad) & (xPos <= 0)) { Env.newTetherCtInA ++; }
		else if ((xPos >= 0) & (xPos <= xInc)) { Env.newTetherCtInB ++; }
		else if ((xPos >= xInc) & (xPos <= 2*xInc)) { Env.newTetherCtInC ++; }
		else if ((xPos >= 2*xInc) & (xPos <= 3*xInc)) { Env.newTetherCtInD ++; }
		else if ((xPos >= 3*xInc) & (xPos < Env.listeriaCylLength + Env.listeriaCapRad)) { Env.newTetherCtInE ++; }
	}
	
	public static void countStrainedTethersInRegions () {
		Env.strainedTetherCtInA = 0;
		Env.strainedTetherCtInB = 0;
		Env.strainedTetherCtInC = 0;
		Env.strainedTetherCtInD = 0;
		Env.strainedTetherCtInE = 0;
		for (int i=0;i<linkerCt;i++) {
			if (theLinkers[i].linked) {
				double stretch = theLinkers[i].stretchTrack.averageVal();
				if ((stretch < Env.maxTetherStretch) & (stretch > 0)) { 
					double xPos = theLinkers[i].locOnLis.x;
					double xInc = Env.listeriaCylLength/3.0;
					if ((xPos >= -Env.listeriaCapRad) & (xPos <= 0)) { Env.strainedTetherCtInA ++; }
					else if ((xPos >= 0) & (xPos <= xInc)) { Env.strainedTetherCtInB ++; }
					else if ((xPos >= xInc) & (xPos <= 2*xInc)) { Env.strainedTetherCtInC ++; }
					else if ((xPos >= 2*xInc) & (xPos <= 3*xInc)) { Env.strainedTetherCtInD ++; }
					else if ((xPos >= 3*xInc) & (xPos < Env.listeriaCylLength + Env.listeriaCapRad)) { Env.strainedTetherCtInE ++; }
					
				}
			}
		}
	}
	
		
	public void removeTether () {
		linked = false;
		tetherant.linkerOff(this);
		tetherant.aNewNuc = false;
		tetherant = null;
		player2 = null;
		if (removable) { arpBound = false; }	// only remove Arp2/3 from ActAs that don't represent equil. Arp bound set
		if (!removable) { arpBound = true; if(Env.keepTrackActAs){activeCounter--;}}	// Replace Arp2/3 to ActAs that represent equil. Arp bound set
		if(Env.keepTrackActAs & replaceable){activeCounter--;}
		if(Env.reportRegionInfo){
			double xPos = this.locOnLis.x;
			double xInc = Env.listeriaCylLength/3.0;
			if ((xPos >= -Env.listeriaCapRad) & (xPos <= 0)) { Env.cutTetherCtInA ++; }
			else if ((xPos >= 0) & (xPos <= xInc)) { Env.cutTetherCtInB ++; }
			else if ((xPos >= xInc) & (xPos <= 2*xInc)) { Env.cutTetherCtInC ++; }
			else if ((xPos >= 2*xInc) & (xPos <= 3*xInc)) { Env.cutTetherCtInD ++; }
			else if ((xPos >= 3*xInc) & (xPos < Env.listeriaCylLength + Env.listeriaCapRad)) { Env.cutTetherCtInE ++; }
		}
		Env.registerCutTether(locOnLis);
	}
	
	public void checkTetherDetach () {
		double tetherStretchFactor = 1;	// modify for some filaments
		
		if (Env.smallFilLargerStrain){
			// multiple reasons we might detach... and conditions under which we don't want to detach
			if ((tetherant.aNewNuc) & (tetherant.length < 15*Env.lengthPerActin) & (tetherant.age < 1)) {
					tetherStretchFactor = 5; // to stop some new nucs from shooting off into space
			}
		}
		
		if (Env.oldTetherProperties){
			if (Rdm.uRdm(this) < detachProb) {
				removeTether();
				return;
			}
		
			if (stretchTrack.averageVal() > tetherStretchFactor*tetherant.getMaxTetherStretch()) { 
				removeTether(); 
				return;
			}
		}
		
	}
	
	public void nucleateSideBranch () {
		A_Arp arp = tetherant.addArp(bindLoc);	// tell the filament to bind an Arp2/3 at this point
		if (arp != null) {
			arpBound = false;		// we used the Arp2/3 to create this side-branch
		}
	}
	
	public void nucleateTetherant () {
		int monsToNuc = 3;
		vec1.scale(-1, normalVecInX);	// opposite direction of normalVec
		Pt3D nucPt = Pt3D.Add(linkPt1, (0.5*monsToNuc)*Env.lengthPerActin+Env.actATetherLength, normalVecInX);
		A_Rod newTetherant = A_Rod.makeNewRod(nucPt, vec1, monsToNuc);
		newTetherant.aNewNuc = true;
		//if (Env.qkMovie) { File_Ops.saveNucInfo(nucPt); }
		addTether (newTetherant, monsToNuc*Env.lengthPerActin);
		arpBound = false;
		if(Env.keepTrackActAs){activeCounter++;}
	}
	
	public void checkForNucleation () {  //REMOVE CHANGE
		//double actinConc = listeria.impP.getProfilin_ATP_ACTINValue(linkPt1);
		if(Env.keepTrackActAs){
			if(activeCounter<Env.numberActAs){
				double actinConc = Env.PROFILIN_ATP_ACTINConc;
				double nucProb = Env.nucProbFactor*Env.kActAActin_On*actinConc*Env.deltaT;
				//talkln ("nucProb = " + String.valueOf(nucProb));
				//talkln ("arpOnProb = " + String.valueOf(arpOnProb));
				//talkln ("ActA_ArpGrabAreaRatio = " + String.valueOf(Env.ActA_ArpGrabAreaRatio));
				double newRdm = Rdm.uRdm(this);
				if (newRdm < nucProb) { nucleateTetherant ();}
			}
		}
		else{
			double actinConc = Env.PROFILIN_ATP_ACTINConc;
			double nucProb = Env.nucProbFactor*Env.kActAActin_On*actinConc*Env.deltaT;
			//talkln ("nucProb = " + String.valueOf(nucProb));
			//talkln ("arpOnProb = " + String.valueOf(arpOnProb));
			//talkln ("ActA_ArpGrabAreaRatio = " + String.valueOf(Env.ActA_ArpGrabAreaRatio));
			double newRdm = Rdm.uRdm(this);
			if (newRdm < nucProb) { nucleateTetherant ();}
		}
	}
	
	public void calculateLocationAndNormalInX () {
		linkPt1.xToX(listeria, locOnLis);	// vector from rec1 to ActA in listeria's local coordinate sys
		linkPt1.inc(listeria.rec1);			// add location of rec1
		normalVecInX.xToX(listeria, normalVec);
	}
	
	private void setLinkPt2 () { 
		linkPt2.copy(tetherant.filamentLocToPt3D(bindLoc));
	}
	
	private void calculateTetherantAngle () {
		vec1.scale(-1,normalVecInX);		// unit vector from ActA inwards
		double theDot = Pt3D.Dot(vec1,tetherant.uVec);
		tetherantAngle = Math.acos(theDot);
	}
	
	public void calculateLinkForce () {
		stretchDist = Pt3D.ptDist(linkPt1, linkPt2);
		stretchTrack.registerValue(stretchDist);
		double impDist = stretchDist-Env.actATetherLength;
		if ((stretchDist < Env.actATetherLength) | (Env.noActALinkForce)) { 
			zeroForce();
			return;
		} else {
			linkUVec.unitVec (stretchDist, linkPt2, linkPt1);  
		}
		if(Env.oldTetherProperties){
			if (! tetherant.fixedInSpace) {
				//linkMag = 0;
				linkMag = 1.0e-6*impDist/((1/tetherant.bTransGam.x + 1/listeria.bTransGam.x)*Env.deltaT);
				if ((linkMag < 0) | (Double.isNaN(linkMag))) { linkMag = 0; }		// failsafe here
			} else {
				/*// ask filament for its stiffness in a particular direction and use that stiffness to calculate the tether force
				linkMag = tetherant.getStiffness(Pt3D.Scale(-1,linkUVec))*(impDist);*/
				linkMag = 1.0e-6*impDist*listeria.bTransGam.x/Env.deltaT;
				if ((linkMag < 0) | (Double.isNaN(linkMag))) { linkMag = 0; }		// failsafe here
			}
		}
		
		if(Env.mechanicalTether & !Env.chemicalTether){
			//if mechanical tether then F = kx (where direction determined by the unitvector)
			if(Env.constantMechanical){
				linkMag = 1.0e-6 * Env.maxTetherForce;
			}
			else{
				linkMag = 1.0e-6*impDist*Env.tetherSpringK;
				if ((linkMag < 0) | (Double.isNaN(linkMag))) { linkMag = 0; }		// failsafe here
			}
		}
		
		if(Env.chemicalTether & !Env.mechanicalTether){
			//if chemical only tether, then force is constant - Fmax
			linkMag = 1.0e-6 * Env.maxTetherForce;
		}
		
		if(Env.mechanicalTether & Env.chemicalTether){
			//if mechanical and chemical tether, force is like spring but off rate is based on stretchdistance (see ActA.checkTetherDetach)
			if(Env.constantMechanical){
				linkMag = 1.0e-6 * Env.maxTetherForce;
			}
			else{
				linkMag = 1.0e-6*impDist*Env.tetherSpringK;
				if ((linkMag < 0) | (Double.isNaN(linkMag))) { linkMag = 0; }		// failsafe here
			}
		}
		
		
		linkMag = Env.linkAtten*linkMag;
		linkForce.scale(linkMag, linkUVec);  // force applied to listeria from tether bond
		
		if (!linkForce.checkPt3D() | Env.tetherForceOff) { 		// once more, check if force is OK
			zeroForce();
		} else {
			forceMagTrack.registerValue(linkMag);
		}
	}
	
	private void zeroForce() {
		linkMag = 0;
		linkForce.zero();
		forceMagTrack.registerValue(0);
	}
	
	/*private void calculateLinkForce () {
		stretchDist = Pt3D.ptDist(linkPt1, linkPt2);
		stretchTrack.registerValue(stretchDist);
		double impDist = stretchDist-Env.actATetherLength;
		if (stretchDist < Env.actATetherLength) {
			linkMag = 0;
			linkForce.zero();
			forceMagTrack.registerValue(0);
			return;
		} else {
			linkUVec.unitVec (stretchDist, linkPt2, linkPt1);  
		}
		
		if (! tetherant.fixedInSpace) {
			//linkMag = 0;
			linkMag = 1.0e-6*impDist/((1/tetherant.bTransGam.x + 1/listeria.bTransGam.x)*Env.deltaT);
			if ((linkMag < 0) | (Double.isNaN(linkMag))) { linkMag = 0; }		// failsafe here
		} else {
			// ask filament for its stiffness in a particular direction and use that stiffness to calculate the tether force
			//linkMag = tetherant.getStiffness(Pt3D.Scale(-1,linkUVec))*(impDist);
			linkMag = 1.0e-6*impDist*listeria.bTransGam.x/Env.deltaT;
			if ((linkMag < 0) | (Double.isNaN(linkMag))) { linkMag = 0; }		// failsafe here
		}
		linkForce.scale(linkMag, linkUVec);  // force applied to listeria from tether bond
		forceMagTrack.registerValue(linkMag);
	}
	*/
	
		
		
	/*private void calculateTetherForce () {
		stretchDist = Pt3D.ptDist(getlinkPt1(), getlinkPt2());
		//stretchTrack.registerValue(stretchDist);
		if (stretchDist < Env.noForceRadius) {
			linkUVec = Pt3D.RandomUnitVec();
		} else {
			linkUVec.unitVec (stretchDist, getlinkPt2(), getlinkPt1());  // ** don't go here if stretchDist = 0 **
		}
		if (! tetherant.fixedInSpace) {
			// figure out the appropriate force to bring rod back to ActA (see notebook)
			double rCrossFMag = Pt3D.CrossMag(tetherant.uVec, linkUVec);	// sin() of angle between them, always between 0 and 1 here
			double r = 1.0e-6*tetherant.length/2.0;		// length from CM to end2 in meters
			double gamX = tetherant.bTransGam.x;	// translational drag for filament
			double gamTh = tetherant.bRotGam.y;		// rotational drag about y or z
			double gamL = listeria.bTransGam.x;		// translational drag for Listeria
			double gamDenomTerm = (gamTh*gamL+r*r*rCrossFMag*gamX*gamL+gamX*gamTh);
			linkMag = (1.0e-6*(stretchDist-Env.noForceRadius)/Env.deltaT)*(gamX*gamTh*gamL/gamDenomTerm);
			checkTetherBreak();	// do this before factoring in other tethers 
			linkMag = linkMag/tetherant.actACount;		// assume all other ActAs are pulling similarly
			if ((linkMag < 0) | (Double.isNaN(linkMag))) { linkMag = 0; }		// failsafe here
			//forceMagTrack.registerValue(linkMag);
		} else {
			// ask filament for its stiffness in a particular direction and use that stiffness to calculate the tether force
			linkMag = tetherant.getStiffness(Pt3D.Scale(-1,linkUVec))*(stretchDist-Env.noForceRadius);
			if ((linkMag < 0) | (Double.isNaN(linkMag))) { linkMag = 0; }		// failsafe here
			//forceMagTrack.registerValue(linkMag);
			checkTetherBreak();	
		}
		
		// Doing this above now checkTetherBreak();	// break the bond, maybe.
		
		if (tetherBroken) {	// if checkTetherBreak has broken bond, set tethering force to zero
			linkForce.zero();
		} else {
			// force applied to listeria from tether bond
			linkForce.scale(linkMag, linkUVec);
		}
		tetherForceCurrent = true;
	} */
	
	public void checkArpOn() {
		if (Rdm.uRdm(this) < arpOnProb) {
			arpBound = true;	// bind arp to this ActA
		}
	}
	
	public void checkArpOff() {
		if (Rdm.uRdm(this) < Env.kBoundActAArp_Off*Env.deltaT) { 
			arpBound = false;
		}
	}
	
	public void checkVaspOff() {
	}
	
	public static void mkActALocHisto () {
		talkln ("Making ActA location histogram...");
		String fileName = null;
		if (!Env.remote) {
			fileName = FileOps.getFileToSave("File for ActA Histogram",null);
		} else {
			fileName = Env.datFilePath + "/" + Env.qkMovieFileName + "ActALoc";
		}
		if (fileName != null) {
			Date timeNow = new Date();
			HistogramPlus actALocHisto = new HistogramPlus(150,-7.98,7.98,null,"ActALocations",null);
			actALocHisto.infoS = new String [2];
			actALocHisto.infoS[0] = "ActA locations for " + fileName + " run on " + timeNow.toString();
			actALocHisto.infoS[1] = "#ActAs = " + Env.numberActAs + " : RdmAmp = " + Env.rdmAmp;
			for (int i=0;i<Linker.linkerCt;i++) {
				Linker curLinker = Linker.theLinkers[i];
				if (curLinker instanceof ActA) {
					ActA curActA = (ActA) curLinker;
					actALocHisto.addValue(curActA.locOnLis.x - Listeria.cylLength/2 + 1.0e-8);	// use center of Lis as origin
				}
			}
			talk ("about to write file...");
			FileOps.mkHistogramFile(actALocHisto,fileName);
			talkln ("Done!");
			
			if ((Env.plotActAHist) & (!Env.remote)) {
				talk ("about to plot ActA locations histogram...");
				HistoPlotter actAHistP = new HistoPlotter();
				actAHistP.setPlotColor(Color.blue);
				actAHistP.setYPixelPad(300);
				actAHistP.plotHist(actALocHisto,true);
				FileOps.takeRawPicture(HistoPlotPanel.theImage,fileName + ".RAW");
				talkln ("Done!");
			}
		} else {
			talkln ("Cancelled!");
		}
	}
			

	
	public void showYourself (Graphics g, double scale, PinCam cam) { 
		locPix.ptToPix(linkPt1,scale,cam.offset,cam.e);
		int ActAPixels = (int) (scale*0.002);
		// some fancier drawing of ActA
		if (!linked) {	// only show the stub line if not tethering
			if (Env.showUntetheredActAs) {
				g.setColor(Env.unboundActAColor);
				g.drawOval(locPix.x-ActAPixels/2, locPix.y-ActAPixels/2, ActAPixels, ActAPixels);
				tPt.add(linkPt1, Rdm.uRdm(this)*Env.actATetherLength, normalVecInX);
				drawFloppy (tPt, g, scale, cam);
			}
		} else {
			g.setColor(Env.boundActAColor);
			g.drawOval(locPix.x-ActAPixels/2, locPix.y-ActAPixels/2, ActAPixels, ActAPixels);
			tPt = linkPt2;
			if (linkMag > 0) {
				tPix.ptToPix(tPt,scale, cam.offset, cam.e);
				Pixel.drawLine(g, locPix, tPix);
			} else {
				drawFloppy(tPt, g, scale, cam);
			}
		}
	}
	
	private void drawFloppy (Pt3D otherPt, Graphics g, double scale, PinCam cam) {
		double flopDist = Pt3D.ptDist(linkPt1, otherPt);
		Pt3D flopUVec = Pt3D.UnitVec(otherPt, linkPt1);
		Pt3D flopTVec = Pt3D.Cross(flopUVec, Pt3D.RandomUnitVec());	// normal to flopUVec but in random direction
		Pt3D dX = new Pt3D();
		Pt3D dR = new Pt3D();
		Pixel oldTPix = new Pixel();
		double ampA = (flopDist/Env.actATetherLength)*0.001;
		int numPts = 50;
		int numLoops = 10;
		double angleInc = numLoops*Math.PI/numPts;
		double xInc = flopDist/numPts;
		oldTPix.ptToPix(linkPt1, scale, cam.offset, cam.e);
		for (int i=0; i<=numPts; i++) {
			dX.scale(i*xInc, flopUVec);
			dR.scale(ampA*Math.sin(i*angleInc), flopTVec);
			tPt.add(dX,dR);
			tPt.inc(linkPt1);
			tPix.ptToPix(tPt, scale, cam.offset, cam.e);
			Pixel.drawLine(g, oldTPix, tPix);
			oldTPix.copy(tPix);
		}
	}
		
	public static void talkln (String info) {
		FileOps.talkln(info);
	}
	
	public static void talk (String info) {
		FileOps.talk(info);
	}
				
}