package rocketBugs;
/* Barrier.java ... this class is used to collect threads that are working on different parts
of the same task together.
*/

/*  <RocketBugs - for the simulation of actin-based Listeria motility>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


public class Barrier {
	Object syncO = new Object();
	Timer timer;
	int numThreads;
	int numThreadsIn = 0;
	int numThreadsOut = 0;
	boolean jobDone = false;
			
	public Barrier (int numThreads, String commandName) {
		this.timer = new Timer (commandName);
		this.numThreads = numThreads;
	}
		
	public void startThreads() {
		synchronized(syncO) {
			timer.start(); 
			numThreadsIn = 0;
			numThreadsOut = 0;
			jobDone = false;
			syncO.notifyAll();
			}
	}
		
	public void waitOnThreads() {
		synchronized(syncO) {
			while (!jobDone) {
				try { syncO.wait(); } catch(Exception e) { talk (" Exception waiting in waitOnThreads "); }
			}
			timer.inc();
		}
	}		
		
	public void threadDone() {
		synchronized(syncO) {
			numThreadsIn += 1;
			while (numThreadsIn < numThreads) {
				try { syncO.wait(); } catch(Exception e) { talk (" Exception waiting in threadDone() "); }
			}
			numThreadsOut += 1;
			if (numThreadsOut == numThreads) { // once the last thread in (first out) comes through... job done
				timer.stop(); 
				jobDone = true;  
			}
			syncO.notifyAll();
			while (numThreadsOut > 0) {
				try { syncO.wait(); } catch(Exception e) { talk (" Exception waiting in threadDone() "); }
			}
		}
	}
	
	private void talk (String info) {
		System.out.println(info);
	}
}