package rocketBugs;
/* Collision ... this object defines a collision between two players.  Stores the force,
position, and identities of those involved.  
*/

/*  <RocketBugs - for the simulation of actin-based Listeria motility>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.Math.*;

public class Collision {
	static Collision [] collisions = new Collision [Env.maxCollisions];
	static int colCt = 0;
	static ColThreads colTs = new ColThreads ();
	static int curCounter = 0;		// keeps track of current time for resetting colCt
	double forceMag;
	Pt3D forceVec;
	Pt3D forcePt;
	Player player1;
	Player player2;
	double attenVal;
	static Object colLock = new Object();		// object for collision synchronization
	
	public Collision (Player player1, Player player2, double forceMag, Pt3D forceVec, Pt3D forcePt) {
		this.player1 = player1;
		this.player2 = player2;
		this.forceMag = forceMag;
		this.forceVec = new Pt3D(forceVec);
		this.forcePt = new Pt3D(forcePt);
		newCollision (this);
	}
	
	public Collision () {
		this.player1 = null;
		this.player2 = null;
		this.forceMag = 0;
		this.forceVec = new Pt3D();
		this.forcePt = new Pt3D();
	}
	
	static class ColThreads extends ThreadSet {
		ColThreads () {
			super (Env.numberOfCollisionThreads, "Collision Threads");
		}
		
		public void divideAndConquer (int jobId) {
			this.jobId = jobId;
			switch (jobId) {
				case Env.colSum1Start:
					for (int i=0; i <= numThreads; i++) {
						jobDiv[i] = i*colCt/numThreads;	// divide the job amongst threads
					}
					spawn(); break;
			}
		}
		
		public void regroup (int jobId) {
			switch (jobId) {
				case Env.colSum1Stop:
					gather(); break;
			}
		}
		
		public void execute (int threadId) {
			switch (jobId) {
				case Env.colSum1Start:
					for (int i = jobDiv[threadId]; i < jobDiv[threadId+1]; i++) {
						collisions[i].enforceCollision ();
					}
					break;
			}
		}
	}
	
	public static void initializeCollisionArray() {
		for (int i=0;i<Env.maxCollisions;i++) {
			collisions[i] = new Collision();
		}
	}
	
	public void sepaku () {
		forceVec = null;
		forcePt = null;
		player1 = null;
		player2 = null;
	}
	
	public void enforceCollision () {
		player1.collision(forcePt, forceVec);
		player2.collision(forcePt, Pt3D.Reverse(forceVec));
		player2.regionCollision(forcePt, Pt3D.Reverse(forceVec));
	}
	
	public static void enforceAllCollisions () {
		for (int i = 0; i < colCt; i++) {
			collisions[i].enforceCollision();
		}
	}
	
	public static void newCollision (Player player1, Player player2, double forceMag, Pt3D forceVec, Pt3D forcePt) {
		synchronized (colLock) {
			if (curCounter != Env.counter) {
				colCt = 0;			
				curCounter = Env.counter; 
			}
			collisions[colCt].player1 = player1;
			collisions[colCt].player2 = player2;
			collisions[colCt].forceMag = forceMag;
			collisions[colCt].forceVec.copy(forceVec);
			collisions[colCt].forcePt.copy(forcePt);
			collisions[colCt].attenVal = 1;
			
			colCt++;
		}
	}
	
	private static void newCollision (Collision newCol) {
		synchronized (colLock) {
			if (curCounter != Env.counter) {
				colCt = 0;			
				curCounter = Env.counter; 
			}
			collisions[colCt] = newCol;
			colCt++;
		}
	}
	
	public void attenCollision () {
		attenVal = 1;
		
		if (player1.structureMember) {
			if (player1.myStructure.colAtten < attenVal) { attenVal = player1.myStructure.colAtten; }
		} else {
			if (player1.colAtten < attenVal) { attenVal = player1.colAtten; }
		}
		
		if (player2.structureMember) {
			if (player2.myStructure.colAtten < attenVal) { attenVal = player2.myStructure.colAtten; }
		} else {
			if (player2.colAtten < attenVal) { attenVal = player2.colAtten; }
		}
		
		// buckling
	/*	if (player1.buckleAtten < attenVal) { attenVal = player1.buckleAtten; }
		if (player2.buckleAtten < attenVal) { attenVal = player2.buckleAtten; } */
		
		forceMag = forceMag*attenVal;
		forceVec.scale(attenVal);
	}
	
	public static void attenAllCollisions () {
		for (int i=0; i < colCt; i++) {
			collisions[i].attenCollision();
		}
	}
	
	public static void reset () {
		colCt = 0;
	}
	
	public static void talkln (String info) {
		FileOps.talkln(info);
	}
	
	public static void talk (String info) {
		FileOps.talk(info);
	}
	
	
}
	