package rocketBugs;
/*
	FileOps.... a class definining all the file i/o
*/

/*  <RocketBugs - for the simulation of actin-based Listeria motility>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.Math;
import java.awt.*;
import java.io.*;
import java.awt.image.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;

/*Used in various places.. the following code
  1 - A_Rod
  2 - A_Node
  3 - A_Blob
  4 - Listeria
  5 - ActinBranch
  6 - ActinMesh
*/

public class FileOps {
	RocketBugs MC;
	static String sepString = ";";
	// file, file writer, and print writer for program messages to file
	static File messagesOutFile;
	static FileWriter messagesOutFW;
	static PrintWriter messagesOutPW;	
	// file, file writer, and print writer for Listeria position file
	static File listeriaPositionFile;
	static FileWriter listeriaPositionFW;
	static PrintWriter lisPosPW;
	// file, file writer, and print writer for nucleation information file
	static File nucInfoFile;
	static FileWriter nucInfoFW;
	static PrintWriter nucInfoPW;
	// file, file writer, and print writer for Listeria regional information file
	static File listeriaRegionFile;
	static FileWriter listeriaRegionFW;
	static PrintWriter lisRegPW;
	// file, file writer, and print writer for Listeria propelling force file
	static File propForceFile;
	static FileWriter propForceFW;
	static PrintWriter propForcePW;
	
	// file, file writer, and print writer for Listeria forces information file
	static File listeriaForceFile;
	static FileWriter listeriaForceFW;
	static PrintWriter lisForcePW;

	
	public FileOps (RocketBugs MC) {
		this.MC = MC;
		if (Env.qkMovie) { setUpFiles();}
	}
	
//	 A class to filter out .java files
	class SourceFileFilter implements FilenameFilter {

    /**
    * Select only .java files.
    *
    * @param dir the directory in which the file was found.
    *
    * @param name the name of the file
    *
    * @return true if and only if the name should be
    * included in the file list; false otherwise.
    */
    public boolean accept ( File dir, String name )
      {
      if ( new File( dir, name ).isDirectory() )
         {
         return false;
         }
      name = name.toLowerCase();
      return name.endsWith( ".java" );
      }
    }
	
//	 A class to filter out .jar files
	class SourceJarFileFilter implements FilenameFilter {

    /**
    * Select only .jar files.
    *
    * @param dir the directory in which the file was found.
    *
    * @param name the name of the file
    *
    * @return true if and only if the name should be
    * included in the file list; false otherwise.
    */
    public boolean accept ( File dir, String name )
      {
      if ( new File( dir, name ).isDirectory() )
         {
         return false;
         }
      name = name.toLowerCase();
      return name.endsWith( ".jar" );
      }
    }
	
	public static void copyFile(File in, File out) throws Exception {
	    FileInputStream fis  = new FileInputStream(in);
	    FileOutputStream fos = new FileOutputStream(out);
	    byte[] buf = new byte[1024];
	    int i = 0;
	    while((i=fis.read(buf))!=-1) {
	      fos.write(buf, 0, i);
	    }
	    fis.close();
	    fos.close();
	}
	
	public static String getFileToSave (String dialogTitle, JFrame parent) {
		File file = null;
		String fileName = null;
		JFileChooser saveTo = new JFileChooser();
		saveTo.setDialogTitle(dialogTitle);
		int fileQuery = saveTo.showSaveDialog(parent);
		if (fileQuery == JFileChooser.APPROVE_OPTION) {
            file = saveTo.getSelectedFile();
            fileName = file.getAbsolutePath();
            talkln (dialogTitle + ": " + fileName);
        } else {
            talkln ("Save cancelled by user");
            return null;
        }
        return fileName;
	}
	
	public void setUpFiles () {
		mkMessageFile(); 
		mkEnvironmentFile();
		mkLisPosFile();
		if(Env.reportRegionInfo){mkLisRegionFile(); mkLisForceFile();}
		mkNucInfoFile();
		mkSourceFolder();
	}
		
	public void mkMessageFile () {
		try {
			messagesOutFile = new File (Env.datFilePath + File.separator + Env.qkMovieFileName + "Messages");
			messagesOutFW = new FileWriter(messagesOutFile);
			messagesOutPW = new PrintWriter(messagesOutFW, true);
			Date timeNow = new Date();
			messagesOutPW.println("This is the message file for " + Env.qkMovieFileName + " run on " + timeNow.toString());
		} catch (IOException ioe) { talkln ("An error creating messagesOut stuff"); }
	}
	
	public void mkEnvironmentFile () {
		File envFile = new File ("Env.java");
		if ( ! envFile.exists() || ! envFile.canRead() ) {
			talkln ("Can't find 'Env.java'... you'd better store those values manually");
		} else {
			try {
				FileReader fileR = new FileReader(envFile);
				BufferedReader in = new BufferedReader(fileR);
				File envOut = new File(Env.datFilePath + File.separator + Env.qkMovieFileName + "Environment");
				FileWriter envOutFW = new FileWriter(envOut);
				PrintWriter envOutPW = new PrintWriter(envOutFW, true);
				Date timeNow = new Date();
				envOutPW.println("This is the environment file for " + Env.qkMovieFileName + " run on " + timeNow.toString());
				String nextLine = in.readLine();
				while (nextLine != null) {
					envOutPW.println(nextLine);
					nextLine = in.readLine();
				}
			} catch (IOException ioe) { talkln ("An error trying to save environment file"); }
		}
	}
	
	public void mkSourceFolder () {
		try {
			File curDir = new File (".");
			System.out.println ("Directory for source files is: " + curDir.getCanonicalPath());
			String [] srcFiles = curDir.list(new SourceJarFileFilter());
			for (int i=0;i<srcFiles.length;i++) {
				File curSrcFile = new File (srcFiles[i]);
				File curOutSrcFile = new File (Env.srcFilePath + File.separator + curSrcFile.getName());
				copyFile(curSrcFile,curOutSrcFile);
			}
		} catch (Exception e) { talkln ("An error trying to copy source files"); }
	}
	
	public void mkLisPosFile () {
		try {
			listeriaPositionFile = new File (Env.datFilePath + File.separator + Env.qkMovieFileName + "ListeriaInfo");
			listeriaPositionFW = new FileWriter(listeriaPositionFile);
			lisPosPW = new PrintWriter(listeriaPositionFW,true);
			Date timeNow = new Date();
			String headString = "This is the Listeria position file for " + Env.qkMovieFileName + " run on " + timeNow.toString() + " - Env.numActAs = " + Env.numberActAs;
			if (Env.useNewToonDist) { headString = headString + " , Env.rdmAmp = " + Env.rdmAmp; }
			lisPosPW.println(headString);
			lisPosPW.print("time;");
			lisPosPW.print("path.x;path.y;path.z;");
			lisPosPW.print("pathAng.x;pathAng.y;pathAng.z;");
			//lisPosPW.print("actinEflex;actinE1;actinE2;actinE3;");
			//lisPosPW.print("actinEflex-Cos;actinE1-Cos;actinE2-Cos;actinE3-Cos;");
			lisPosPW.print("newTether;cutTether;#Tethers;#strainedTethersByHalf;#strainedTethersMoreThanHalf;");
			lisPosPW.print("#totalStrained;");
			lisPosPW.print("newArp;");
			lisPosPW.print("forceInx-noThermal;forceInx;colForceInx;linkForceInx;frictionForceInx;");
			lisPosPW.print("tipsInCylSection;bTransGam.x;");
			//lisPosPW.print("end1.x;end1.y;end1.z;end2.x;end2.y.end2.z;");
			//lisPosPW.print("coord.x;coord.y;coord.x);
			//lisPosPW.print("angle.x;angle.y;angle.z;");
			//lisPosPW.print("thmKick.x;thmKick.y;thmKick.z;");
			//lisPosPW.print("thmKick/3.x;thmKick/3.y;thmKick/3.z;");
			//lisPosPW.print("thmKick/6.x;thmKick/6.y;thmKick/6.z;");
			//lisPosPW.print("thmTwist.x;thmTwist.y;thmTwist.z;");
			//lisPosPW.print("thmTwist/3.x;thmTwist/3.y;thmTwist/3.z;");
			//lisPosPW.print("thmTwist/6.x;thmTwist/6.y;thmTwist/6.z;");
			//lisPosPW.print("#actinMonomersRegion;#Arps;#ArpBranches;ArpBranchActinRatio;");
			lisPosPW.print("closeTips(ave);");
			lisPosPW.print("fricForceInx(ave);");
			lisPosPW.print("normForce(ave);");
			lisPosPW.print("tipsOnLisSphere(ave);");
			lisPosPW.println("collisionForceInx(ave);");
		} catch (IOException ioe) { talkln ("An error creating messagesOut stuff"); }
	}
	
	public void mkNucInfoFile () {
		try {
			nucInfoFile = new File (Env.datFilePath + File.separator + Env.qkMovieFileName + "NucInfo");
			nucInfoFW = new FileWriter(nucInfoFile);
			nucInfoPW = new PrintWriter(nucInfoFW,true);
			Date timeNow = new Date();
			nucInfoPW.println("This is the nucleation info file for " + Env.qkMovieFileName + " run on " + timeNow.toString());
			nucInfoPW.println("time;newBranch.uVecX;newBranch.uVecY;newBranch.uVecZ");
		} catch (IOException ioe) { talkln ("An error creating messagesOut stuff"); }
	}
	
	public static void mkHistogramFile (String fileName, double[] intervalVals, int [] ctVals) {
		try { 
			File histoFile = new File (Env.datFilePath + File.separator + Env.qkMovieFileName + fileName);
			FileWriter histoFW = new FileWriter(histoFile);
			PrintWriter histoPW = new PrintWriter(histoFW, true);
			histoPW.println("interval; count");
			for (int i=0; i<ctVals.length; i++) {
				histoPW.println(String.valueOf(intervalVals[i]) + ";" + String.valueOf(ctVals[i]));
			}
		} catch (IOException ioe) { talkln ("Error creating a histogram file"); }
	}
	

	public void saveListeriaPositionalInfo (Listeria listeria) {
		printValue(Env.simulationTime,lisPosPW);
		printPt3D(listeria.pathCoord, lisPosPW);
		printPt3D(listeria.pathAng, lisPosPW);
		printValue(Env.newTetherCt,lisPosPW);
		printValue(Env.cutTetherCt,lisPosPW);
		printValue(listeria.lastLinkerCt,lisPosPW);
		printValue(Linker.countStrainedTethers1(),lisPosPW);
		printValue(Linker.countStrainedTethers2(),lisPosPW);
		printValue((Linker.countStrainedTethers1() + Linker.countStrainedTethers2()),lisPosPW);
		printValue(Env.newArpCt,lisPosPW);
		printValue(listeria.bForceSumNoThermal.x,lisPosPW);
		printValue(listeria.bForceSum.x,lisPosPW);
		printValue(listeria.colForceSumInx.x,lisPosPW);
		printValue(listeria.linkForceSumInx.x,lisPosPW);
		printValue(listeria.frictionForceMag,lisPosPW);
		printValue(listeria.tipsInCylSection,lisPosPW);
		printValue(listeria.bTransGam.x,lisPosPW);
		//printPt3D(listeria.end1, lisPosPW);
		//printPt3D(listeria.end2, lisPosPW);
		//printPt3D(listeria.coord, lisPosPW);
		//printPt3D(listeria.angle, lisPosPW);
		//A_Rod.countArpAndActMonsInRegion();
		//printValue(Env.monCt,lisPosPW);
		//printValue(Env.arpCt,lisPosPW);
		//printValue(Env.arpBranchCt,lisPosPW);
		//printValue(Env.arpBranchActMonRatio,lisPosPW);
		printValue((double)Env.closeTipCt/(double)Env.lisInfoFreq,lisPosPW);
		printValue((double)Env.aveFricForceInx/(double)Env.lisInfoFreq,lisPosPW);
		printValue((double)Env.aveNormForce/(double)Env.lisInfoFreq,lisPosPW);
		printValue((double)Env.tipsOnLisSphere/(double)Env.lisInfoFreq,lisPosPW);
		printValue((double)Env.aveColForceInx/(double)Env.lisInfoFreq,lisPosPW);
		lisPosPW.println("");
	}
	
	public static void saveNucInfo (Pt3D nucPos) {
		nucInfoPW.print(String.valueOf(Env.simulationTime));
		printDivider(nucInfoPW);
		printPt3D(nucPos, nucInfoPW);
		nucInfoPW.println("");
	}
	
	public void mkLisRegionFile () {
		try {
			listeriaRegionFile = new File (Env.datFilePath + File.separator + Env.qkMovieFileName + "ListeriaRegInfo");
			listeriaRegionFW = new FileWriter(listeriaRegionFile);
			lisRegPW = new PrintWriter(listeriaRegionFW,true);
			Date timeNow = new Date();
			lisRegPW.println("This is the Listeria region file for " + Env.qkMovieFileName + " run on " + timeNow.toString());
			lisRegPW.print("time;");
			lisRegPW.print("path.x;path.y;path.z;");
			lisRegPW.print("forceInxInA;forceInxInB;forceInxInC;forceInxInD;forceInxInE;");
			//lisRegPW.print("colForceInxInA;colForceInxInB;colForceInxInC;colForceInxInD;colForceInxInE;");
			//lisRegPW.print("linkForceInxInA;linkForceInxInB;linkForceInxInC;linkForceInxInD;linkForceInxInE;");
			lisRegPW.print("#TethersInA;#TethersInB;#TethersInC;#TethersInD;#TethersInE;");
			lisRegPW.print("#strainedTethersInA;#strainedTethersInB;#strainedTethersInC;#strainedTethersInD;#strainedTethersInE;");
			lisRegPW.print("#newTethersInA;#newTethersInB;#newTethersInC;#newTethersInD;#newTethersInE;");
			lisRegPW.print("#cutTethersInA;#cutTethersInB;#cutTethersInC;#cutTethersInD;#cutTethersInE;");
			lisRegPW.print("#actinInA;#actinInB;#actinInC;#actinInD;#actinInE;");
			lisRegPW.print("ArpActinRatioInA;ArpActinRatioInB;ArpActinRatioInC;ArpActinRatioInD;ArpActinRatioInE;");
			lisRegPW.println("");		//put this one down here so it can be a println (make a new paragraph)
		} catch (IOException ioe) { talkln ("An error creating messagesOut stuff"); }
	}
	
	public void saveListeriaRegionalInfo (Listeria listeria) {
		printValue(Env.simulationTime,lisRegPW);
		printPt3D(listeria.pathCoord, lisRegPW);
		printValue(listeria.forceSumInA.x,lisRegPW);
		printValue(listeria.forceSumInB.x,lisRegPW);
		printValue(listeria.forceSumInC.x,lisRegPW);
		printValue(listeria.forceSumInD.x,lisRegPW);
		printValue(listeria.forceSumInE.x,lisRegPW);
		printValue(listeria.lastLinkerCtInA,lisRegPW);
		printValue(listeria.lastLinkerCtInB,lisRegPW);
		printValue(listeria.lastLinkerCtInC,lisRegPW);
		printValue(listeria.lastLinkerCtInD,lisRegPW);
		printValue(listeria.lastLinkerCtInE,lisRegPW);
		ActA.countStrainedTethersInRegions();
		printValue(Env.strainedTetherCtInA,lisRegPW);
		printValue(Env.strainedTetherCtInB,lisRegPW);
		printValue(Env.strainedTetherCtInC,lisRegPW);
		printValue(Env.strainedTetherCtInD,lisRegPW);
		printValue(Env.strainedTetherCtInE,lisRegPW);
		printValue(Env.newTetherCtInA,lisRegPW);
		printValue(Env.newTetherCtInB,lisRegPW);
		printValue(Env.newTetherCtInC,lisRegPW);
		printValue(Env.newTetherCtInD,lisRegPW);
		printValue(Env.newTetherCtInE,lisRegPW);
		printValue(Env.cutTetherCtInA,lisRegPW);
		printValue(Env.cutTetherCtInB,lisRegPW);
		printValue(Env.cutTetherCtInC,lisRegPW);
		printValue(Env.cutTetherCtInD,lisRegPW);
		printValue(Env.cutTetherCtInE,lisRegPW);
		A_Rod.countRegionArpActin();
		printValue(Env.monCtInA,lisRegPW);
		printValue(Env.monCtInB,lisRegPW);
		printValue(Env.monCtInC,lisRegPW);
		printValue(Env.monCtInD,lisRegPW);
		printValue(Env.monCtInE,lisRegPW);
		printValue(Env.arpBranchRatioInA,lisRegPW);
		printValue(Env.arpBranchRatioInB,lisRegPW);
		printValue(Env.arpBranchRatioInC,lisRegPW);
		printValue(Env.arpBranchRatioInD,lisRegPW);
		printValue(Env.arpBranchRatioInE,lisRegPW);
		lisRegPW.println("");
	}
	
	public void mkLisForceFile () {
		try {
			listeriaForceFile = new File (Env.datFilePath + File.separator + Env.qkMovieFileName + "ListeriaForceInfo");
			listeriaForceFW = new FileWriter(listeriaForceFile);
			lisForcePW = new PrintWriter(listeriaForceFW,true);
			Date timeNow = new Date();
			lisForcePW.println("This is the Listeria region file for " + Env.qkMovieFileName + " run on " + timeNow.toString());
			lisForcePW.print("time;");
			lisForcePW.print("path.x;path.y;path.z;");
			lisForcePW.print("forceInxInA;forceInxInB;forceInxInC;forceInxInD;forceInxInE;");
			lisForcePW.print("colForceInxInA;colForceInxInB;colForceInxInC;colForceInxInD;colForceInxInE;");
			lisForcePW.print("linkForceInxInA;linkForceInxInB;linkForceInxInC;linkForceInxInD;linkForceInxInE;");
			lisForcePW.println("");		//put this one down here so it can be a println (make a new paragraph)
		} catch (IOException ioe) { talkln ("An error creating mkLisForceFile stuff"); }
	}
	
	public void saveListeriaForceInfo (Listeria listeria) {
		printValue(Env.simulationTime,lisForcePW);
		printPt3D(listeria.pathCoord, lisForcePW);
		printValue(listeria.forceSumInA.x,lisForcePW);
		printValue(listeria.forceSumInB.x,lisForcePW);
		printValue(listeria.forceSumInC.x,lisForcePW);
		printValue(listeria.forceSumInD.x,lisForcePW);
		printValue(listeria.forceSumInE.x,lisForcePW);
		printValue(listeria.colForceSumInA.x,lisForcePW);
		printValue(listeria.colForceSumInB.x,lisForcePW);
		printValue(listeria.colForceSumInC.x,lisForcePW);
		printValue(listeria.colForceSumInD.x,lisForcePW);
		printValue(listeria.colForceSumInE.x,lisForcePW);
		printValue(listeria.linkForceSumInA.x,lisForcePW);
		printValue(listeria.linkForceSumInB.x,lisForcePW);
		printValue(listeria.linkForceSumInC.x,lisForcePW);
		printValue(listeria.linkForceSumInD.x,lisForcePW);
		printValue(listeria.linkForceSumInE.x,lisForcePW);
		lisForcePW.println("");
	}

	
	private static void printDivider (PrintWriter pw) {
		pw.print(sepString);
	}
	
	private static void printValue (int val, PrintWriter pw) {
		pw.print(String.valueOf(val));
		printDivider(pw);
	}
	
	private static void printValue (double val, PrintWriter pw) {
		pw.print(String.valueOf(val));
		printDivider(pw);
	}
	
	private static void printPt3D (Pt3D pt, PrintWriter pw) {
		pw.print(String.valueOf(pt.x));
		printDivider(pw);
		pw.print(String.valueOf(pt.y));
		printDivider(pw);
		pw.print(String.valueOf(pt.z));
		printDivider(pw);
	}
	
	public void takeRawPicture (String fileName) {
		FileOutputStream fos;
		DataOutputStream ds;
		byte alpha,red,green,blue;
		RocketBugs.theFrame.showAll();
		int imWidth = ZoomPanel.winSize;
		int imHeight = ZoomPanel.winSize;
		int [] pxls = new int [imWidth*imHeight];
		byte [] rowOfBytes = new byte [imWidth*4];
		PixelGrabber pg = new PixelGrabber(ZoomPanel.theImage,0,0,imWidth,imHeight,pxls,0,imWidth);
		 try {
            		pg.grabPixels();
      		  } catch (InterruptedException e) {
          		talkln ("interrupted waiting for pixels!");
           		 return;
        	}
        	if ((pg.getStatus() & ImageObserver.ABORT) != 0) {
          	 	talkln ("image fetch aborted or errored");
            		return;
       	 	}
  
		try {
			fos = new FileOutputStream (fileName);
			ds = new DataOutputStream (fos);
			for (int j = 0; j < imHeight; j++) {
				for (int i = 0; i < imWidth; i++) {
					alpha =  (byte) ((pxls[j*imWidth+i] >> 24) & 0xff);
					red = (byte) ((pxls[j*imWidth+i] >> 16) & 0xff);
					green = (byte) ((pxls[j*imWidth+i] >> 8) & 0xff);
					blue =  (byte) ((pxls[j*imWidth+i]        ) & 0xff);
					rowOfBytes[4*i] = alpha;
					rowOfBytes[4*i+1] = red;
					rowOfBytes[4*i+2] = green;
					rowOfBytes[4*i+3] = blue;
				}
				ds.write(rowOfBytes,0,4*imWidth);
			}
		} catch (IOException ioe) { talkln ("some sort of error writing binary file"); }
	}
	
	public static void takeRawPicture (Image theImage, String fileName) {
		FileOutputStream fos;
		DataOutputStream ds;
		byte alpha,red,green,blue;
		int imWidth = theImage.getWidth(null);
		int imHeight = theImage.getHeight(null);
		int [] pxls = new int [imWidth*imHeight];
		byte [] rowOfBytes = new byte [imWidth*4];
		PixelGrabber pg = new PixelGrabber(theImage,0,0,imWidth,imHeight,pxls,0,imWidth);
		try {
            		pg.grabPixels();
      		} catch (InterruptedException e) {
          		talkln ("interrupted waiting for pixels!");
           		return;
        	}
        	if ((pg.getStatus() & ImageObserver.ABORT) != 0) {
          	 	talkln ("image fetch aborted or errored");
            		return;
       	 	}
  
		try {
			fos = new FileOutputStream (fileName);
			ds = new DataOutputStream (fos);
			for (int j = 0; j < imHeight; j++) {
				for (int i = 0; i < imWidth; i++) {
					alpha =  (byte) ((pxls[j*imWidth+i] >> 24) & 0xff);
					red = (byte) ((pxls[j*imWidth+i] >> 16) & 0xff);
					green = (byte) ((pxls[j*imWidth+i] >> 8) & 0xff);
					blue =  (byte) ((pxls[j*imWidth+i]        ) & 0xff);
					rowOfBytes[4*i] = alpha;
					rowOfBytes[4*i+1] = red;
					rowOfBytes[4*i+2] = green;
					rowOfBytes[4*i+3] = blue;
				}
				ds.write(rowOfBytes,0,4*imWidth);
			}
		} catch (IOException ioe) { talkln ("some sort of error writing binary file from frame"); }
	}
	
	public void takeQuickPicture (String fileName) { 
		Player [] P = Player.thePlayers;
		Player curP;
		A_Rod rod;
		A_Node node;
		A_Blob blob;
		Listeria listeria;
		int noPs = Player.playerCt;
		try {
			FileOutputStream fos = new FileOutputStream (fileName);
			DataOutputStream ds = new DataOutputStream (fos);
			ds.writeInt(-1); // signal that we're about to give deltaT
			ds.writeFloat((float)Env.deltaT);
			ds.writeInt(noPs);
			for (int i = 0; i<noPs; i++) {
				curP = P[i];
				if (curP instanceof A_Rod) {
					rod = (A_Rod) curP;
					ds.writeByte(1);	// identify as a rod
					if (rod.plusCapped) { ds.writeByte(1); } else { ds.writeByte(0); }
					if (rod.minusCapped) { ds.writeByte(1); } else { ds.writeByte(0); }
					Pt3D.writePt3D(ds,rod.end1);
					Pt3D.writePt3D(ds,rod.end2);
					Pt3D.writePt3D(ds,rod.angle);
					ds.writeInt(rod.numOfMonomers);
					ds.writeInt(rod.numADPPiMonomers);
					ds.writeInt(rod.numADPMonomers);
					ds.writeFloat((float) rod.helixAng);
				} else {
					if (curP instanceof A_Node) {
						node = (A_Node) curP;
						ds.writeByte(2);	// identify as a node
						Pt3D.writePt3D(ds,node.coord);
						for (int j = 0; j < 6; j++) { ds.writeInt(node.linkers[j].idnum); }
					} else {
						if (curP instanceof A_Blob) {
							blob = (A_Blob) curP;
							ds.writeByte(3); // identify as a blob
							Pt3D.writePt3D(ds,blob.coord);
							ds.writeFloat((float) blob.radius);
						} else {
							if (curP instanceof Listeria) {
								listeria = (Listeria) curP;	
								ds.writeByte(4);	// identify as a listeria
								Pt3D.writePt3D(ds,listeria.rec1);
								Pt3D.writePt3D(ds,listeria.rec2);
								Pt3D.writePt3D(ds,listeria.angle);
							} else {
								ds.writeByte(255);
							}
						}
					}
				}
			} 
		} catch (IOException ioe) { talkln ("some sort of error writing binary file"); }
	}
	
	public void makeMovieFrame (String fileName, String fileType) {
		if (fileType.equals("raw")) {
			talk ("Writing Raw Movie Frame  " + fileName + "..... ");
			takeRawPicture(fileName + "." + fileType);
			talkln ("done");
		}
		if (fileType.equals("qk")) {
			talk ("Writing Quick Movie Frame  " + fileName + ".... ");
			takeQuickPicture (fileName + "." + fileType);
			talkln ("done");
		}
	}
	
	public static void mkHistogramFile (HistogramPlus hist, String fileName) {
		try { 
			if (fileName == null ) { return; }
			File histoFile = new File (fileName);
			FileWriter histoFW = new FileWriter(histoFile);
			PrintWriter histoPW = new PrintWriter(histoFW, true);
			// column headings
			hist.printIntervalTitle(histoPW,sepString); 
			hist.printBinTitles(histoPW,sepString);
			histoPW.println("");
			// data
			for (int i=0;i<hist.binCt;i++) {
				hist.printInterval(histoPW,sepString,i);
				hist.printBinSum(histoPW,sepString,i); 
				if (i<hist.infoS.length) { histoPW.print(hist.infoS[i]); }
				histoPW.println("");
			}
			talk ("wrote histogram file with " + String.valueOf(hist.ptCt) + " pts");
			if (hist.reportAve) { talk (" and " + String.valueOf(hist.passCt) + " passes"); }
			talkln ("");
			
		} catch (IOException ioe) { talkln ("Error creating a histogram file"); }
	}
	
	public void saveConfig () {
		// this version is called from GUI... we therefore need to stop the integration-step
		MC.waitOnTimeLoop();
		saveConfig ("testfile.data");
		MC.notifyTimeLoop();
	}
	
	public void saveConfig (String fileName) {
		// this version is called directly from within run loop... no need to suspend anything
		// perform maintenance before save
		for (int i=0; i < Player.playerCt; i++) {
			if (Player.thePlayers[i].idnum != i) { talkln ("Mismatch in id for " + String.valueOf(Player.thePlayers[i])); }
		}
		try {
			FileOutputStream fos = new FileOutputStream (fileName);
			DataOutputStream ds = new DataOutputStream (fos);
			// first write the number of players
			ds.writeInt(Player.playerCt);
			talk ("Saving config with " + String.valueOf(Player.playerCt) + "....");
			// for each player send to saveYourself
			for (int i = 0; i < Player.playerCt; i++) {
				Player.thePlayers[i].saveYourself(ds);
			}
		} catch (Exception e) {System.out.println (e); }
		talkln ("done with save");
	}
	
	public void loadConfig (boolean preRun) {
		if (! preRun) { MC.waitOnTimeLoop(); }
		double simTimeIn = Env.simulationTime;	// hold onto simulationTime value
		MC.clearAll();
		if (preRun) { Env.simulationTime = simTimeIn; } 	// set simulationTime to value in IF this is a prerun load
		Player nextP;
		byte pType;
		boolean readOne=false;
		try {
			FileInputStream fis = new FileInputStream (Env.icFile);
			DataInputStream ds = new DataInputStream (fis);
			// set Env variables to default
			Env.listeriaPresent = false;
			// first read the number of players
			int numPlayers = ds.readInt();
			talkln ("Loading config with " + String.valueOf(numPlayers) + "....");
			// then ask each player to read itself and add to player array
			Player.playerCt = 0;	// reset playerCt
			for (int i = 0; i < numPlayers; i++) {
				pType = ds.readByte();
				//talk ("pType = " + String.valueOf(pType));
				if (pType == 1) { 
					//talk (" reading rod...");
					nextP = A_Rod.readYourself(ds, MC); 
					//talkln ("  done.");
				}
				if (pType == 2) { nextP = A_Blob.readYourself(ds, MC); }
				if (pType == 3) { nextP = A_Node.readYourself(ds, MC); }
				if (pType == 4) { 
					//talk(" reading Listeria...");
					Env.listeriaPresent = true;
					nextP = Listeria.readYourself(ds, MC); 
					Player.listeria = (Listeria)nextP;
					//talkln("done.");
				}
				if (pType == 5) { 
					//talk (" reading actin branch...");
					nextP = ActinBranch.readYourself(ds, MC); 
					/*SuperStructure nextSup = (SuperStructure)nextP;
					SuperStructure.structures[nextSup.strucId] = nextSup;
					SuperStructure.structureCt++;*/
					//talkln ("done");
				}
				if (pType == 6) { 
					nextP = ActinMesh.readYourself(ds, MC); 
					/*SuperStructure nextSup = (SuperStructure)nextP;
					SuperStructure.structures[nextSup.strucId] = nextSup;
					SuperStructure.structureCt++;*/
				}
				// Player.thePlayers[nextP.idnum] = nextP; Don't need this anymore... each Player adds itself
			}
			talkln ("	finished reading players...");
			MC.sphImp.resetGrid();	// reset sphImp grid
		} catch (Exception e) {System.out.println (e); }
		// initialize object links, etc after reading entire array of players.
		talkln  ("	initializing after read...");
		for (int i=0; i < Player.playerCt; i++) {
			Player.thePlayers[i].initializeAfterRead();
		}
		if ((! Env.remote) & (MC.theFrame != null)) { MC.theFrame.updateDisplay(Env.simulationTime, Player.playerCt); }
		talkln ("done loading config");
		MC.notifyTimeLoop();
	}
	
	public static void talkln (String info) {
		System.out.println(info);
		if (Env.qkMovie) { messagesOutPW.println(info); }
	}
	
	public static void talk (String info) {
		System.out.print(info);
		if (Env.qkMovie) { messagesOutPW.print(info); }
	}
}
		