package rocketBugs;
/*
	ForceResolver.... the class that takes collisions and resolves them into appropriate forces
*/

/*  <RocketBugs - for the simulation of actin-based Listeria motility>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.Math;

public class ForceResolver {
	double membraneSpring = 10;
	static final double rotLimit = 5*Math.PI/180;
	
	public ForceResolver () {
	}
	
	public void newBlobBlobCollision (A_Blob blob1, A_Blob blob2, double radiiSum,double blobDist) {
		double forceMag = 1.0e-6*(blob1.gamBlob*blob2.radius)*(1.0-blobDist/radiiSum)/Env.deltaT;  // calculation for required force
		Pt3D conRay = Pt3D.Sub(blob1.coord, blob2.coord);
		Pt3D unitVec = Pt3D.Scale(1/blobDist, conRay);
		Pt3D forces = Pt3D.Scale(forceMag, unitVec);
		Collision.newCollision (blob1, blob2, forceMag, forces, Player.zeroVec);
	}
	
	public void newBlobRodCollision (A_Blob blob, A_Rod rod, Pt3D conPt, double blobToCon,int contactloc) {
		Pt3D conRay;
		double forceMag;
		if (contactloc == 0) {  // contactloc tells us what kind of contact  0-> middle, 1-> end1, 2 ->end2
			double Rihat2 = Pt3D.ptDist(conPt,rod.coord);	// distance from contact point to rod centroid
			forceMag = 1.0e-6*(blob.radius+rod.radius-blobToCon)/((1/rod.bTransGam.y + Rihat2/rod.bRotGam.y + 1/blob.gamBlob)*Env.deltaT);
			conRay = Pt3D.Sub(rod.coord, blob.coord);
		} else {
			if (contactloc == 1) {
				forceMag = 1.0e-6*(blob.radius-blobToCon)/((1/blob.gamBlob + 1/rod.bTransGam.x)*Env.deltaT);
				conRay = Pt3D.Sub(rod.end1, blob.coord);
			} else {	// contactloc = 2
				forceMag = 1.0e-6*(blob.radius-blobToCon)/((1/blob.gamBlob + 1/rod.bTransGam.x)*Env.deltaT);
				conRay = Pt3D.Sub(rod.end2, blob.coord);
			}
		}
		Pt3D unitVec = Pt3D.Scale(1/blobToCon,conRay);
		Pt3D forces = Pt3D.Scale(forceMag, unitVec);
		Collision.newCollision (rod, blob, forceMag, forces, conPt);
	}
	
	public void newBlobNodeCollision (A_Blob blob, A_Node node, Pt3D conPt, double blobToCon) {
		/*double nodeToConDist, nodeDist, fracDist;
		nodeDist = node.curLength;
		nodeToConDist = Math.sqrt(Math.pow(node.xpos-xcon,2)+Math.pow(node.ypos-ycon,2));
		fracDist = nodeToConDist/nodeDist;
		forceMag = 1.0e-6*(blob.radius-blobToCon)*node.gamOrthog*blob.gamBlob/(Env.deltaT*(node.gamOrthog+blob.gamBlob));
		forcesX = forceMag*(xcon-blob.xpos)/blobToCon;
		forcesY = forceMag*(ycon-blob.ypos)/blobToCon;
		blob.newCollision(xcon,ycon,-forcesX,-forcesY);
		node.linker2.newCollision(xcon,ycon,fracDist*forcesX,fracDist*forcesY);
		node.newCollision(xcon,ycon,(1-fracDist)*forcesX,(1-fracDist)*forcesY); */
	}
	
	public void newBlobListeriaCollision (A_Blob blob, Listeria LM, Pt3D conPt, double conDist, int conType) {
		Pt3D unitVec, blobCon, LMCon;
		double forceMag;
		if (conType == 0) {
			unitVec = Pt3D.Sub(1/conDist, blob.coord, conPt);
			LMCon = Pt3D.Add(conPt, LM.cylDiam/2, unitVec);
			blobCon = Pt3D.Add(blob.coord, -blob.radius, unitVec);
			forceMag = 1.0e-6*(blob.radius + LM.cylDiam/2 - conDist)/((1/blob.gamBlob + 1/LM.bTransGam.y)*Env.deltaT);
		} else {
			if (conType == 1) {
				unitVec = Pt3D.Sub(1/conDist,blob.coord,LM.rec1);
				LMCon = Pt3D.Add(LM.rec1, LM.capRad, unitVec);
				blobCon = Pt3D.Add(blob.coord, -blob.radius, unitVec);
				forceMag = 1.0e-6*(blob.radius + LM.capRad - conDist)/((1/blob.gamBlob + 1/LM.bTransGam.x)*Env.deltaT);
			} else {	// conType == 2
				unitVec = Pt3D.Sub(1/conDist, blob.coord,LM.rec2);
				LMCon = Pt3D.Add(LM.rec2, LM.capRad, unitVec);
				blobCon = Pt3D.Add(blob.coord, -blob.radius, unitVec);
				forceMag = 1.0e-6*(blob.radius + LM.capRad - conDist)/((1/blob.gamBlob + 1/LM.bTransGam.x)*Env.deltaT);
			}
		}
		Pt3D forces = Pt3D.Scale(forceMag, unitVec);
		Collision.newCollision (blob, LM, forceMag, forces, LMCon);
	}
	
	
	public static void newRodListeriaCollision (A_Rod rod, Listeria LM, Pt3D conPt1, Pt3D conPt2, double conDist, int conType) {
		//talkln ("rod-listeria collision");
		Pt3D uVec = new Pt3D();
		Pt3D rodCon = new Pt3D();
		Pt3D LMCon = new Pt3D();
		Pt3D uVecInLMFrame = new Pt3D();
		double frictionForce = 0;
		double impDist = 0;
		double forceMag = 0;
		switch (conType) {
			case 0:
				rodCon = conPt1;
				LMCon = conPt2;
				uVec.sub(1/conDist, rodCon, LMCon);
				impDist = LM.cylDiam/2 + rod.radius - conDist;
				break;
			case 10:
				//talk ("Collision of end1 with cylinder");
				uVec.unitVec(conDist,rod.end1,conPt1);
				uVecInLMFrame = Pt3D.XToNewx(LM,uVec);
				LMCon = rod.end1;
				rodCon = rod.end1;
				impDist = LM.cylDiam/2 - conDist;
				break;
			case 20:
				//talk ("Collision of end2 with cylinder");
				uVec.unitVec(conDist,rod.end2,conPt1);
				uVecInLMFrame = Pt3D.XToNewx(LM,uVec);
				LMCon = rod.end2;
				rodCon = rod.end2;
				impDist = LM.cylDiam/2 - conDist;
				break;
			case 11:
				//talk ("End collision with end 1");
				uVec.sub(1/conDist, rod.end1, LM.rec1);
				LMCon = rod.end1;
				rodCon = rod.end1;
				impDist = LM.capRad - conDist;
				break;
			case 12:
				//talk ("End collision with end 2");
				uVec.sub(1/conDist, rod.end1, LM.rec2);
				LMCon = rod.end1;
				rodCon = rod.end1;
				impDist = LM.capRad - conDist;
				break;
			case 21:
				//talk ("End collision with end 1");
				uVec.sub(1/conDist, rod.end2, LM.rec1);
				LMCon = rod.end2;
				rodCon = rod.end2;
				impDist = LM.capRad - conDist;
				break;
			case 22:
				//talk ("End collision with end 2");
				uVec.sub(1/conDist, rod.end2, LM.rec2);
				LMCon = rod.end2;
				rodCon = rod.end2;
				impDist = LM.capRad - conDist;
				break;
			case 1:
				//talk ("Side collision with end 1");
				uVec.sub(1/conDist, conPt1, LM.rec1);
				LMCon.add(LM.rec1, LM.capRad, uVec);
				rodCon.add(conPt1, -rod.radius, uVec);
				impDist = rod.radius + LM.capRad - conDist;
				break;
			case 2: 
				//talk ("Side collision with end 2"); 
				uVec.sub(1/conDist, conPt1, LM.rec2);
				LMCon.add(LM.rec2, LM.capRad, uVec);
				rodCon.add(conPt1, -rod.radius, uVec);
				impDist = rod.radius + LM.capRad - conDist;
				break;
		}
		
		// first figure out angle between rod and uVec... use to guess whether rotation or translation predominant
		double angTween = Math.abs(Math.acos(Pt3D.Dot(uVec,rod.uVec)));
		double r = Pt3D.ptDist(rod.coord, rodCon);
		if ((angTween > rotLimit) & (r > 3*Env.lengthPerActin)) {	// hack.. angle large then assume all rotation
			forceMag = 1.0e-6*impDist*rod.bRotGam.y/(r*r*Env.deltaT);
		} else {
			forceMag = 1.0e-6*impDist/((1/rod.bTransGam.x + 1/LM.bTransGam.x)*Env.deltaT);
		}

		forceMag = Env.collisionAtten*forceMag;
		Pt3D forces = Pt3D.Scale(forceMag, uVec);
		
		// add friction forces
		if (Env.useBugFriction) {
			if (conType == 10 | conType == 20) {
				new FrictionEvent(rod,LM,rodCon,forceMag);	// add friction force to list, direction and scaling determined in Listeria.setFriction()
			}
		}
		
		Collision.newCollision (rod, LM, forceMag, forces, rodCon);
	} 
	
	public void newRodRodCollision (A_Rod rod1, A_Rod rod2, Pt3D conPt, Pt3D conUnitVec, double impDist) {
		// first find appropriate drag values
		double rod1Drag = rod1.getEffectiveDrag(conUnitVec);
		double rod2Drag = rod2.getEffectiveDrag(Pt3D.Scale(-1,conUnitVec));
		//double rod1Drag = rod1.getEffectiveDrag();
		//double rod2Drag = rod2.getEffectiveDrag();
		// then figure the appropriate force to move them apart
		double forceMag = 1.0e-6*(impDist)/((1/rod1Drag + 1/rod2Drag)*Env.deltaT);
		Pt3D forces = Pt3D.Scale(forceMag, conUnitVec);
		Collision.newCollision (rod2, rod1, forceMag, forces, conPt);
	}
	
	public void newRodNodeCollision (A_Rod rod, A_Node node, double xcon, double ycon, double nodeToConDist, int contactType) {
		/*if (contactType == 1) {
			double impingeDist,end1Dist,end2Dist,fracDist;
			double C1,C2,S1,S2;
			end1Dist = Math.sqrt(Math.pow(xcon-rod.xend1,2)+Math.pow(ycon-rod.yend1,2));
			end2Dist = Math.sqrt(Math.pow(xcon-rod.xend2,2)+Math.pow(ycon-rod.yend2,2));
			impingeDist = Math.min(end1Dist,end2Dist);
			if (impingeDist > .05 ) { impingeDist = .05; }
			forceMag = (1.0e-6)*impingeDist *(2*node.gamOrthog)/Env.deltaT;
			fracDist = nodeToConDist/node.curLength;
			if (fracDist > .5) {
				C1 = -(node.ypos-node.linker1.ypos)/node.linker1.curLength;		// cos(th+90) = -sin(th)
				S1 = (node.xpos-node.linker1.xpos)/node.linker1.curLength;			// sin(th+90) = cos(th)
				C2 = -(node.linker2.ypos-node.ypos)/node.curLength;				// cos(th+90) = -sin(th)
				S2 = (node.linker2.xpos-node.xpos)/node.curLength;				// sin(th+90) = cos(th)
				//forcesX = forceMag*(C2 + (1-2*fracDist)*C1)/(2-2*fracDist); 	
				//forcesY = forceMag*(S2 + (1-2*fracDist)*S1)/(2-2*fracDist);
			} else {
				C1 = -(node.linker2.ypos-node.ypos)/node.curLength;
				S1 = (node.linker2.xpos-node.xpos)/node.curLength;
				C2 = -(node.linker2.linker2.ypos-node.linker2.ypos)/node.linker2.curLength;
				S2 = (node.linker2.linker2.xpos-node.linker2.xpos)/node.linker2.curLength;
				//forcesX = forceMag*(C1 + (2*fracDist-1)*C2)/(2*fracDist);
				//forcesY = forceMag*(S1 + (2*fracDist-1)*S2)/(2*fracDist);
			}
			forcesX = forceMag*(C2+C1)/2;
			forcesY = forceMag*(S2+S1)/2;
			rod.newCollision(xcon,ycon,forcesX,forcesY);
			node.linker2.newCollision(xcon,ycon,-fracDist*forcesX,-fracDist*forcesY);
			node.newCollision(xcon,ycon,-(1-fracDist)*forcesX,-(1-fracDist)*forcesY);
		} else {
			double impingeDist = A_Node.graceCircRad-nodeToConDist;
			forceMag = (1.0e-6)*impingeDist*node.gamOrthog/Env.deltaT;
			forcesX = forceMag*(xcon-node.xpos)/nodeToConDist;
			forcesY = forceMag*(ycon-node.ypos)/nodeToConDist;
			rod.newCollision(xcon,ycon,forcesX,forcesY);
			node.newCollision(xcon,ycon,-forcesX,-forcesY);
		} */		
	}
	
	
	public void newNodeNodeCollision (A_Node nodeball,A_Node nodeline,double xcon,double ycon,double impingedist) {
		/*forceMag = 1.0e-6*impingedist*nodeball.gamOrthog*nodeline.gamOrthog/(Env.deltaT*(nodeball.gamOrthog+nodeline.gamOrthog));
		forcesX = forceMag*(xcon-nodeball.xpos)/(A_Node.graceCircRad-impingedist);
		forcesY = forceMag*(ycon-nodeball.ypos)/(A_Node.graceCircRad-impingedist);
		nodeball.newCollision(0,0,-forcesX,-forcesY);
		double nodeToConDist = Math.sqrt(Math.pow(nodeline.xpos-xcon,2)+Math.pow(nodeline.ypos-ycon,2));
		double fracDist = nodeToConDist/nodeline.curLength;
		nodeline.newCollision(0,0,(1-fracDist)*forcesX,(1-fracDist)*forcesY);
		nodeline.linker2.newCollision(0,0,fracDist*forcesX,fracDist*forcesY);
		//endocytosis
		/*A_Node swapNode = nodeball.linker2;
		nodeball.linker2 = nodeline.linker2;
		nodeball.linker2.linker1 = nodeball;
		nodeline.linker2 = swapNode;
		nodeline.linker2.linker1 = nodeline; */
	}
	
	private void talkln (String info) {
		System.out.println(info);
	}
}