package rocketBugs;
/*	
	Listeria.... extends player and is listeria shaped... what else.  Expresses ACT A.
*/

/*  <RocketBugs - for the simulation of actin-based Listeria motility>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.Math;
import java.awt.*;
import java.io.*;
import java.util.*;


public class Listeria extends Player {
	static final double length = Env.listeriaLength;	// length of listeria in micrometers
	static final double cylDiam = Env.listeriaCylDiam;	//  diameter of cylindrical baterial section
	static final double capRad = Env.listeriaCapRad;	// radius of cap hemisphere.
	static final double capRadSqrd = Env.listeriaCapRad*Env.listeriaCapRad;	// for quick collision check
	static final double halfLength = Env.listeriaLength/2;	// for quick collision check
	static final double cylLength = length-2*capRad;	// length of cylinder section
	static final double halfCylLength = cylLength/2;	// half the cylinder length
	Pt3D end1 = new Pt3D();						// location of cap1 tip
	Pt3D end2 = new Pt3D(); 					// location of cap2 tip
	Pt3D rec1 = new Pt3D();						// location of cap1 center
	Pt3D rec2 = new Pt3D();						// location of cap2 center
	
	// friction force related
	Pt3D lastbVeloc = new Pt3D();				// store last velocity
	ValueTracker aveLastbVeloc = new ValueTracker (5, ValueTracker.PT3D_TYPE);
	double frictionForceMag = 0;				// sum up frictional component of force
	Pt3D frictionalTorq = new Pt3D();			// sum up frictional torques from colliding filaments
	Pt3D sansFric = new Pt3D();				// used to calculate direction and limit mag of friction
	
	Pt3D [] boxCorn = new Pt3D [4];				// store points that define box around outside Listeria
	Pt3D [] recCorn = new Pt3D [4];				// store points of box around cylindrical section of Listeria

	// some declarations related to painting the listeria
	int numCircs = 4;
	int numPts = 10;
	int numLines = 90;
	int numCapPts = 50;
	Pt3D [][] capPaintPts = new Pt3D [numCircs+1][numPts+1];	// store points in body axes for painting spherical caps
	Pt3D [] cylPaintPts = new Pt3D [numLines+1];				// ... for painting cylindrical section
	Pt3D [][] capPts = new Pt3D [numLines+1][numCapPts+1];
	
	// baseline drags and diffusivities
	Pt3D bTransGam_base = new Pt3D();
	Pt3D bRotGam_base = new Pt3D();
	Pt3D bTransDiff_base = new Pt3D();
	Pt3D bRotDiff_base = new Pt3D();
	
	static final double grabDistSqrd = Env.ActA_ArpGrabDist*Env.ActA_ArpGrabDist;	// grab distance sqrd
	
	// for reporting
	Pt3D bForceSumNoThermal = new Pt3D();
	Pt3D curThermalKick = new Pt3D();
	Pt3D curThermalTwist = new Pt3D();
	ValueTracker thermalKickBy3 = new ValueTracker(3,1);	// time average force over 3 integration steps
	ValueTracker thermalKickBy6 = new ValueTracker(6,1);	// over 6 steps
	ValueTracker thermalTwistBy3 = new ValueTracker(3,1);	// time average torque over 3 integration steps
	ValueTracker thermalTwistBy6 = new ValueTracker(6,1);	// over 6 steps
	Pt3D pathCoord = new Pt3D();
	Pt3D pathAng = new Pt3D();
	boolean haywire = false;
	
	// histogram profiles
	static HistogramPlus end2Locs = new HistogramPlus (20,-length,length,Env.datFilePath,"end2Locs",null);
	static HistogramPlus tethersFormed = new HistogramPlus (20,-length,length,Env.datFilePath,"tethersFormed",null);
	static HistogramPlus tethersBroken = new HistogramPlus (20,-length,length,Env.datFilePath,"tethersBroken",null);
	static HistogramPlus tethers = new HistogramPlus (20,-length,length,Env.datFilePath,"tethers",null);
	static HistogramPlus fActinAngles = new HistogramPlus (20,0,Math.PI,Env.datFilePath,"fActinAngles",null);
	static HistogramPlus pathForce = new HistogramPlus (20,-halfLength,halfLength,Env.datFilePath,"pathForce",null);
	static HistogramPlus radialForce = new HistogramPlus (20,-halfLength,halfLength,Env.datFilePath,"radialForce",null);
	
	
	
	// **parameters for ActA creation and management
	static HistogramPlus actAProbs, actALocs, counterHist;
	static final Object actALock = new Object();			// generic object for synchronizing ActA creation/destruction
	//temp for adjusting ActA#'s in env
	static double equilArpBoundActAs =  Env.numberActAs * Env.equilActAAdjust;	// equil value for # of active ActAs
	//static double equilArpBoundActAs = (Env.kArpFluxIn/Env.kBoundActAArp_Off) * Env.equilActAAdjust;	// equil value for # of active ActAs
	static int equilArpBoundActAsInt =  (int)(equilArpBoundActAs);	// equil value for # of active ActAs
	Pt3D locOnLis = new Pt3D(); 	// recycled Pt3D for ActA location and other places
	Pt3D forceOnLis = new Pt3D();	// recycled Pt3D used here and there
	double cylProb = Env.cylActAArea/(Env.capActAArea + Env.cylActAArea);	// probability that a new ActA will end up on cylinder

	// ** viscous beard of actin simulation
	static int tipsInCylSection = 0;		// for tracking number of filaments with a tip within Env.viscousShellThickness on cylindrical section
	
	
/*	// parameters for tethering of filaments to Listeria
	double tethercount = 0;					// just a counter to track number of filaments tethered
	Pt3D tetherForceVec, tetherTorqueVec;	 	// recycled for each ActA tether
	Pt3D tetherForceSum = new Pt3D();		// The sum of all tether forces
	Pt3D tetherTorqueSum = new Pt3D();		// The sum of all tether torques*/
	
	public Listeria (Pt3D coord, Pt3D initUVec, SphImpPlayers impP)  {
		super();
		Env.listeriaPresent = true;	// set flag stating that this run contains a listeria
		this.impP = impP;
		this.coord.copy(coord);
		this.uVec.copy(initUVec);
		// viscous resistance definitions... for prolate ellipsoid of revolution ... see Berg p. 57 and 84
		// Diffusivity should be (m^2/s)
		double a = 1.0e-6*length/2;
		double b = 1.0e-6*cylDiam/2;
		bTransGam.x = 4*Math.PI*Env.aeta*a/(Math.log(2*a/b) - .5);
		bTransGam.y = 8*Math.PI*Env.aeta*a/(Math.log(2*a/b) + .5);
		bTransGam.z = bTransGam.y;
		bRotGam.x = 16*Math.PI*Env.aeta*a*b*b/3;
		bRotGam.y = (8*Math.PI*Env.aeta*Math.pow(a,3)/3)/(Math.log(2*a/b) - .5);
		bRotGam.z = bRotGam.y;
		
		// *************************
		bTransDiff.div(Env.Boltz*Env.tempK, bTransGam);	// Einstein's relation D=kT/gamma
		bRotDiff.div(Env.Boltz*Env.tempK, bRotGam);
		
		bTransGam_base.copy(bTransGam);
		bRotGam_base.copy(bRotGam);
		bTransDiff_base.copy(bTransDiff);
		bRotDiff_base.copy(bRotDiff);
		
		//Player.setViscBlobDrags(bTransGam.y, bRotGam.y);	//need to think about which one of these x,y,z want to use or make a standard
		
		playerColor = Env.listeriaColor;
		// Use the equilibrium value for "active" (Arp2/3 bound) proteins... create that many initially	
		makeEquilArpBoundActAs (equilArpBoundActAsInt); 	// create ActA proteins with fixed location on surface of listeria
		//System.out.println("equilActAAdjust =" + String.valueOf(Env.equilActAAdjust));
		//System.out.println("equilArpBoundActAsInt =" + String.valueOf(equilArpBoundActAsInt));
		//createToupeeOfActA();
		// ***************
		makePaintPoints ();				// do some expensive calculation only once in life of listeria
		initialize ();
	}
	
	public void initialize () {
		// this method assumes the uVec has been set, or it unchanged
		// euler angles from uVec
		angle.eulerAnglesFromUVec(uVec);
		// calculate cos and sin of Euler angles only once during each time step
		cosAng.cosine(angle);
		sinAng.sine(angle);
		// find the transformation matrices at this time step
		transMat ();
		// find the transverse unit vectors that defines the orientation of the rod
		t1Vec.xToX(this, yUnitVector); 
		t2Vec.xToX(this, zUnitVector);
		// find the cap end points and cap hemisphere centers
		end1.add(coord, -length/2, uVec);
		end2.add(coord, length/2, uVec);
		rec1.add(coord, -(length/2 - capRad), uVec);
		rec2.add(coord, length/2 - capRad, uVec);
	}
	
	public void setForceEvents () {
		// store forces in direction of motion.. for reporting purposes only
		bForceSumNoThermal.copy(bForceSum);
		colForceSumInx.XTox(this,colForceSum);
		linkForceSumInx.XTox(this,linkForceSum);
	}
	
	public void setRegionForceEvents () {
		// sum region forces
		forceSumInA.add(colForceSumInA, linkForceSumInA);
		forceSumInB.add(colForceSumInB, linkForceSumInB);
		forceSumInC.add(colForceSumInC, linkForceSumInC);
		forceSumInD.add(colForceSumInD, linkForceSumInD);
		forceSumInE.add(colForceSumInE, linkForceSumInE);
		
		// store forces in direction of motion.. for reporting purposes only
		colForceSumInA.XTox(this);
		colForceSumInB.XTox(this);
		colForceSumInC.XTox(this);
		colForceSumInD.XTox(this);
		colForceSumInE.XTox(this);
		
		linkForceSumInA.XTox(this);
		linkForceSumInB.XTox(this);
		linkForceSumInC.XTox(this);
		linkForceSumInD.XTox(this);
		linkForceSumInE.XTox(this);
		
		forceSumInA.XTox(this);
		forceSumInB.XTox(this);
		forceSumInC.XTox(this);
		forceSumInD.XTox(this);
		forceSumInE.XTox(this);
	}
	
	public void setThermalEvents () {
		curThermalKick.copy(randForces);
		curThermalTwist.copy(randTorques);
		thermalKickBy3.registerValue(randForces);
		thermalKickBy6.registerValue(randForces);
		thermalTwistBy3.registerValue(randTorques);
		thermalTwistBy6.registerValue(randTorques);
	}
	
	public void collision (Pt3D forcePt, Pt3D forceVec) {
		colForceSum.inc(forceVec);
		colTorqueSum.inc(getTorque(forcePt,forceVec));
		colliderCt++;
		
		// all below is for registering forces in histograms
		forceOnLis.XTox(this, forceVec);
		locOnLis.sub(forcePt,coord);	// vector from Listeria.coord to forcePt
		locOnLis.XTox(this);			// convert to Listeria body-fixed coordinates
		double xPos = locOnLis.x;		// need to store 'cause zero out below
		
		pathForce.addValue(xPos,forceOnLis.x);
		Env.addColForceInx(forceOnLis.x);
		
		locOnLis.x = 0;		// now locOnLis is a vector that points radially out to forcePt, in Listeria frame
		double forceRadial = Pt3D.Dot(forceVec, locOnLis);
		radialForce.addValue(xPos,forceRadial);
	}
		
	public double setFriction () {
		// here we determine the ensuing bug motion without friction, to determine direction and potentially limit magnitude
		// scale if necessary
		sansFric.add(colForceSum, linkForceSum);
		sansFric.XTox(this, sansFric);
		double xForce = sansFric.x;
		if (Env.shakeTransListeria) { 
			xForce += randForces.x;	// sum collision, link and brownian components
		}
		double fricScale = 1;	// default, friction is less than propulsive force
		double fricRatio = Math.abs(xForce)/FrictionEvent.fricMag();
		if (fricRatio < 1) { fricScale = fricRatio; } // attenuate friction forces if sum is larger than force this time-step
		if (xForce > 0) {
			fricScale *= -1;
		} else if (xForce < 0) {
			fricScale *= 1;
		}
		
		return fricScale;
	}
	
	public void movePlayer () {
		sumForces();
		// explicit Euler approximation to ODE solution
		// first work in coordinates aligned with the rod....
		// transform forces and torques into body-fixed axis....
		bForceSum.XTox(this, forceSum);
		bTorqueSum.XTox(this, torqueSum);
		if ((Env.opposingForcePresent) & (Env.simulationTime > 5)) { bForceSum.x += Env.opposingForce; }	// add in force resisting motion
		setForceEvents();
		if(Env.reportRegionInfo){
			setRegionForceEvents();	//convert all region forces to Listeria.x
		}
		// add random forces, given in body-fixed frame, and calculate forces for next time step
		if (Env.shakeTransListeria) { 
			randForces.scale(Env.lisShakeAttenFactor);	// artificially scale Brownian forces
			bForceSum.inc(randForces); 
		}
		if (Env.shakeRotListeria) { 
			randTorques.scale(Env.lisShakeAttenFactor);	// artificially scale Brownian torques
			bTorqueSum.inc(randTorques); 
		}
		if ((Env.shakeTransListeria) | (Env.shakeRotListeria)) {
			setThermalEvents();
			calcRandomForces();
		}
		// now that the forces and torques are in the body fixed frame, we apply the eoms....
		bVeloc.div(1.0e6, bForceSum,bTransGam);		// in micron/sec
		bAngVeloc.div(bTorqueSum, bRotGam);		// in radians/sec
		
		// ....then transform back into fixed coordinate frame
		if ((bVeloc.checkPt3D()) & (bAngVeloc.checkPt3D())) {		// check for NaN before proceeding
			lastbVeloc.copy(bVeloc);
			aveLastbVeloc.registerValue(lastbVeloc);
			veloc.xToX(this, bVeloc);
			
			//to apply the body-fixed angular velocities, approximate new unit vector from arc of rotations
			double endTransInZ = -bAngVeloc.y * Env.deltaT;	// arclength out at 1 micron
			double endTransInY = bAngVeloc.z * Env.deltaT;
			uVec.setVals(1,endTransInY,endTransInZ);	// in body-fixed, not a unit vector yet
			uVec.xToX(this);	// make in fixed-frame, not a unit vector yet
			uVec.unitVec();		// make a unit vector
			
			// and finally move everything
			coord.inc(Env.deltaT, veloc);		// just position = velocity*time
			
			// body-fixed x angular velocity is same as euler angle phi rotation
			angle.z += bAngVeloc.x*Env.deltaT;
		
			// update path coordinate
			pathCoord.inc(Env.deltaT, bVeloc); // path coordinates are just path velocity * deltaT
			pathAng.inc(Env.deltaT,bAngVeloc); // body fixed angles
		} else {
			talkln ("*** Some force for the listeria has gone haywire!");
			reportStats();
			reportForces();
			talkln ("***********************************************");
			haywire = true;
		}
		
		if (!Pt3D.CheckPt3D(coord)) { talkln ("** ERROR **  Listeria position is " + coord.reportCoords()); }
		
		// convert center of mass position and uVec into end positions and Euler angles
		initialize ();
	
		// reset counters
		resetCounters();	// reset collision counters and force sums
	}
	
	public void resetCounters() {
		super.resetCounters();
		frictionForceMag = 0;
	}
	
	public void checkViscDragIncrease () {
		double dragFactor=1;
		int x = countTipsInCylSection();
		
		switch (Env.viscType) {
		case Env.LINEAR_VISC:
			dragFactor = Env.baseViscFactor + Env.lisViscDragSlope*x;
			break;
		case Env.SQUARE_VISC:
			dragFactor = Env.baseViscFactor + Env.sqViscCoeff*x*x;
			break;
		case Env.RATIONAL_VISC:
			// implement dragFactor of form C + D*A^N/(A^N + K^N)
			dragFactor = Env.baseViscFactor + Env.coeffViscFactor*Math.pow(x,Env.coopViscFactor)/(Math.pow(x,Env.coopViscFactor) + Math.pow(Env.halfMaxViscFactor,Env.coopViscFactor));
			break;
		}
		
		bTransGam.scale(dragFactor,bTransGam_base);
		bRotGam.scale(dragFactor,bRotGam_base);
		bTransDiff.div(Env.Boltz*Env.tempK, bTransGam);	// Einstein's relation D=kT/gamma
		bRotDiff.div(Env.Boltz*Env.tempK, bRotGam);
	}
	
	/*public void checkViscDragIncrease () {
		double dragFactor;
		countTipsInCylSection();
		int tipsToUse = tipsInCylSection;
		if (tipsToUse > 500) { tipsToUse = 500; }	// saturate abruptly for now
		dragFactor = 1 + Env.lisViscDragSlope*Math.pow(tipsToUse,2);
		
		bTransGam.scale(dragFactor,bTransGam_base);
		bRotGam.scale(dragFactor,bRotGam_base);
		bTransDiff.div(Env.Boltz*Env.tempK, bTransGam);	// Einstein's relation D=kT/gamma
		bRotDiff.div(Env.Boltz*Env.tempK, bRotGam);
	}*/
		
	public void checkRodTether (A_Rod rod, Pt3D tetherPt) {
		double bindSection = 1*Env.actATetherLength;
		double bindLoc = rod.length - Rdm.uRdm(this)*bindSection;	// random location chosen near tip
		// check if too close to the end of a filament or another ActA
		if (!rod.validActASite(bindLoc)) {
			Env.registerFailedTether(); 
			return; 	
		}
		// probability decrease based on distance from surface
		double distanceToBindPt = distToSurface(rod.filamentLocToPt3D(bindLoc));
		double bindProb = (Env.actATetherLength-distanceToBindPt)/Env.actATetherLength; // linear decrease in probability
		// probability decrease based on distribution of ActA
		Pt3D tetherPtInx = tox(tetherPt); 		//convert tetherPt to Listeria coordinate system
		//tetherPtInx.sub(tetherPtInx,coord);
		double actAThereProb = getActAThereProb(tetherPtInx.x);
		if(Env.useActAThereProb){//**USING ActA distribution probability histogram to modify probability of binding along sides
			if (Rdm.uRdm(this) < bindProb*actAThereProb*Env.poleOnlyAdjustFactor) {
				Pt3D ptToBind = rod.filamentLocToPt3D(bindLoc);
				Pt3D actAPt = closestPtOnSurface(ptToBind);
				Pt3D actAPtInx = tox(actAPt);
				ActA actAToTether = activateActAProtein(actAPtInx);
				actAToTether.removable = true;		// distinguish ActA created this way from our static population
				actAToTether.addTether(rod, bindLoc);
				//(Env.reportRegionInfo){actAToTether.registerRegionTether(actAPtInx);}
				if (Env.ArpBranchLink){
					//***insert how decide the chance of ArpBound based on how many of Env.numberActA's are already active
					//double chanceOfArpBound = equilArpBoundActAs*Env.ActA_ArpGrabAreaRatio;	// area covered by equil # of Arp2/3s
					double chanceOfArpBound = Env.percentNucleators;
					if (Rdm.uRdm(this) < chanceOfArpBound) {
						actAToTether.arpBound = true;
						if(Env.keepTrackActAs){
							actAToTether.replaceable = true;
							ActA.activeCounter++;
						}
						
						//talkln (" and it's Arp2/3 bound");
					} else {
						//talkln (" and it's NOT Arp2/3 bound");
					}
				}
			} else {
				Env.registerFailedTether();
			}
		}else{//**NOT USING actAdistribution to determine binding along sides, only the bindProb
			if (Rdm.uRdm(this) < bindProb * Env.poleOnlyAdjustFactor) {//not multiplied by actAThereProb
				Pt3D ptToBind = rod.filamentLocToPt3D(bindLoc);
				Pt3D actAPt = closestPtOnSurface(ptToBind);
				Pt3D actAPtInx = tox(actAPt);
				ActA actAToTether = activateActAProtein(actAPtInx);
				actAToTether.removable = true;		// distinguish ActA created this way from our static population
				actAToTether.addTether(rod, bindLoc);
				//if(Env.reportRegionInfo){actAToTether.registerRegionTether(actAPtInx);}
				if (Env.ArpBranchLink){
					//double chanceOfArpBound = equilArpBoundActAs*Env.ActA_ArpGrabAreaRatio;	// area covered by equil # of Arp2/3s
					double chanceOfArpBound = Env.percentNucleators;
					if (Rdm.uRdm(this) < chanceOfArpBound) {
						actAToTether.arpBound = true;
						//talkln (" and it's Arp2/3 bound");
					} else {
						//talkln (" and it's NOT Arp2/3 bound");
					}
				}
			} else {
				Env.registerFailedTether();
			}
		}
	}

	private double getActAThereProb(double xlocation){
		if(Env.actAFromHist){
			return actAProbs.getValue(xlocation);	// the coordinate center for actAProbs is at rec1 not coord, hence the translation
		}
		else{
			if(Env.poleOnlyActA){
				if(xlocation < 0){
					return 1.0; //if position is on the pole then probability of binding is 1
				}else{
					return 0.0;
				}
			}
			else{
				if(xlocation > 0 & xlocation < Env.listeriaCylLength){
					return 0.0;
				}else{
					return 1.0;
				}
			}
		}
	}
	
	public Pt3D closestPtOnSurface (Pt3D pt) {
		Pt3D ptInx = tox(pt);
		Pt3D ptOnSurfaceInx;
		if (ptInx.x > 0) {	// on cylinder
			Pt3D ptUpCyl = new Pt3D(ptInx.x, 0, 0);
			ptOnSurfaceInx = Pt3D.UnitVec(ptInx, ptUpCyl);	// this is a unit vector
			ptOnSurfaceInx.scale(cylDiam/2.0);				// scale unit vector by radius of cylinder
			ptOnSurfaceInx.inc(ptUpCyl);					// add vector up cylinder
		} else {			// on spherical cap
			ptOnSurfaceInx = Pt3D.UnitVec(ptInx);			// unit vector from rec1 (origin of x) to ptInx
			ptOnSurfaceInx.scale(capRad);				// scale unit vector by radius of cap
		}
		return toX (ptOnSurfaceInx);
	}
	
	public double distToSurface (Pt3D pt) {
		Pt3D ptInx = tox(pt);
		Pt3D ptOnSurfaceInx;
		if (ptInx.x > 0) {	// on cylinder
			Pt3D ptUpCyl = new Pt3D(ptInx.x, 0, 0);
			ptOnSurfaceInx = Pt3D.UnitVec(ptInx, ptUpCyl);	// this is a unit vector
			ptOnSurfaceInx.scale(cylDiam/2.0);				// scale unit vector by radius of cylinder
			ptOnSurfaceInx.inc(ptUpCyl);					// add vector up cylinder
		} else {			// on spherical cap
			ptOnSurfaceInx = Pt3D.UnitVec(ptInx);			// unit vector from rec1 (origin of x) to ptInx
			ptOnSurfaceInx.scale(capRad);				// scale unit vector by radius of cap
		}
		return Pt3D.ptDist(ptInx, ptOnSurfaceInx);
	}
			
	
	public Pt3D tox (Pt3D convertMe) {
		// includes rotation and translation... rec1 is origin
		Pt3D ptInx = Pt3D.Sub(convertMe, rec1);
		ptInx.XTox(this, ptInx);
		return ptInx;
	}
	
	public Pt3D toX (Pt3D convertMe) {
		// rec1 is origin
		Pt3D ptInX = Pt3D.xToNewX(this, convertMe);
		ptInX.inc(rec1);
		return ptInX;
	}
	
	public void reportForces() {
		talkln ("	forceSum is " + forceSum.reportCoords());
		talkln (" 	torqueSum is " + torqueSum.reportCoords());
		talkln ("	bForceSum is " + bForceSum.reportCoords());
		talkln ("	bTorqueSum is " + bTorqueSum.reportCoords());
		talkln ("	colForceSumInx is " + colForceSumInx.reportCoords());
		talkln ("	linkForceSumInx is " + linkForceSumInx.reportCoords());
		talkln ("	randForces is " + randForces.reportCoords());
		talkln ("	randTorques is " + randTorques.reportCoords());
		talkln ("	bVeloc is " + bVeloc.reportCoords());
		talkln ("	bAngVeloc is " + bAngVeloc.reportCoords());
	}
	
	public void reportStats () {
		talk ("End1 at " + end1.reportCoords());
		talkln ("End2 at " +end2.reportCoords());
		talk ("Number of tethered filaments = " + String.valueOf(linkerCt));
	}
	
	public void makeEquilArpBoundActAs (int equilArpBound) {
		mkActAHistogram();	// histogram of ActA locations
		for (int i=0; i < equilArpBound; i++) { 
			ActA newArpBound = createRdmActiveActA();
			if (newArpBound != null) { newArpBound.arpBound = true; }	// this ActA is active because an Arp2/3 has bound
		}
		ActA.mkActALocHisto();
		//mkActAProbsHisto(); //goes with method of this name (commented out)
	}
	
	public static int countTipsInCylSection () {
		tipsInCylSection = 0;
		A_Rod curR;
		for (int i=0;i<Player.playerCt;i++) {
			if (Player.thePlayers[i] instanceof A_Rod) {
				curR = (A_Rod)Player.thePlayers[i];
				if (curR.inCylinderViscousBox()) {
					tipsInCylSection++;
				}
			}
		}
		return tipsInCylSection;
	}
	
	private void ckArpFlux () {
		if (Rdm.uRdm(this) < Env.kArpFluxIn*Env.deltaT) {
			ActA newArpBound = createRdmActiveActA ();
			if (newArpBound != null) { newArpBound.arpBound = true;	}	// this ActA is active because an Arp2/3 has bound
		}
	}
	
	private ActA createRdmActiveActA(){
		if (Env.actAFromHist){
			return createRdmActiveActAFromHist();
		}else {
			if(Env.poleOnlyActA){
				return createRdmActivePoleOnlyActA();
			}else{
				return createRdmActiveBothPolesActA();
			}	
		}
	}
	
	
	// new method to create uniform ActA's only on a single spherical cap - poleonlyActA
	private ActA createRdmActivePoleOnlyActA() {
		double theta=0, psi;
		double xRad=0, yRad, zRad;
		
		theta = (Math.PI /2)*Rdm.uRdm(this);
		
		while (Rdm.uRdm(this)> Math.sin(theta)){
			theta = (Math.PI /2)*Rdm.uRdm(this);
		} // this part makes sure that if theta is closer to pole, ActA's are distributed there less often
		
		psi = 2*Math.PI*Rdm.uRdm(this);
		
		xRad = -1*capRad*Math.cos(theta);
		yRad = capRad*Math.sin(theta)*Math.cos(psi);
		zRad = capRad*Math.sin(theta)*Math.sin(psi);
		locOnLis = new Pt3D(xRad,yRad,zRad);
		ActA newActA = activateActAProtein (locOnLis);
		return newActA;
	}
	
	//new method to create uniform ActA's only on both spherical caps bothpolesActA
	private ActA createRdmActiveBothPolesActA() {
		double theta=0, psi;
		double xRad=0, yRad, zRad;
		
		theta = (Math.PI /2)*Rdm.uRdm(this);
		
		while (Rdm.uRdm(this)> Math.sin(theta)){
			theta = (Math.PI /2)*Rdm.uRdm(this);
		}// this part makes sure that if theta is closer to pole, ActA's are distributed there less often
		
		psi = 2*Math.PI*Rdm.uRdm(this);
		
		xRad = -1*capRad*Math.cos(theta);
		if (Rdm.uRdm(this) < 0.5) {
			xRad = Env.listeriaCylLength + capRad*Math.cos(theta);
			}
		
		yRad = capRad*Math.sin(theta)*Math.cos(psi);
		zRad = capRad*Math.sin(theta)*Math.sin(psi);
		locOnLis = new Pt3D(xRad,yRad,zRad);
		ActA newActA = activateActAProtein (locOnLis);
		return newActA;
	}
	
	private ActA createRdmActiveActAFromHist () {
		Random actARnd = new Random();	// set seed to 1825 make identical distributions
		double theta=0, psi;
		double xRad=0, yRad, zRad;
		double lnRdm;
		// generate random location along length... check histogram if OK
		boolean seekingSlot = true;
		while (seekingSlot) {
			//lnRdm = Rdm.uRdm(this);
			lnRdm = actARnd.nextDouble();
			xRad = lnRdm*length - capRad; // zero is at rec1 (center of cap at end1)
			if (actALocs.decValue(xRad)) { seekingSlot = false; }  // found a valid location for a new ActA
		}
		if ((xRad < 0) | (xRad > cylLength)) {	// on hemispherical caps
			if (xRad < 0) { theta = Math.acos(-xRad/capRad); }
			if (xRad > cylLength) { theta = Math.acos(-(xRad-cylLength)/capRad); }
		} else {		// on cylinder
			theta = Math.PI/2.0;
		}
		psi = 2*Math.PI*actARnd.nextDouble();
		yRad = capRad*Math.sin(theta)*Math.cos(psi);
		zRad = capRad*Math.sin(theta)*Math.sin(psi);
		locOnLis = new Pt3D(xRad,yRad,zRad);
		ActA newActA = activateActAProtein (locOnLis);
		return newActA;
	}
	
	private void mkActAHistogram () {
		//double [] SRMeasure = new double [] {0.00705,0.02173,0.04172,0.06276,0.07789,0.09134,0.09849,0.09980,0.09643,0.09289,0.08474,0.07267,0.05922,0.04376,0.02714,0.01543,0.00482,0.00211,0.0,0.0};
		//first Sus type1 dist double [] SRMeasure = new double [] {0.033,0.043,0.056,0.064,0.067,0.065,0.062,0.063,0.060,0.057,0.057,0.054,0.050,0.047,0.046,0.042,0.037,0.030,0.027,0.020,0.013,0.007};
		
		if (Env.useNewToonDist) { Env.setNewToonDist(); }
		
		double [] SRMeasure = Env.actAMeasure;
		double [] SRProb = Env.actAProb;
		
		actAProbs = new HistogramPlus (SRProb.length,-capRad,length-capRad,null,"ActAProbs",null);	// the normalized probability distribution ....check dimensions of histogram... changes with diff. exp. measurements
		actALocs = new HistogramPlus (SRMeasure.length,-capRad,length-capRad,null,"ActALocs",null);	// the discrete loaction of ActA nucleators
		counterHist = new HistogramPlus (SRMeasure.length,-capRad,length-capRad,null,"ActALocs",null);	// a copy of ActAloc h
		// load up histogram with ActAs based on Susanne Rafelski measurements (SRMeasure)
		for (int i=0;i<SRMeasure.length;i++) {
			actAProbs.addValue(i,SRProb[i]);
			int curBinSum = (int) (Math.floor(SRMeasure[i]*equilArpBoundActAsInt));
			actALocs.addValue(i,curBinSum);
			//talkln ("location " + String.valueOf(actALocs.binIntervals[i]) + " has " + String.valueOf(actALocs.binSums[i]));
			if(Env.keepTrackActAs){
				counterHist.addValue(i,curBinSum);
			}
		}
		
		// write out the actALocs and Probs data
		String fileName = null;
		if (!Env.remote) {
			fileName = FileOps.getFileToSave("LocsAndProb", null);
		} else if (Env.qkMovie) {
			fileName = Env.datFilePath + File.separator + "ActA";
		}
		if (fileName != null) {
			FileOps.mkHistogramFile(actALocs, fileName + "Locs");
			FileOps.mkHistogramFile(actAProbs,fileName + "Probs");
		}
		
		// check ptCt versus desired number of ActAs
		if (actALocs.ptCt < equilArpBoundActAsInt) { 
			int numToAdd = equilArpBoundActAsInt - actALocs.ptCt;
			for (int j=0;j<numToAdd;j++) { actALocs.addRandom(); }
		}
		if (actALocs.ptCt > equilArpBoundActAsInt) {
			int numToDec = actALocs.ptCt - equilArpBoundActAsInt;
			for (int j=0;j<numToDec;j++) { actALocs.decRandom(); }
		}
			
		talkln ("Placed " + String.valueOf(actALocs.ptCt) + " points in histogram.  Equil. value is " + String.valueOf(equilArpBoundActAsInt));
		if (!Env.remote) { this.mkActAProbsHisto(); }
	}
	
	public static void updateEnd2LocHisto() { 
		A_Rod curR;
		for (int i=0;i<Player.playerCt;i++) {
			if (Player.thePlayers[i] instanceof A_Rod) {
				curR = (A_Rod)Player.thePlayers[i];
				end2Locs.addValue(curR.end2InLMFrame.x);
			}
		}
	}
	
	public static void updateTethersHisto() {
		Linker curL;
		for (int i=0;i<Linker.linkerCt;i++) {
			curL = Linker.theLinkers[i];
			if (curL.linked) {
				tethers.addValue(curL.locOnLis.x-Listeria.halfCylLength);
			}
		}
	}
	
	public static void updateFActinAnglesHisto() {
		A_Rod curR;
		for (int i=0;i<Player.playerCt;i++) {
			if (Player.thePlayers[i] instanceof A_Rod) {
				curR = (A_Rod)Player.thePlayers[i];
				double dotLMUVec = Pt3D.Dot(curR.uVec, Player.listeria.uVec);
				double angTween = Math.acos(dotLMUVec);
				if (dotLMUVec < 0) { angTween = Math.PI - angTween; }	// so all angles swung around from positive LM.uVec
				fActinAngles.addValue(angTween);
			}
		}
	}

	public static void updateHistos() {
		updateEnd2LocHisto();
		updateTethersHisto();
		updateFActinAnglesHisto();
		//updateTethersFormedHisto();	//nothing done
		//updateTethersBrokenHisto();	//nothing done
		//updatePathForceHisto();		//nothing done
		//updateRadialForceHisto();		//nothing done
		
	}
	
	public static void resetHistos() {
		end2Locs.clearBins();
		tethers.clearBins();
		tethersFormed.clearBins();
		tethersBroken.clearBins();
		fActinAngles.clearBins();
		pathForce.clearBins();
		radialForce.clearBins();
		// don't reset filLife histo
	}
	
	//a histogram to see if getActAThereProb() is working... a temporary thing
	private void mkActAProbsHisto(){
		HistogramPlus actAProbsHist = new HistogramPlus(500,-2,2,null,"ActAProbsDist",null);
		for (int i=0; i<500; i++){
			double xInc = Env.listeriaLength / 500;
			double x = (-1*Env.listeriaCapRad) + (i*xInc);
			double probAtX = getActAThereProb(x);
			//actAProbsHist.addValue(i, probAtX); 	//divides xaxis into 500, so if do -2to2 will plot entire x axis
			actAProbsHist.addValue(x, probAtX); //modification allows plotting in correct position on x axis
		}
		//HistoPlotter.plotHist(actAProbsHist, true);
		talk ("about to plot ActAprobs  histogram...");
		HistoPlotter actAProbsHistP = new HistoPlotter();
		actAProbsHistP.setPlotColor(Color.blue);
		actAProbsHistP.setYPixelPad(300);
		actAProbsHistP.plotHist(actAProbsHist,true);
		//File_Ops.takeRawPicture(actAProbsHistP.hPanel.theImage,fileName + ".RAW");
		talkln ("Done!");
		
	}
	
	private void createToupeeOfActA () {
		double theta, psi, xRad, yRad, zRad, xTot;
		for (int i=0;i<100;i++) {
			theta = (10*Math.PI/180)*Rdm.uRdm(this);
			xRad = capRad*Math.cos(theta);	// positive sign 'cause +x is towards end2
			xTot = Env.listeriaLength-2*Env.listeriaCapRad+xRad; 
			psi = 2*Math.PI*Rdm.uRdm(this);
			yRad = capRad*Math.sin(theta)*Math.cos(psi);
			zRad = capRad*Math.sin(theta)*Math.sin(psi);
			locOnLis = new Pt3D(xTot,yRad,zRad);
			ActA newActA = activateActAProtein (locOnLis);
			newActA.arpBound = true;
		}
	}
	
	public ActA activateActAProtein (Pt3D locOnLis) {
		synchronized (actALock) {
			ActA newActA = new ActA (this, locOnLis);
			Linker.addLinker(newActA);
			return newActA;
		}
	}
	
	
	public void makePaintPoints () {
		double capRadSqrd = Math.pow(capRad,2);
		// Concentric circle pts
		double xRad, yRad, zRad, yLim;
		for (int i = 0; i <=numCircs; i++) {
			xRad = i*capRad/numCircs;
			if (i == numCircs) { xRad += -capRad/(4*numCircs); }
			yLim = Math.sqrt(capRadSqrd - Math.pow(xRad,2));
			for (int j = 0; j <= numPts; j++) {
				yRad = j*yLim/numPts;
				zRad = Math.sqrt(capRadSqrd - Math.pow(yRad,2) - Math.pow(xRad,2));
				if (Double.isNaN(zRad)) { zRad = 0; };
				capPaintPts[i][j] = new Pt3D (xRad,yRad,zRad);
			}
		}
			
		// Cylindrical section pts
		double theta;
		double thetaInc = 2*Math.PI/numLines;
		for (int i = 0; i <= numLines; i++) {
			theta = i*thetaInc;
			yRad = (cylDiam/2)*Math.cos(theta);
			zRad = (cylDiam/2)*Math.sin(theta);
			cylPaintPts[i] = new Pt3D (0,yRad,zRad);
		}
		
		// Spherical caps
		double phi;
		double phiInc = Math.PI/(2*numCapPts);
		for (int i = 0; i <= numLines; i++) {
			theta = i*thetaInc;
			for (int j = 0; j <= numCapPts; j++) {
				phi = j*phiInc;
				xRad = capRad*Math.cos(phi);
				yRad = capRad*Math.sin(phi)*Math.cos(theta);
				zRad = capRad*Math.sin(phi)*Math.sin(theta);
				capPts [i][j] = new Pt3D(xRad,yRad,zRad);
			}
		}			
	}
				
	public void showYourself (Graphics g, double scale, PinCam cam) {
		g.setColor(playerColor);
		// note: xoffset, yoffset are non-zero only for the zoom window
		// Concentric circle pts
		Pt3D curPt;
		Pt3D [] pts = new Pt3D[4];
		Pixel [][] segPix = new Pixel [8][2];
		double zSign;
		// initialize arrays of Pt3D and Pixel
		for (int i = 0; i < 4; i++ ) { pts[i] = new Pt3D(); }
		for (int i = 0; i < 8; i ++) { segPix[i][0] = new Pixel();  segPix[i][1] = new Pixel(); }
		
	/*	for (int i = 0; i <=numCircs; i++) {
			for (int j = 0; j <= numPts; j++) {
				curPt = capPaintPts[i][j];
				for (int k =0; k < 5; k+=4) {
					if (k == 0) { zSign = 1; } else { zSign = -1; }
					pts[0].add(rec1,Pt3D.xToNewX (this, new Pt3D(-curPt.x, curPt.y, zSign*curPt.z)));
					pts[1].add(rec1,Pt3D.xToNewX (this, new Pt3D(-curPt.x, -curPt.y, zSign*curPt.z)));
					pts[2].add(rec2,Pt3D.xToNewX (this, new Pt3D(curPt.x, curPt.y, zSign*curPt.z)));
					pts[3].add(rec2,Pt3D.xToNewX (this, new Pt3D(curPt.x, -curPt.y, zSign*curPt.z)));
					for (int m = 0; m < 4; m++) {
						segPix[m+k][1].ptToPix(pts[m],scale,cam.offset,cam.e);
					}
				}
				
				if (j != 0) { for (int n = 0; n < 8; n++) { Pixel.drawLine(g, segPix[n][0], segPix[n][1]); } }
				for (int n = 0; n < 8; n++) { segPix[n][0].copy(segPix[n][1]); }
			}
		}  */
		
		// The cylindrical section of the Listeria
		Pt3D dR = new Pt3D();
		Pt3D ptA = new Pt3D(); Pt3D ptB = new Pt3D();
		Pixel pixA = new Pixel(); Pixel pixB = new Pixel();
		for (int j = 0; j <= numLines; j++) {
			curPt = cylPaintPts[j];
			dR.xToX(this, curPt);
			ptA.add(rec1, dR);
			ptB.add(rec2, dR);
			pixA.ptToPix(ptA,scale,cam.offset,cam.e);
			pixB.ptToPix(ptB,scale,cam.offset,cam.e);
			Pixel.drawLine(g,pixA,pixB);
			//if (j < 1) { g.setColor(Color.yellow); g.drawLine(pixA.x, pixA.y, pixB.x, pixB.y); } else { g.setColor(playerColor); }
		}
		
		// spherical cap of the Listeria
		Pt3D [][] cPts = new Pt3D [2][2];
		Pixel [][] cPix = new Pixel [2][2];
		for (int i = 0; i < 2; i++) { cPts[i][0] = new Pt3D(); cPts[i][1] = new Pt3D(); }
		for (int i = 0; i < 2; i++) { cPix[i][0] = new Pixel(); cPix[i][1] = new Pixel(); }
		
		for (int i = 0; i <= numLines; i++) {
			dR.xToX(this, capPts[i][0]);
			cPts[0][0].add(rec2, dR);
			cPix[0][0].ptToPix(cPts[0][0],scale,cam.offset,cam.e);
			dR.xToX(this, new Pt3D(-capPts[i][0].x, capPts[i][0].y, capPts[i][0].z));
			cPts[1][0].add(rec1, dR);
			cPix[1][0].ptToPix(cPts[1][0],scale,cam.offset,cam.e);
			for (int j = 1; j <= numCapPts; j++) {
				dR.xToX(this, capPts[i][j]);
				cPts[0][1].add(rec2, dR);
				cPix[0][1].ptToPix(cPts[0][1],scale,cam.offset,cam.e);
				dR.xToX(this, new Pt3D(-capPts[i][j].x, capPts[i][j].y, capPts[i][j].z));
				cPts[1][1].add(rec1, dR);
				cPix[1][1].ptToPix(cPts[1][1],scale,cam.offset,cam.e);
				Pixel.drawLine(g, cPix[0][0], cPix[0][1]);
				Pixel.drawLine(g, cPix[1][0], cPix[1][1]);
				cPix[0][0].copy(cPix[0][1]);
				cPix[1][0].copy(cPix[1][1]);
			}
		}
	}
				
	
	public void stereoVision (Graphics g, double scale, PinCam camL, PinCam camR) {
		g.setColor(playerColor);
		// note: xoffset, yoffset are non-zero only for the zoom window
		// Concentric circle pts
		Pt3D curPt;
		Pt3D [] pts = new Pt3D[4];
		Pixel [][] segPixL = new Pixel [8][2];
		Pixel [][] segPixR = new Pixel [8][2];
		// initialize Pt3D and Pixel arrays
		for (int i = 0; i < 4; i++ ) { pts[i] = new Pt3D(); }
		for (int i = 0; i < 8; i ++) { segPixL[i][0] = new Pixel();  segPixL[i][1] = new Pixel(); }
		for (int i = 0; i < 8; i ++) { segPixR[i][0] = new Pixel();  segPixR[i][1] = new Pixel(); }
		
		double zSign;
		for (int i = 0; i <=numCircs; i++) {
			for (int j = 0; j <= numPts; j++) {
				curPt = capPaintPts[i][j];
				for (int k =0; k < 5; k+=4) {
					if (k == 0) { zSign = 1; } else { zSign = -1; }
					pts[0].add(rec1,Pt3D.xToNewX (this, new Pt3D(-curPt.x, curPt.y, zSign*curPt.z)));
					pts[1].add(rec1,Pt3D.xToNewX (this, new Pt3D(-curPt.x, -curPt.y, zSign*curPt.z)));
					pts[2].add(rec2,Pt3D.xToNewX (this, new Pt3D(curPt.x, curPt.y, zSign*curPt.z)));
					pts[3].add(rec2,Pt3D.xToNewX (this, new Pt3D(curPt.x, -curPt.y, zSign*curPt.z)));
					for (int m = 0; m < 4; m++) {
						segPixL[m+k][1] = camL.ptToPix(pts[m],scale);
						segPixR[m+k][1] = camR.ptToPix(pts[m],scale);
					}
				}
				if (j != 0) { 
					for (int n = 0; n < 8; n++) { 
						if ((segPixL[n][0].show) | (segPixL[n][1].show)) { Pixel.drawLine(g, segPixL[n][0], segPixL[n][1]); }
						if ((segPixR[n][0].show) | (segPixR[n][1].show)) { Pixel.drawLine(g, segPixR[n][0], segPixR[n][1]); }
					}
				}
				for (int n = 0; n < 8; n++) { 
					segPixL[n][0] = segPixL[n][1]; 
					segPixR[n][0] = segPixR[n][1]; 
				}
			}
		} 
		
		// The cylindrical section of the Listeria
		Pt3D dR = new Pt3D();
		Pt3D ptA = new Pt3D(); Pt3D ptB = new Pt3D();
		Pixel pixAL = new Pixel(); Pixel pixBL = new Pixel();
		Pixel pixAR = new Pixel(); Pixel pixBR = new Pixel();
		for (int j = 0; j <= numLines; j++) {
			curPt = cylPaintPts[j];
			dR.xToX(this, curPt);
			ptA.add(rec1, dR);
			ptB.add(rec2, dR);
			pixAL = camL.ptToPix(ptA,scale);
			pixBL = camL.ptToPix(ptB,scale);
			pixAR = camR.ptToPix(ptA,scale);
			pixBR = camR.ptToPix(ptB,scale);
			if ((pixAL.show) | (pixBL.show)) { Pixel.drawLine(g,pixAL, pixBL); }
			if ((pixAR.show) | (pixBR.show)) { Pixel.drawLine(g,pixAR, pixBR); }
		}
	}

	public void saveYourself (DataOutputStream ds) {
		try {
			ds.writeByte(4);		// the Listeria identifier
			ds.writeInt(idnum);	
			Pt3D.writePt3D(ds, coord);									
			Pt3D.writePt3D(ds, uVec);
		} catch (IOException ioe) { talk ("some sort of error in saving Listeria info"); }	
	}
	
	public static Player readYourself (DataInputStream ds, RocketBugs MC) {
		try {
			int idnum = ds.readInt();
			Pt3D coord = Pt3D.readPt3D(ds);
			Pt3D uVec = Pt3D.readPt3D(ds);
			Linker.linkerCt = 0;	// remove previously created linkers
			Listeria newLis = new Listeria (coord, uVec, MC.sphImp);
			newLis.initialize();
			return newLis;
		} catch (IOException ioe) { talk ("some sort of error in reading Listeria info"); }
		return null;	
	}
	
	
}