package rocketBugs;
/* the Pixel class... just a 2D integer point {x,y} */

/*  <RocketBugs - for the simulation of actin-based Listeria motility>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.Math;
import java.awt.*;

public class Pixel {
	int x,y;
	double viewDist;
	boolean show = true;
	
	public Pixel () {  this.x = 0; this.y = 0; }
	public Pixel (int [] pt) { this.x = pt[0]; this.x = pt[1]; }
	public Pixel (int x, int y) { this.x = x; this.y = y; }
	public Pixel (boolean show) { this.x = -1; this.y = -1; this.show = show; }
		
	public void ptToPix (Pt3D pt, double scale, Pt3D offset, Pt3D eVec) {
		this.x = (int) Math.rint((eVec.z*(pt.x-offset.x) - eVec.x*(pt.z-offset.z))*scale);
		this.y = (int) Math.rint((pt.y-offset.y)*scale);
	}
	
	public static Pixel ptToNewPix (Pt3D pt, double scale, Pt3D offset, Pt3D eVec) {
		int x = (int) Math.rint((eVec.z*(pt.x-offset.x) - eVec.x*(pt.z-offset.z))*scale);
		int y = (int) Math.rint((pt.y-offset.y)*scale);
		return new Pixel(x,y);
	}
	
	public static Pt3D pixToPt (Pixel pix, double scale, double [] offset, double [] trigAng) {
		return pixToPt (new double [] {pix.x, pix.y}, scale, offset, trigAng);
	}
	
	public static Pt3D pixToPt (double [] pix, double scale, double [] offset, double [] trigAng) {
		double x,y,z;
		x = offset[0] + trigAng[1]*(double)(pix[0])/scale;
		y = offset[1] + (double)(pix[1])/scale;
		z = offset[2]  - trigAng[0]*(double)(pix[0])/scale;
		return new Pt3D (x,y,z);
	}
	
	public static Pixel pixAdd (Pixel pix1, Pixel pix2) {
		return new Pixel (pix1.x+pix2.x, pix1.y+pix2.y);
	}
	
	public void copy (Pixel pix) {
		this.x = pix.x; this.y = pix.y; this.show = pix.show; this.viewDist = pix.viewDist;
	}
	
	public static void drawLine (Graphics g, Pixel A, Pixel B) {
		drawLine(g, new int [] {A.x,A.y}, new int [] {B.x,B.y});
	}
	
	public static void drawLine (Graphics g, int [] pt1, int [] pt2) {
		g.drawLine(pt1[0],pt1[1],pt2[0],pt2[1]);
	}

}