package rocketBugs;
/*
	The Player Class... the parent class of all the things that move (and some that don't)  in our model
*/

/*  <RocketBugs - for the simulation of actin-based Listeria motility>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.Math;
import java.awt.*;
import java.io.*;

public class Player implements java.io.Serializable {
	static Player [] thePlayers = new Player [Env.maxPlayers];	// list of players in this simulation
	static int playerCt = 0;	// number of players in thePlayers list
	static Listeria listeria;		// the one and only
	static PlayerThreads pTs = new PlayerThreads ();
	static RocketBugs MC = null;		// players may need to reference main file
	static SphImpPlayers impP;		// all explict players need to be able to reference implicit players
	static Rdm rdm;				// link to one Rdm instance... access to all random numbers used
	// establish the limits of the environment in which players exist
	static final Pt3D xUnitVector = new Pt3D(1,0,0);	// unit vector along body-fixed x-axis
	static final Pt3D yUnitVector = new Pt3D(0,1,0);	// unit vector along body-fixed y-axis
	static final Pt3D zUnitVector = new Pt3D(0,0,1);	// unit vector along body-fixed z-axis
	static final Pt3D zeroVec = new Pt3D();	// just a zero Pt3D
	static final Object playerLock = new Object();	// generic object for synchronizing new player creation
	double age;
	//viscous blobs
	//static Pt3D viscBlobTransGam = new Pt3D();
	//static Pt3D viscBlobRotGam = new Pt3D();
	int numViscBlobs = 0;
	
	short idnum = -1;	// array position of this player in complete player list
	int branchId = -1;	// array position of this player in branch player list	*** these ids can change each step
	int meshId = -1;	// array position of this player in mesh player list
	Color playerColor;
	Pt3D bTransGam = new Pt3D(); 	// body-fixed viscous translational resistances in (x,y,z}.
	Pt3D bRotGam = new Pt3D();		// body-fixed viscous rotational coefficients (Wx, Wy, Wz)
	Pt3D bRestrictRotGam = new Pt3D(); 	// used for restricted rotational motion of filaments and branches
	Pt3D bTransDiff = new Pt3D();		// body-fixed translational diffusion coefficients, from bTransGam through Einstein's relation
	Pt3D bRotDiff = new Pt3D();		// body-fixed rotational diffusion coefficients
	Pt3D coord = new Pt3D();  			// the coordinate point for the CM 
	Pt3D angle = new Pt3D();			// the angle values for the player
	Pt3D veloc = new Pt3D();			// the fixed frame translational velocity values Xdot, Ydot, Zdot
	Pt3D angVeloc = new Pt3D();		// the angular velocities psidot, thetadot, phidot
	Pt3D bVeloc = new Pt3D(); 		// the body-fixed frame velocities xdot, ydot, zdot
	Pt3D bAngVeloc = new Pt3D();	// the body-fixed frame angular velocities Wx, Wy, Wz
	Pt3D uVec = new Pt3D();		// the unit vector that describes the orientation of the player
	Pt3D t1Vec = new Pt3D();		// the first transvers vector for this body... in y direction
	Pt3D t2Vec = new Pt3D();		// the second tranverse vector.. in z direction
	Pt3D cosAng = new Pt3D();	// cosines of Euler angles {psi,theta,phi}
	Pt3D sinAng = new Pt3D();	// sines of Euler angles {psi,theta,phi}
	double [][] transXTox = new double [3][3];	// transformation matrix from fixed to body-fixed frame
	double [][] transxToX = new double [3][3];	// the inverse transformation... body-fixed to fixed
	boolean fixedInSpace = false;		// flag to indicate if this player is tied down to surroundings
	boolean remove = false;			// flag to indicate 'death' of this player
	boolean structure = false;			// flag to indicate if this player is a superstructure of other players
	boolean selected = false;			// flag to indicate if this player has been GUI selected
	SuperStructure myStructure = null;			// link to some secondary structure that this Player belongs to
	boolean structureMember = false;	// flag to indicate this Player belongs to a secondary structure
	int structureId;					// id number in myStructure's member list
	// force, torque sums
	Pt3D linkForceSum = new Pt3D();		// forces from linkers
	Pt3D linkTorqueSum = new Pt3D();	// torques from linkers
	Pt3D colForceSum = new Pt3D();		// collision force sum
	Pt3D colTorqueSum = new Pt3D();		// collision torque sum
	Pt3D fricForceSum = new Pt3D();		// friction force sum
	Pt3D fricTorqueSum = new Pt3D(); 	// friction torque sum
	Pt3D colForceSumInx = new Pt3D();	// holds value of Env.colFactor * colForceSum
	Pt3D linkForceSumInx = new Pt3D();	// holds value of Env.linkFactor * linkForceSum
	Pt3D randForces = new Pt3D();		// random translational forces (Fx,Fy,Fz)
	Pt3D randTorques = new Pt3D();		// random rotational torques (Tx,Ty,Tz)
	Pt3D forceSum = new Pt3D(); 			// fixed-frame force sums... FX, FY, FZ
	Pt3D torqueSum = new Pt3D();		// Euler axes torque sums
	Pt3D bForceSum = new Pt3D();		// body-fixed force sum
	Pt3D bTorqueSum = new Pt3D();		// body-fixed torque sum
	Pt3D dR = new Pt3D();				// vector from application of force to CM
	// some Pt3Ds used in calculating random forces
	Pt3D v1 = new Pt3D();
	Pt3D v2 = new Pt3D();
	Pt3D rsq = new Pt3D();
	Pt3D facterm =new Pt3D(); 
	Pt3D fac1 = new Pt3D();
	Pt3D fac2 = new Pt3D();
	Pt3D tempPt = new Pt3D();
	// each player has its own rdm index
	int uRdmIndex = (int)(Math.floor(Env.mtRNG.raw()*Env.numOfRdmNumbers));
	int cRdmIndex = (int)(Math.floor(Env.mtRNG.raw()*Env.numOfRdmNumbers));
		
	//for reporting forces in regions A through D
	Pt3D colForceSumInA = new Pt3D();		// collision force sum in region A
	Pt3D colForceSumInB = new Pt3D();		// collision force sum in region B
	Pt3D colForceSumInC = new Pt3D();		// collision force sum in region C
	Pt3D colForceSumInD = new Pt3D();		// collision force sum in region D
	Pt3D colForceSumInE = new Pt3D();		// collision force sum in region E
	Pt3D linkForceSumInA = new Pt3D();		// force from linkers in region A
	Pt3D linkForceSumInB = new Pt3D();		// force from linkers in region B
	Pt3D linkForceSumInC = new Pt3D();		// force from linkers in region C
	Pt3D linkForceSumInD = new Pt3D();		// force from linkers in region D
	Pt3D linkForceSumInE = new Pt3D();		// force from linkers in region E
	Pt3D forceSumInA = new Pt3D(); 			// fixed-frame force sums... FX, FY, FZ in region A
	Pt3D forceSumInB = new Pt3D(); 			// fixed-frame force sums... FX, FY, FZ in region B
	Pt3D forceSumInC = new Pt3D(); 			// fixed-frame force sums... FX, FY, FZ in region C
	Pt3D forceSumInD = new Pt3D(); 			// fixed-frame force sums... FX, FY, FZ in region D
	Pt3D forceSumInE = new Pt3D(); 			// fixed-frame force sums... FX, FY, FZ in region E
	int linkerCtInA, linkerCtInB, linkerCtInC, linkerCtInD, linkerCtInE;
	int lastLinkerCtInA, lastLinkerCtInB, lastLinkerCtInC, lastLinkerCtInD, lastLinkerCtInE;
	
	
	// max force and move for each player.... useful in attenuating forces and error checking
	double maxForce;
	double maxMove;
	double colAtten;
	double linkAtten;
	double buckleAtten;
	double snapAtten;
	// adhesions
	int adhesionCount = 0;
	
	// special, in testing, to keep or not
	boolean justRemoved = false;
	
	// for config saving only
	int myStructuresIdNumber;				// only used for resetting from a config file
	
	// for time-saving
	boolean updateNow = false;				// once a player is mechanically dead, update less frequently
	int updateCt = 0;						// counter of steps since an update
	
	int colliderCt,linkerCt,lastColliderCt,lastLinkerCt;	// counters of actual and possible collisions
	
	// for collision tests
	RetObj retObj = new RetObj();
	
	public Player () {
		// initialize all counters, forces, markers, and the velocity vector
		age = 0;
		linkerCt = 0;
		colliderCt = 0;
		linkerCtInA =0;
		linkerCtInB =0;
		linkerCtInC =0;
		linkerCtInD =0;
		Player.addPlayer(this);
	}
	
	static class PlayerThreads extends ThreadSet {
		PlayerThreads () {
			super (Env.numberOfPlayerThreads, "Player Threads");
		}
	
		public void divideAndConquer (int jobId) {
			this.jobId = jobId;
			switch (jobId) {
				case Env.moveStart:
					for (int i=0; i <= numThreads; i++) {
						jobDiv[i] = i*playerCt/numThreads;	// divide the job amongst threads
					}
					spawn(); break;
				case Env.interactStart:
					for (int i=0; i <= numThreads; i++) {
						jobDiv[i] = i*playerCt/numThreads;	// divide the job amongst threads
					}
					spawn(); break;
				case Env.listeriaColStart:
					for (int i=0; i <= numThreads; i++) {
						jobDiv[i] = i*playerCt/numThreads;	// divide the job amongst threads
					}
					spawn(); break;
			}
			
		}
		
		public void regroup (int jobId) {
			switch (jobId) {
				case Env.moveStop:
					gather(); break;
				case Env.interactStop:
					gather(); break;
				case Env.listeriaColStop:
					gather(); break;
			}
		}
		
		public void execute (int threadId) {
			switch (jobId) {
				case Env.moveStart:
					for (int i = jobDiv[threadId]; i < jobDiv[threadId+1]; i++) {
						if (!thePlayers[i].structureMember) {
							thePlayers[i].movePlayer();
						}
					}
					break;
				case Env.interactStart:
					for (int i = jobDiv[threadId]; i < jobDiv[threadId+1]; i++) {
						thePlayers[i].interactionSim(Env.deltaT);
					}
					break;
				case Env.listeriaColStart:
					for (int i = jobDiv[threadId]; i < jobDiv[threadId+1]; i++) {
						if (!thePlayers[i].justRemoved) {
							thePlayers[i].checkListeriaCollision(listeria);
						}

					}
					break;
			}
		}
	}
	
	public class RetObj {
		// this is the object passed by from line-line and line-point intersect tests
		Pt3D conPt1, conPt2, ray1, ray2, ray3, ray4;
		double conDist = 0;
		boolean collision = false;
		
		public RetObj () {
			conPt1 = new Pt3D();
			conPt2 = new Pt3D();
			ray1 = new Pt3D();
			ray2 = new Pt3D();
			ray3 = new Pt3D();
			ray4 = new Pt3D();
		}
		
		public void reset () {
			collision = false;
		}
	}
	
	public void sepaku() {
		playerColor = null;
		bTransGam = null;
		bRotGam = null;
		bRestrictRotGam = null;
		bTransDiff = null;
		bRotDiff = null;
		coord = null;
		angle = null;
		veloc = null;
		angVeloc = null;
		bVeloc = null;
		bAngVeloc = null;
		uVec = null;
		t1Vec = null;
		t2Vec = null;
		cosAng = null;
		sinAng = null;
		transXTox = null;
		transxToX = null;
		myStructure = null;			// link to some secondary structure that this Player belongs to
		// force, torque sums
		linkForceSum = null;
		linkTorqueSum = null;
		colForceSum = null;
		colTorqueSum = null;
		colForceSumInx = null;
		linkForceSumInx = null;
		colForceSumInA = null;
		colForceSumInB = null;
		colForceSumInC = null;
		colForceSumInD = null;
		colForceSumInE = null;
		linkForceSumInA = null;
		linkForceSumInB = null;
		linkForceSumInC = null;
		linkForceSumInD = null;
		linkForceSumInE = null;
		forceSumInA = null;
		forceSumInB = null;
		forceSumInC = null;
		forceSumInD = null;
		forceSumInE = null;

		randForces = null;
		randTorques = null;
		forceSum = null;
		torqueSum = null;
		bForceSum = null;
		bTorqueSum = null;
		dR = null;
		// some Pt3Ds used in calculating random forces
		v1 = new Pt3D();
		v2 = new Pt3D();
		rsq = new Pt3D();
		facterm =new Pt3D(); 
		fac1 = new Pt3D();
		fac2 = new Pt3D();
		tempPt = null;
	
		justRemoved = true;
	}
		
	// Here are some methods that players can override to actually do something
	public void initialize () {}
	public void movePlayer () {}
	public void checkBuckling (double colForce) { buckleAtten = 1; }
	public void checkSnap (double linkForce) { snapAtten = 1; }
	public void checkListeriaCollision (Listeria listeria) { }
	public void interactionSim (double dT) {}
	public void subsumedMove (Pt3D centroid, Pt3D cOff, Pt3D uVec, Pt3D tVec) {}
	public void viscousBlobSim (double effectiveLength, double dT) {};
	public void reportStats () {}
	public void showYourself (Graphics g, double scale, PinCam cam) {}
	public void stereoVision (Graphics g, double scale, PinCam camL, PinCam camR) {}
	public void saveYourself (DataOutputStream ds) {}
	public static Player readYourself (DataInputStream ds, RocketBugs MC) { return null; }
	public void initializeAfterRead () {}
	public void initializeFromFileAction (Player [] thePlayers) {}
	
	public static void reportRunStats() {
		int totalRodCt = 0;
		int cappedRodCt = 0;
		int structureCt = 0;
		int numMechDead = 0;
		A_Rod tempRod;
		for (int i=0; i < playerCt; i++) {
			if (thePlayers[i] instanceof A_Rod) {
				totalRodCt++;
				tempRod = (A_Rod)thePlayers[i];
				if (tempRod.plusCapped) { cappedRodCt++; }
			}
			if (thePlayers[i] instanceof SuperStructure) {
				structureCt++;
			}
		}
		talkln ("Player Stats: #Rods = " + String.valueOf(totalRodCt) + " with " + String.valueOf(cappedRodCt) + " capped.  #Structures = " + String.valueOf(structureCt) + "  #mechDead = " + String.valueOf(numMechDead));
	}
	
	/*public static void setViscBlobDrags (double transGam, double rotGam) {
		double N = 1.0/Env.numVBlobEqualBug;
		viscBlobTransGam = new Pt3D(N*transGam,N*transGam,N*transGam);
		viscBlobRotGam = new Pt3D(N*rotGam,N*rotGam,N*rotGam);
	}*/
	
	public static void playerCleanup () {
		int j = 0;
		while (j < playerCt) {
			Player curP = thePlayers[j];
			if (curP.remove) {
				if (curP.structureMember) { curP.myStructure.removeMember(curP); }
				if (curP.structure) { SuperStructure.removeStructure((SuperStructure)curP); }
				removePlayer(curP);
			} else {
				j++;	// move on to next in list
			}
		}
	}
	
	public static void addPlayer (Player addMe) {
		synchronized (playerLock) {
			addMe.idnum = (short)playerCt;
			thePlayers[playerCt] = addMe;
			playerCt++;
		}
	}
	
	public static void removePlayer (Player rmMe) {
		synchronized (playerLock) {
			short swapId = rmMe.idnum;						// store swap location
			thePlayers[swapId] = thePlayers[playerCt-1];	// swap last in list into this spot
			thePlayers[swapId].idnum = swapId;				// give new Id
			rmMe.sepaku();									// null out all links in the player to remove
			playerCt += -1;									// decrement player counter
		}
	}
	
	public void collision (Pt3D forcePt, Pt3D forceVec) {
		colForceSum.inc(forceVec);
		colTorqueSum.inc(getTorque(forcePt,forceVec));
		colliderCt++;
		// if this player is part of a secondary structure then pass this force on...
		if (structureMember) { myStructure.collision(forcePt, forceVec); }
	}
	
	public void link (Pt3D linkPt, Pt3D linkForce) {
		linkForceSum.inc(linkForce);
		linkTorqueSum.inc(getTorque(linkPt,linkForce));
		linkerCt++;
		// if this player is part of a secondary structure then pass this force on...
		if (structureMember) { myStructure.link(linkPt, linkForce); }
	}
	
	public void frictionForce (Pt3D forcePt, Pt3D forceVec) {
		fricForceSum.inc(forceVec);
		fricTorqueSum.inc(getTorque(forcePt,forceVec));
		// if this player is part of a secondary structure then pass this force on...
		if (structureMember) { myStructure.frictionForce(forcePt, forceVec); }
	}
	
	
	public void sumForces () {
		forceSum.add(colForceSum, linkForceSum);
		torqueSum.add(colTorqueSum,linkTorqueSum);
		
		if (Env.useBugFriction) {
			forceSum.inc(fricForceSum);
			torqueSum.inc(fricTorqueSum);
		}
	}
	
	
	/*private void collisionYLinkAtten () {
		unAttenColForce.scale(Env.colFactor,colForceSum);
		unAttenLinkForce.scale(Env.linkFactor,linkForceSum);
		double colYLink;
		if (Env.attenuateLinks){
			colYLink = Pt3D.vecMag(Pt3D.Add(unAttenColForce, unAttenLinkForce));
		}else{
			colYLink = Pt3D.vecMag(unAttenColForce);
		}
		
		if (colYLink > getMaxForce()) {
			colAtten = getMaxForce()/colYLink;
			if(Env.attenuateLinks){
				linkAtten = colAtten;
			}else{linkAtten = 1;}
		} else {
			colAtten = 1;
			linkAtten = 1;
		}
		
		checkBuckling(colYLink);
		checkSnap(colYLink);
		
		colForceSum.zero();
		colTorqueSum.zero();
		colliderCt = 0;
		linkForceSum.zero();
		linkTorqueSum.zero();
		linkerCt = 0;
		linkerCtInA =0;
		linkerCtInB =0;
		linkerCtInC =0;
		linkerCtInD =0;
		colForceSumInA.zero();
		colForceSumInB.zero();
		colForceSumInC.zero();
		colForceSumInD.zero();
		linkForceSumInA.zero();
		linkForceSumInB.zero();
		linkForceSumInC.zero();
		linkForceSumInD.zero();
	} */
	
	/*private void collisionAtten () {
		double colMag = Pt3D.vecMag(colForceSum);
		if (colMag > getMaxForce()) {
			colAtten = getMaxForce()/colMag;
		} else {
			colAtten = 1;
		}
		colForceSum.zero();
		colTorqueSum.zero();
		colliderCt = 0;
	}*/
	
	/*private void linkerAtten () { 
		double linkMag = Pt3D.vecMag(linkForceSum);
		if (linkMag > getMaxForce()) {
			linkAtten = getMaxForce()/linkMag;
		} else {
			linkAtten = 1;
		}
		linkForceSum.zero();
		linkTorqueSum.zero();
		linkerCt = 0;
	}*/
	
	public void regionCollision (Pt3D forcePt, Pt3D forceVec) {
		Pt3D forcePtInx = Player.listeria.tox(forcePt);
		double xPos = forcePtInx.x;
		double xInc = Env.listeriaCylLength/3.0;
		if ((xPos >= -Env.listeriaCapRad) & (xPos <= 0)) { colForceSumInA.inc(forceVec); }
		else if ((xPos >= 0) & (xPos <= xInc)) { colForceSumInB.inc(forceVec); }
		else if ((xPos >= xInc) & (xPos <= 2*xInc)) { colForceSumInC.inc(forceVec); }
		else if ((xPos >= 2*xInc) & (xPos <= 3*xInc)) { colForceSumInD.inc(forceVec); }
		else if ((xPos >= 3*xInc) & (xPos < Env.listeriaCylLength + Env.listeriaCapRad)) { colForceSumInE.inc(forceVec); }
		
		// if this player is part of a secondary structure then pass this force on...
		if (structureMember) { myStructure.regionCollision(forcePt, forceVec); }
	}
	
	public void regionLink (Pt3D linkPt, Pt3D linkForce) {
		Pt3D linkPtInx = Player.listeria.tox(linkPt);
		double xPos = linkPtInx.x;
		double xInc = Env.listeriaCylLength/3.0;
		if ((xPos >= -Env.listeriaCapRad) & (xPos <= 0)) { linkForceSumInA.inc(linkForce);linkerCtInA++; }
		else if ((xPos >= 0) & (xPos <= xInc)) { linkForceSumInB.inc(linkForce);linkerCtInB++; }
		else if ((xPos >= xInc) & (xPos <= 2*xInc)) { linkForceSumInC.inc(linkForce);linkerCtInC++; }
		else if ((xPos >= 2*xInc) & (xPos <= 3*xInc)) { linkForceSumInD.inc(linkForce);linkerCtInD++; }
		else if ((xPos >= 3*xInc) & (xPos < Env.listeriaCylLength + Env.listeriaCapRad)) { linkForceSumInE.inc(linkForce);linkerCtInE++; }
		
		// if this player is part of a secondary structure then pass this force on...
		if (structureMember) { myStructure.regionLink(linkPt, linkForce); }
	}
	
	public Pt3D getTorque (Pt3D pt, Pt3D force) {
		// calculate vector from CM to force application point in meters
		// find torques as dRxF
		dR.sub(1e-6, pt, coord);
		return Pt3D.Cross(dR, force);
	}
	
	public void resetCounters() {
		lastLinkerCt = linkerCt;	//I think so that can make linkerCt = 0 but still have a linkerCt to report in ListInfo
		lastColliderCt = colliderCt;
		linkerCt = 0;
		colliderCt = 0;
		colForceSum.zero();
		colTorqueSum.zero();
		linkForceSum.zero();
		linkTorqueSum.zero();
		fricForceSum.zero();
		fricTorqueSum.zero();
		colForceSumInA.zero();
		colForceSumInB.zero();
		colForceSumInC.zero();
		colForceSumInD.zero();
		colForceSumInE.zero();
		linkForceSumInA.zero();
		linkForceSumInB.zero();
		linkForceSumInC.zero();
		linkForceSumInD.zero();
		linkForceSumInE.zero();
		lastLinkerCtInA = linkerCtInA;
		lastLinkerCtInB = linkerCtInB;
		lastLinkerCtInC = linkerCtInC;
		lastLinkerCtInD = linkerCtInD;
		lastLinkerCtInE = linkerCtInE;
		linkerCtInA =0;
		linkerCtInB =0;
		linkerCtInC =0;
		linkerCtInD =0;
		linkerCtInE =0;
	}
	
	public void transMat () {
		transXTox [0][0] = cosAng.x*cosAng.y;
		transXTox [0][1] = sinAng.x*cosAng.y;
		transXTox [0][2] = -sinAng.y;
		transXTox [1][0] = -sinAng.x*cosAng.z+cosAng.x*sinAng.y*sinAng.z;
		transXTox [1][1] = cosAng.x*cosAng.z+sinAng.x*sinAng.y*sinAng.z;
		transXTox [1][2] = cosAng.y*sinAng.z;
		transXTox [2][0] = sinAng.x*sinAng.z+cosAng.x*sinAng.y*cosAng.z;
		transXTox [2][1] = -cosAng.x*sinAng.z+sinAng.x*sinAng.y*cosAng.z;
		transXTox [2][2] = cosAng.y*cosAng.z;
		// the inverse transformation is found as the transpose of this matrix
		// since the direction cosine matrix is orthogonal.
		for (int i = 0; i < 3; i++) {
			for (int j = 0; j < 3; j++) {
				transxToX [i][j] = transXTox [j][i];
			}
		}
	}
	
	public void fixAngles () {
		// this method ensures that 	psi (angle.x) is between 0 and 2*PI
		//							theta (angle.y) is between -PI/2 and PI/2
		//							phi (angle.z) is between 0 and 2*PI
		double twoPI = 2*Math.PI;
		while (angle.x > twoPI) {angle.x += -twoPI; }
		while (angle.x < 0) {angle.x += twoPI; }
		while (angle.z > twoPI) {angle.z += -twoPI; }
		while (angle.z < 0) {angle.z += twoPI; }
		if (angle.y > Math.PI/2) {
			angle.y = Math.PI-angle.y;
			if (angle.x < Math.PI) {angle.x += Math.PI; } else {angle.x += -Math.PI; }
		}
		if (angle.y < -Math.PI/2) {
			angle.y = -Math.PI-angle.y;
			if (angle.x < Math.PI) {angle.x += Math.PI; } else {angle.x += -Math.PI; }
		} 
	}
	
	public void calcRandomForces () {
		if (Env.brownianJiggle) {
			// this method takes uniform deviates and finds random numbers with a 
			// Gaussian distribution of mean=0, variance=2Dt, as applies for diffusive
			// motion of a particle with diffusivity D.
			UCircRnd xVals, yVals, zVals;
			// get stored random value pairs in unit circle {v1,v2,rsq,facterm}
			xVals = Rdm.cRdm(this);
			yVals = Rdm.cRdm(this);
			zVals = Rdm.cRdm(this);
			// rearrange values into v1, v2, rsq, and facterm Pt3Ds
			v1.x = xVals.v1; v1.y = yVals.v1; v1.z = zVals.v1;
			v2.x = xVals.v2; v2.y = yVals.v2; v2.z = zVals.v2;
			rsq.x = xVals.rsq; rsq.y = yVals.rsq; rsq.z = zVals.rsq;
			facterm.x = xVals.facterm; facterm.y = yVals.facterm; facterm.z = zVals.facterm;
			// this part actually depends on the objects diffusion and drag coefficients
			tempPt.mult(bTransDiff, facterm);
			fac1.vecSqrt(tempPt);
			tempPt.mult(bRotDiff, facterm);
			fac2.vecSqrt(tempPt);
			randForces.mult(1/Env.deltaT, v1, fac1, bTransGam);
			randTorques.mult(1/Env.deltaT, v2, fac2, bRotGam);
		} else {
			randForces.zero();
			randTorques.zero();
		}
	}
	
	public static double LPfactor (double size) {
		// uses an equation that approximately fits Luby-Phelps data for size based decrease in
		// diffusivity in cytoplasm.  Size should be in m, but our equation uses nm.
		double sizeInNanos = size*1e3;
		double A = 1.8;
		double B = 2.0;
		double C = -0.2;
		double DOverD0 = A*Math.exp(-B*sizeInNanos/(sizeInNanos+B)) + C;
		return DOverD0;
	}
	
	public boolean memberOfPlayerList (int ckidnum, int [] theList, int numInList) {
		// this method checks if the player has already been added to the collider list
		int i = numInList-1;
		boolean alreadythere = false;
		while ((i >= 0) & (! alreadythere)) {
			if (ckidnum == theList[i]) {
				alreadythere = true;
			}
			i+= -1;
		}
		return alreadythere;
	}
	
	public boolean sameStructure (Player ckPlayer) {
		// this method checks if the players have the same actin branch or actin mesh
		if ((structureMember) & (ckPlayer.structureMember)) {
			if (myStructure == ckPlayer.myStructure) { return true; } else { return false; }
		} else {
			return false;
		}
		
	}
	
	public double getEffectiveDrag () {
		// this method is just a quick hack for some right magnitude answer
		if (structureMember) {
			return myStructure.getEffectiveDrag();
		} else {
			return bTransGam.x;
		}
	}
	
	public double getEffectiveDrag (Pt3D lineOfAction) {
		// this method is a little better than above... still doesn't consider rotation
		if (structureMember) {
			return myStructure.getEffectiveDrag(lineOfAction);
		} else {
			Pt3D lineOfActionInx = Pt3D.XToNewx(this,lineOfAction);
			double effDrag = Pt3D.PlusDot(lineOfActionInx, bTransGam);
			return effDrag;
		}
	}
	
	public void dragModifications (double actualLength, double effectiveLength) {
		if(Env.useOldDrag){
			//old method pre110105 when started to play with turning off fixation and doing adhesion only
//			 Increase drag coefficients, over shaped-based values, by scale factors ****
			if (Env.applyLubyPhelpsFactor) {
				double LPfact = LPfactor(effectiveLength);
				//double LPfact = LPfactor(actualLength);
				bTransGam.scale(1/LPfact);				//**
				bRotGam.scale(1/LPfact);				//**
			}
			
			//If want to try applyLubyPhelps for only filaments<10mon long:
			/*if ((Env.applyLubyPhelpsFactor) & (actualLength < (10*Env.lengthPerActin))) {
				double LPfact = LPfactor(actualLength);
				//double LPfact = LPfactor(effectiveLength);
				bTransGam.scale(1/LPfact);				//**
				bRotGam.scale(1/LPfact);				//**
			}*/
			
			// Increase drag coefficients according to "adhesions"... a hack ***
			if ((Env.applyAdhesions) & (adhesionCount > 0)) {
				bTransGam.scale(adhesionCount*Env.adhesionFactor);
				bRotGam.scale(adhesionCount*Env.adhesionFactor);
			}
			
			// age-based adhesion
			if (Env.ageBasedAdhesions) {
				bTransGam.scale(age*Env.ageAdhesionFactor+1);
				bRotGam.scale(age*Env.ageAdhesionFactor+1);
			}
			

			if (Env.lengthBasedAdhesions) {
				double scaleFac = effectiveLength*Env.lengthAdhesionFactor+1;
				//double scaleFac = actualLength*Env.lengthAdhesionFactor+1;
				bTransGam.scale(scaleFac);
				bRotGam.scale(scaleFac);
			}
			
			//viscous blob based adhesions
			if (Env.viscousBlobs) {
				bTransGam.add(bTransGam,numViscBlobs,Env.blobTransGam);
				bRotGam.add(bRotGam,numViscBlobs,Env.blobRotGam);
			}	
			
		}else if (!Env.useOldDrag){
//			 Increase drag coefficients, over shaped-based values, by scale factors ****
			if ((Env.applyLubyPhelpsFactor) & (actualLength < (10*Env.lengthPerActin))) {
				double LPfact = LPfactor(actualLength);
				bTransGam.scale(1/LPfact);				//**
				bRotGam.scale(1/LPfact);				//**
			}
			
			// Increase drag coefficients according to "adhesions"... a hack ***
			if ((Env.applyAdhesions) & (adhesionCount > 0)) {
				bTransGam.scale(adhesionCount*Env.adhesionFactor);
				bRotGam.scale(adhesionCount*Env.adhesionFactor);
			}
			
			// age-based adhesion
			if (Env.ageBasedAdhesions) {
				bTransGam.scale(age*Env.ageAdhesionFactor+1);
				bRotGam.scale(age*Env.ageAdhesionFactor+1);
			}
			

			if (Env.lengthBasedAdhesions) {
				double scaleFac = actualLength*Env.lengthAdhesionFactor+1;
				bTransGam.scale(scaleFac);
				bRotGam.scale(scaleFac);
			}
			
			if ((Env.lengthBasedAdhesionsByFunction) & (actualLength < (10*Env.lengthPerActin))) {	
				//do nothing
			}else if((Env.lengthBasedAdhesionsByFunction) & (actualLength >= (10*Env.lengthPerActin))) {
				double scaleFac = (1/Env.lengthPerActin) * effectiveLength;	//1/lengthPerActin such that it cancels the lengthPerActin in "effectiveLength"
				bTransGam.scale(scaleFac);
				bRotGam.scale(scaleFac);
			}
		}
	}
	
	private static int [] growCollisionArray (int [] colArray) {
		int [] newArray = new int [colArray.length + Env.collArrayStep];
		System.arraycopy(colArray, 0, newArray, 0, colArray.length);
		return newArray;
	}
	
	private static Pt3D [] growPt3DArray (Pt3D [] pt3DArray) {
		Pt3D [] newArray = new Pt3D [pt3DArray.length + Env.collArrayStep];
		System.arraycopy(pt3DArray, 0, newArray, 0, pt3DArray.length);
		return newArray;
	}
	
	public static void showAllPlayers (Graphics g, double scale, PinCam cam) {
		for (int i=0; i < playerCt; i++) {
			thePlayers[i].showYourself(g,scale,cam);
		}
	}
	
	public static void stringToScreen (Graphics g, String sOut, int xPix, int yPix) {
		g.setFont(new Font ("TimesRoman", Font.PLAIN, 12));
		g.drawString (sOut, xPix, yPix);
	}
	
	// **** For collision detection ****
	public static void lineSegmentIntersectTest (Pt3D pt1A, Pt3D pt1B, Pt3D pt2A, Pt3D pt2B, RetObj retO) {
		// this method implements an adaption of the "Faster Line Segment Intersection" technique of 
		// Franklin Antonio presented in "Graphics Gems III", ed David Kirk, IBM 1992 and "Intersection of 
		// Two Lines in Three-space" by Ronald Goldman in "Graphics Gems", 1990.
		// the points received define two line segments:  pt1A-pt1B and pt2A-pt2B
		// the object, RetObj, returned hold two Pt3D and one double
		double smallNum = 1e-10;
		retO.reset();
		
		retO.ray1.sub(pt1B,pt1A);
		retO.ray2.sub(pt2B,pt2A);
		retO.ray3.sub(pt2A,pt1A);
		retO.ray4.cross (retO.ray1,retO.ray2);
		if ((retO.ray4.x < smallNum) & (retO.ray4.y < smallNum) & (retO.ray4.z < smallNum)) { 	// change this criterion to < some small #
			return;	 			// then stop 'cause the segments are parallel
		} else {
			double denom = Pt3D.Dot (retO.ray4,retO.ray4);
			double alpha = Pt3D.Dot(retO.ray4, Pt3D.Cross(retO.ray3,retO.ray2))/denom;
			if ((alpha >= 0) & (alpha <= 1)) {
				double beta = Pt3D.Dot(retO.ray4, Pt3D.Cross(retO.ray3,retO.ray1))/denom;
				if ((beta >= 0) & (beta <= 1)) {
					// if we've gotten this far we only need to check that the lines aren't skew... i.e. is there
					// one distinct intersection point or do we have the two points of closest approach?
					retO.collision = true;
					retO.conPt1.add(pt1A, alpha, retO.ray1);
					retO.conPt2.add(pt2A, beta, retO.ray2);
					retO.conDist = Pt3D.ptDist (retO.conPt1,retO.conPt2);
				}
			}
		}		
	}
	
	
	public static void pointAndLineIntersectTest (Pt3D point, Pt3D ptA, Pt3D ptB, RetObj retO) {
		// Point and Line Segment Intersection test... 
		// see derivation of the following formulae in work book... uses dot product as zero to enforce
		// the perpendicularity and parameterization of line segment to check if the perpendicular drop
		// from sphere to line is on the line segment.
		// A line segment is {x1,y1,z1,x2,y2,z2}
		// A point is {x,y,z}
		retO.reset();
		
		retO.ray1.sub(ptB,ptA);
		retO.ray2.sub(point,ptA);
		double numer = Pt3D.Dot(retO.ray2,retO.ray1);
		double denom = Pt3D.vecMagSqrd(retO.ray1);
		double alpha = numer/denom;
		if ((alpha <= 1) & (alpha >= 0)) {	// the perpendicular projection is on the line segment...  then check distance
			retO.collision = true;
			retO.conPt1.add(ptA, alpha,retO.ray1);		// define perpendicular point
			retO.conDist = Pt3D.ptDist (retO.conPt1,point);
		}
	}
	
	public static void pointAndLMCylSlowWay (Pt3D point, Listeria LM, RetObj retO) {
		retO.reset();
		
		retO.ray1.sub(point,LM.rec1);
		retO.ray1.XTox(LM);
		if ((retO.ray1.x > 0) & (retO.ray1.x < Listeria.cylLength)) {
			retO.collision = true;
			retO.conPt1.add(LM.rec1,retO.ray1.x,LM.uVec);
			retO.conDist = Pt3D.ptDist(retO.conPt1,point);
		}
	}
			
	public static void talkln (String info) {
		FileOps.talkln(info);
	}
	
	public static void talk (String info) {
		FileOps.talk(info);
	}
	
}