package rocketBugs;
/*
	Player Tools..... some procedures to manipulate and maintain structures of players such as membranes and filaments
	and also to create instances of players.
*/

/*  <RocketBugs - for the simulation of actin-based Listeria motility>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.Math;
import java.awt.*;

public class PlayerTools {
	static RocketBugs MC;
	static Rdm rdm;
	static Player [] thePlayers;
	static A_Node [] theMembrane;
	
	public PlayerTools (RocketBugs MC) {
		this.MC = MC;
		rdm = MC.rdm;
		thePlayers = Player.thePlayers;
		//theMembrane = MC.theMembrane;
	}
	
	public Player findClosestPlayer (Pt3D point, boolean twoD) {
		talkln ("In 'findClosestPlayer'");
		// find the player closest to the mouse click
		Player tweezedPlayer = null;
		Player curPlayer = null;
		double mindist = 1.0;
		double playerdist = mindist;
		for (int i = 0; i < Player.playerCt; i++) {
			curPlayer = thePlayers[i];
			if (!twoD) {
				playerdist = Pt3D.ptDist(curPlayer.coord,point);
			} else {
				playerdist = Math.sqrt(Math.pow(curPlayer.coord.x-point.x,2)+Math.pow(curPlayer.coord.y-point.y,2));
			}
			if (playerdist < mindist) {
				tweezedPlayer = curPlayer;
				mindist = playerdist;
			}
		}
		talk ("The closest player is ");
		if (tweezedPlayer == null) {talkln ("null");} else {tweezedPlayer.reportStats();}
		return tweezedPlayer;
	}
	
	public void sortCheck (Player [] Ps, int numPs, double [] trigAng) {
		int swapNum;
		for (int i = 0; i < numPs-1; i++) {
			if ((Ps[i+1].coord.z*trigAng[1] + Ps[i+1].coord.x*trigAng[0]) > (Ps[i].coord.z*trigAng[1] + Ps[i].coord.x*trigAng[0])) {
				sortPlayersByRocketBugsZ (Ps, numPs, trigAng);
			}
		}
	}
	
	public void sortPlayersByRocketBugsZ(Player [] Ps, int numPs, double [] trigAng) {
		Player swapP;
		for (int i = 0; i < numPs-1; i++) {
			for (int j = i+1; j < numPs; j++) {
				if ((Ps[j].coord.z*trigAng[1]+Ps[j].coord.x*trigAng[0]) > (Ps[i].coord.z*trigAng[1]+Ps[i].coord.x*trigAng[0])) {
					swapP = Ps[j];
					Ps[j] = Ps[i];
					Ps[j].idnum = (short)j;
					Ps[i] = swapP;
					Ps[i].idnum = (short)i;
				}
			}
		}
	}
	
	public double calculateAreaInMembrane () {
		double totalArea = 0;
	/*	A_Node firstNode,curNode,nextNode;
		firstNode = theMembrane[0];
		curNode = theMembrane[0];
		nextNode = null;
		while (nextNode != firstNode) {
			nextNode = curNode.linker2;
			totalArea += (nextNode.ypos-curNode.ypos)*(curNode.xpos+nextNode.xpos)/2;
			curNode = nextNode;
		}
	*/ 	return totalArea; 
	}
			
	public void makeRandomBlob (Pt3D mins, Pt3D maxs) {
		Pt3D randomPt = Pt3D.Random();
		Pt3D newCoord = Pt3D.Add(Pt3D.Mult(randomPt, Pt3D.Sub(maxs,mins)),mins);
		double newRad = Env.mtRNG.raw()*(A_Blob.maxRad-A_Blob.minRad)+A_Blob.minRad;
		//double newRad = A_Blob.maxRad;
		synchronized (Player.playerLock) { 	// need to sync to avoid id conflicts
			A_Blob newBlob = new A_Blob(newCoord,newRad,Player.playerCt);
			int redComp = (int)(rdm.uRdm()*255);
			int greenComp = (int)(rdm.uRdm()*255);
			int blueComp = (int)(rdm.uRdm()*255);
			//newBlob.playerColor = new Color(redComp,greenComp,blueComp);
		}
	}
	
	public void makeRandomRod (Pt3D mins, Pt3D maxs, int numOfMonomers) {
		Pt3D randomPt = new Pt3D(Rdm.uRdm(), Rdm.uRdm(), Rdm.uRdm());
		Pt3D newCoord = Pt3D.Add(Pt3D.Mult(randomPt, Pt3D.Sub(maxs,mins)),mins);
		Pt3D newUVec = Pt3D.RandomUnitVec();
		A_Rod.makeNewRod(newCoord, newUVec, numOfMonomers);
	}
	

	public static void removeRod (A_Rod rmMe) {
		if (rmMe.branches > 0) { talkln (" ***ERROR.. removing a rod with branches"); }
		//talk ("**** removing a stubby uncapped filament");
		Player.playerCt += -1;
		if (rmMe.idnum != Player.playerCt) {
			thePlayers[rmMe.idnum] = thePlayers[Player.playerCt];
			thePlayers[rmMe.idnum].idnum = rmMe.idnum;
		}
		thePlayers[Player.playerCt] = null;
		rmMe = null;
	}
		
		

	public void makeMembraneNodes () {
	/*	double numberOfNodes = 1200;
		double initLipidNum = 400.0;
		double x = 10;
		double y = 2;
		double ang = 0;
		A_Node firstNode = null;
		A_Node lastNode = null;
		boolean red = true;
		for (int i = 0; i < numberOfNodes; i++) {
			A_Node newMemNode = new A_Node (coord,initLipidNum,MC.numberOfMembraneSegs,Player.playerCt);
			if (red) { 
				newMemNode.playerColor = Color.red;
				red = false;
			} else {
				red = true;
			}
			if (i == 0) { 
				firstNode = newMemNode; 
				lastNode = newMemNode;
			} else {
				newMemNode.linker1 = lastNode;
				lastNode.linker2 = newMemNode;
			}
			theMembrane[MC.numberOfMembraneSegs] = newMemNode;
			MC.numberOfMembraneSegs++;
			x += initLipidNum*newMemNode.lengthPerLipid*Math.cos(ang);
			y += initLipidNum*newMemNode.lengthPerLipid*Math.sin(ang);
			ang +=  (double) (2*Math.PI/(numberOfNodes));
			lastNode = newMemNode;
		}
		firstNode.linker1 = lastNode;
		lastNode.linker2 = firstNode;
		for (int i = 0; i < numberOfNodes; i++) {
			theMembrane[i].initialize();
		}		*/
	}
	
	public void membraneMaintenance () {
	/*	A_Node node,newNode,assimNode;
		double xinit,yinit;
		int i = 0;
		while (theMembrane[i] != null) {
			node = theMembrane[i];
			if (node.curLength > A_Node.maxLength) {
				xinit = (node.xpos + node.linker2.xpos)/2;
				yinit = (node.ypos + node.linker2.ypos)/2;
				newNode = new A_Node (xinit,yinit,node.lipidCount/2,MC.numberOfMembraneSegs,Player.playerCt);
				newNode.playerColor = new Color(125,200,255);
				node.lipidCount = node.lipidCount/2;
				newNode.linker2 = node.linker2;
				node.linker2.linker1 = newNode;
				newNode.linker1 = node;
				node.linker2 = newNode;
				theMembrane[MC.numberOfMembraneSegs] = newNode;
				MC.numberOfMembraneSegs++;
				node.initialize();
				newNode.initialize();
			} else {
				if (node.curLength < A_Node.minLength) {
					assimNode = node.linker1;
					assimNode.lipidCount += node.lipidCount;
					assimNode.linker2 = node.linker2;
					node.linker2.linker1 = assimNode;
					assimNode.initialize();
					// switch node out of lists....replace with last member of list
					int replaceLoc = node.memIdnum;
					MC.numberOfMembraneSegs += -1;
					theMembrane[replaceLoc] = theMembrane[MC.numberOfMembraneSegs];
					theMembrane[replaceLoc].memIdnum = replaceLoc;
					theMembrane[MC.numberOfMembraneSegs] = null;
					replaceLoc = node.idnum;
					Player.playerCt += -1;
					thePlayers[replaceLoc] = thePlayers[Player.playerCt];
					thePlayers[replaceLoc].idnum = replaceLoc;
					thePlayers[Player.playerCt] = null;
					i += -1;				
				}
			}
			i++;
		} */
	}
	
	public void superStructureMaintenance () {
		for (int i=0; i < SuperStructure.structureCt; i++) {
			SuperStructure.structures[i].calculateProperties ();
		}
	}
							
	
	private static void talkln (String info) {
		System.out.println(info);
	}
	
	private static void talk (String info) {
		System.out.print(info);
	}
}











