package rocketBugs;
/*
	Main_Frame.... the main GUI frame
*/

/*  <RocketBugs - for the simulation of actin-based Listeria motility>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import infoCCD.Info;
import infoCCD.ResourceGetter;

import java.awt.*;
import java.awt.event.*;
import java.lang.Math;
import javax.swing.*;
import java.text.*;

public class RBFrame extends JFrame implements ActionListener,KeyListener,ComponentListener,MouseListener, MouseMotionListener,WindowListener {
	static RocketBugs MC;
	static PlayerHome playerHome;
	static ZoomFrame zoomFrame;
	static StereoFrame stereoFrame;
	static Player [] thePlayers;
	static SphImpPlayers sphImp;
	boolean tweezersOn = false;
	boolean zoomOn = false;
	boolean stereoOn = false;
	boolean paintOn = true;
	JToggleButton runButton,paintButton;
	JButton jiggleButton;
	JPopupMenu fileOpsMenu;
	JLabel playerNum, simTime;
	JPanel southPanel = new JPanel();
	JPanel eastPanel = new JPanel();
	static ResourceGetter rG = new ResourceGetter(Info.runType);

	// Cursors
	Cursor normalCursor = new Cursor(DEFAULT_CURSOR);
	Cursor handCursor = new Cursor(HAND_CURSOR);
	Cursor crossCursor = new Cursor(CROSSHAIR_CURSOR);
	Cursor waitCursor = new Cursor(WAIT_CURSOR);
	
	int sidePanWidth = 35;
	int botPanHeight = 45;
	Dimension initFrameSize = new Dimension (500+sidePanWidth,500+botPanHeight);
	double objInset = 5;
	
	Player tweezedPlayer = null;
	Color tweezedPlayerOriginalColor;
	
	DecimalFormat timeOutFormat = new DecimalFormat ("#0.00#; #0.00#");
	
	public RBFrame (Player [] thePlayers, RocketBugs MC) {
		this.MC = MC;
		this.thePlayers = thePlayers;
		sphImp = MC.sphImp;
		
		//make the InfoCCD functionality
		new Info(true,true,false);
		makeMenuBar();
		
		this.setTitle ("*&@%# Comet Tails");
		this.setSize (initFrameSize);
		this.setVisible(true);
		
		// create the place where the players will live
		playerHome = new PlayerHome (initFrameSize.width-sidePanWidth,initFrameSize.height-botPanHeight, this);
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add (playerHome, BorderLayout.CENTER);
		playerHome.initialize();
		
		// create instance of zoom frame
		zoomFrame = new ZoomFrame (new double [] {Env.dimensions.x/2, Env.dimensions.y/2,0},10,this);
		addKeyListener(zoomFrame);
		
		// create instance of stereo frame
		stereoFrame = new StereoFrame (new double [] {Env.dimensions.x/2, Env.dimensions.y/2,0},10,this);
		
		// make the button panels
		makeButtonPanels ();
		makeMenuPanel ();
		
		addKeyListener(this);
		addComponentListener(this);
		addMouseListener(this);
		addMouseMotionListener(this);
		addWindowListener(this);
		this.setVisible(true);
		this.requestFocus();
		
	}
	
	public void showAll() {
		if (paintOn) {
			playerHome.clearImage();
			playerHome.showPlayers();
			if (Env.showLinkers) { playerHome.showLinkers(); }		
			playerHome.repaint();
			if (zoomOn) {
				if (Env.rotateRocketBugsAngleC) { zoomFrame.rotateRocketBugsAngleC(); }
				if (Env.rotateRocketBugsAngleCC) { zoomFrame.rotateRocketBugsAngleCC(); }
				if (Env.chgZIn) { zoomFrame.zoomIn(); }
				if (Env.chgZOut) { zoomFrame.zoomOut(); }
				playerHome.showRocketBugsArea (zoomFrame);
				if (Env.epsOut) { 
					boolean doEps = zoomFrame.zoomPanel.openEpsFile(); 
					zoomFrame.showRocketBugs(sphImp);
					if (doEps) {zoomFrame.zoomPanel.closeEpsFile(); }
				} else {
					zoomFrame.showRocketBugs(sphImp);
				}
			}
			if (stereoOn) {
				stereoFrame.showStereo (thePlayers, Player.playerCt, sphImp);
			}
		}
	}
	
	public void updateDisplay(double simulationTime, int playerCt) {
		simTime.setText(" Sim Time = " + String.valueOf(timeOutFormat.format(simulationTime)));
		playerNum.setText(" # Players = " + String.valueOf(playerCt));
	}
	
	public void turnRocketBugsOn () {
		MC.waitOnTimeLoop();
		zoomOn = true;
		zoomFrame.initialize();
		turnTweezersOff();
		MC.notifyTimeLoop();
	}
	
	public void turnRocketBugsOff () {
		zoomOn = false;
		turnTweezersOff();
		setCursor (normalCursor);
		zoomFrame.setCursor (normalCursor);
	}
	
	public void turnStereoOn () {
		MC.waitOnTimeLoop();
		stereoOn = true;
		stereoFrame.initialize();
		turnTweezersOff();
		MC.notifyTimeLoop();
	}
	
	public void turnStereoOff () {
		stereoOn = false;
		turnTweezersOff();
		setCursor (normalCursor);
		stereoFrame.setCursor (normalCursor);
	}
	
	public void turnTweezersOn () {
		tweezersOn = true;
		setCursor (handCursor);
		zoomFrame.setCursor (handCursor);
	}
	
	public void turnTweezersOff () {
		tweezersOn = false;
		if (tweezedPlayer != null) {
			//tweezedPlayer.playerColor = tweezedPlayerOriginalColor;
			tweezedPlayer.selected = false;
			tweezedPlayer = null;
		}
		if (zoomOn) {
			setCursor (crossCursor);
			zoomFrame.setCursor (crossCursor); 
		} else {
			setCursor (normalCursor);
			zoomFrame.setCursor (normalCursor);
		}
	}
	
	public void actionPerformed (ActionEvent e) {
		Pt3D minVec = new Pt3D(1,1,1);
		String act = e.getActionCommand();
		if (act.equals("Add Another Blob")) { MC.playerTools.makeRandomBlob(minVec, Env.dimensions);}
		if (act.equals("Add 20 Blobs")) { for (int i = 0; i != 20; i++) { MC.playerTools.makeRandomBlob(minVec, Env.dimensions); }}
		if (act.equals("Add 100 Blobs")) { for (int i = 0; i != 100; i++) { MC.playerTools.makeRandomBlob(minVec, Env.dimensions); }}
		if (act.equals("Add Another Rod")) { MC.playerTools.makeRandomRod(minVec,Env.dimensions,A_Rod.minMonomers);}
		if (act.equals("Add 20 Rods")) { for (int i = 0; i != 20; i++) { MC.playerTools.makeRandomRod(minVec,Env.dimensions,200);}}//A_Rod.minMonomers); }}
		if (act.equals("Add 100 Rods")) { for (int i = 0; i != 100; i++) { MC.playerTools.makeRandomRod(minVec,Env.dimensions,A_Rod.minMonomers); }}
		if (act.equals("RocketBugs")) { turnRocketBugsOn(); }
		if (act.equals("Stereo View")) { turnStereoOn(); }
		if (act.equals("Start Running")) {
			MC.paused = false;
			runButton.setActionCommand ("Pause Running");
			runButton.setToolTipText ("Pause the Simulation");
		}
		if (act.equals("Pause Running")) {
			MC.paused = true;
			runButton.setActionCommand ("Start Running");
			runButton.setToolTipText ("Start the Simulation");
		}
		if (act.equals("Paint/No Paint")) { if (paintOn) { paintOn = false; } else { paintOn = true; }}
		if (act.equals("Tweezers")) {
			if (!tweezersOn) { turnTweezersOn (); } else { turnTweezersOff(); }
		}
		if (act.equals("Jiggle")) {
			if (Env.brownianJiggle) {
				Env.brownianJiggle = false;
			} else {
				Env.brownianJiggle = true;
			}
		}
		if (act.equals("Take Snapshot")) {
			MC.waitOnTimeLoop(); 
			showAll();
		 	MC.fileOps.takeRawPicture("SnapShot.raw");
		 	MC.notifyTimeLoop();
		 }
		if (act.equals("Make Movie")) { NameFrame nameFrame = new NameFrame(MC); }
		
		if (act.equals("Clear")) { 
			MC.waitOnTimeLoop();
			Player.playerCt = 0;
			MC.notifyTimeLoop();
		}
		if (act.equals("Save Configuration")) { 
			setCursor (waitCursor);
			MC.fileOps.saveConfig(); 
			if (zoomOn) { setCursor(crossCursor); } else { setCursor (normalCursor); }
		}
		if (act.equals("Load Configuration")) { 
			setCursor (waitCursor);
			MC.fileOps.loadConfig(false); 
			if (zoomOn) { setCursor(crossCursor); } else { setCursor (normalCursor); }
		}
	}
	
	public void windowActivated (WindowEvent e) {}
	public void windowClosing (WindowEvent e) { System.exit(0); }
	public void windowDeactivated (WindowEvent e) {}
	public void windowDeiconified (WindowEvent e) {}
	public void windowIconified (WindowEvent e) {}
	public void windowOpened (WindowEvent e) {}
	public void windowClosed (WindowEvent e) {}
	
	public void keyReleased (KeyEvent e) {}
	public void keyTyped (KeyEvent e) {}
	public void keyPressed (KeyEvent e) {
		String act = e.getKeyText(e.getKeyCode());
		if (act.equals("I")) {
			if (Env.courseToShow > 0) { Env.courseToShow += -1; }
		}
		if (act.equals("O")) {
			if (Env.courseToShow < sphImp.rDim-1) { Env.courseToShow += 1; }
		}
		if (act.equals("M")) {
			sphImp.ambientC[Env.ARPa] += 1.0;
			talkln ("Ambient active ARP2/3 concentration is " + String.valueOf(sphImp.ambientC[Env.ARPa]));
		}
		if (act.equals("L")) {
			sphImp.ambientC[Env.ARPa] += -1.0;
			if (sphImp.ambientC[Env.ARPa] < 0) { sphImp.ambientC[Env.ARPa] = 0; }
			talkln ("Ambient active ARP2/3 concentration is " + String.valueOf(sphImp.ambientC[Env.ARPa]));
		}
	}
	
	public void mouseClicked (MouseEvent e) {
		Pt3D point = new Pt3D(0,0,-1);
		if ((tweezersOn) & (! zoomFrame.chgAng) & (! zoomFrame.chgZ)) {
			if (tweezedPlayer != null) {
				//tweezedPlayer.playerColor = tweezedPlayerOriginalColor;		// unselect and uncolor current tweezedPlayer
				tweezedPlayer.selected = false;
				tweezedPlayer = null;
			}
			MC.waitOnTimeLoop();
			if (e.getComponent() == zoomFrame) {
				point = zoomFrame.zoomPanel.convertPixelToPoint(e.getX(),e.getY());
				tweezedPlayer = MC.playerTools.findClosestPlayer (point, true);	// use flag for closest point in 3D
			} else {
				point.x = (double) (e.getX()/playerHome.scale);
				point.y = (double) (e.getY()/playerHome.scale);
				tweezedPlayer = MC.playerTools.findClosestPlayer (point, true);		// flagged for 2D distance... closest thing you see in plane
			}
			
			if (tweezedPlayer != null) {
				//tweezedPlayerOriginalColor = tweezedPlayer.playerColor;
				//tweezedPlayer.playerColor = new Color(200,175,255);
				tweezedPlayer.selected = true;
				if (stereoOn) { stereoFrame.newFocus(tweezedPlayer); }
				tweezedPlayer.reportStats();
			}
			MC.notifyTimeLoop();
		}
		if ((zoomOn) & (!tweezersOn) & (! zoomFrame.chgAng) & (! zoomFrame.chgZ)) {
			MC.waitOnTimeLoop();
			if (e.getComponent() == zoomFrame) {
				point = zoomFrame.zoomPanel.convertPixelToPoint(e.getX(),e.getY());
			} else {
				point.x = (double) (e.getX()/playerHome.scale);
				point.y = (double) (e.getY()/playerHome.scale);
				point.z = zoomFrame.zoomPanel.zoomCen.z;	//don't change z
				zoomFrame.resetRocketBugsAngle();	// if clicked in playerHome reset zoom angle to zero
			}
			zoomFrame.newRocketBugsCenter(point);
			MC.notifyTimeLoop();
		}
	}
	public void mouseEntered (MouseEvent e) {}
	public void mouseExited (MouseEvent e) {}
	public void mousePressed (MouseEvent e) {
		if (e.isPopupTrigger()) {
			fileOpsMenu.show(this,e.getX(),e.getY());
		}
	}
	public void mouseReleased (MouseEvent e) {}
	public void mouseDragged (MouseEvent e) {
		if ((zoomOn) & (zoomFrame.chgAng)) {
			if (zoomFrame.firstDrag) {
				zoomFrame.zoomPanel.setCursorPos (e.getX(), e.getY());
				zoomFrame.firstDrag = false;
			}
			zoomFrame.zoomPanel.chgRocketBugsAng (e.getX(), e.getY());
			zoomFrame.zoomPanel.setCursorPos (e.getX(), e.getY());
		}
		if ((zoomOn) & (zoomFrame.chgZ)) {
			if (zoomFrame.firstDrag) {
				zoomFrame.zoomPanel.setCursorPos (e.getX(), e.getY());
				zoomFrame.firstDrag = false;
			}
			zoomFrame.zoomPanel.chgRocketBugsZ (e.getX(), e.getY());
			zoomFrame.zoomPanel.setCursorPos (e.getX(), e.getY());
		}
		if ((tweezersOn) & (tweezedPlayer != null)) {
			Pt3D point = new Pt3D(0,0,tweezedPlayer.coord.z);
			if (e.getComponent() == zoomFrame) {
				point = zoomFrame.zoomPanel.convertPixelToPoint(e.getX(),e.getY());
			} else {
				point.x = (double) (e.getX()/playerHome.scale);
				point.y = (double) (e.getY()/playerHome.scale);
			}
			if (tweezedPlayer.structureMember) {
				tweezedPlayer.myStructure.tweezedMove(point);
			} else {
				tweezedPlayer.coord= point;
				tweezedPlayer.initialize();
			}
		}
	}
	public void mouseMoved (MouseEvent e) {}
	
	public void componentHidden (ComponentEvent e) {}
	public void componentShown (ComponentEvent e) {}
	public void componentMoved (ComponentEvent e) {}
	public void componentResized (ComponentEvent e) {
		MC.waitOnTimeLoop();
		playerHome = new PlayerHome (this.getSize().width-sidePanWidth,this.getSize().height-botPanHeight,this);
		getContentPane().add (playerHome, BorderLayout.CENTER);
		playerHome.initialize();
		paintComponents(getGraphics());
		MC.notifyTimeLoop();
	}
	
	public void makeMenuBar () {
		MenuBar theMenuBar = new MenuBar();
		this.setMenuBar( theMenuBar );

		theMenuBar.add(Info.infoMenu);
		
	}
	
	public void makeButtonPanels () {
		Font buttonFont = new Font ("TimesRoman", Font.PLAIN, 10);
		// The South Button Panel
		JButton newBlobButton = new JButton("Add Another Blob");  // Button to add more blobs
		newBlobButton.setFont(buttonFont);
		JButton twentyBlobsButton = new JButton("Add 20 Blobs");  	// Button to add 20 more blobs
		twentyBlobsButton.setFont(buttonFont);
		JButton hundredBlobsButton = new JButton("Add 100 Blobs"); // 100 more blobs
		hundredBlobsButton.setFont(buttonFont);
		JButton newRodButton = new JButton ("Add Another Rod"); // Add a rod
		newRodButton.setFont(buttonFont);
		JButton twentyRodsButton = new JButton("Add 20 Rods");
		twentyRodsButton.setFont(buttonFont);
		JButton hundredRodsButton = new JButton("Add 100 Rods");
		hundredRodsButton.setFont(buttonFont);
		newBlobButton.addActionListener(this);
		twentyBlobsButton.addActionListener(this);
		hundredBlobsButton.addActionListener(this);
		newRodButton.addActionListener(this);
		twentyRodsButton.addActionListener(this);
		hundredRodsButton.addActionListener(this);
		playerNum = new JLabel();						// Textfield to show number of current players
		playerNum.setForeground(Color.white);
		playerNum.setBackground(Color.black);
		playerNum.setText(String.valueOf(Player.playerCt));
		simTime = new JLabel();							// Textfield to show simulation time value
		simTime.setForeground(Color.white);
		simTime.setBackground(Color.black);
		simTime.setText(String.valueOf(Env.simulationTime));
		
		southPanel.setLayout(new GridLayout (2,4));
		southPanel.add(newBlobButton);
		southPanel.add(twentyBlobsButton);
		//southPanel.add(hundredBlobsButton);
		southPanel.add(playerNum);
		//southPanel.add(newRodButton);
		southPanel.add(newRodButton);
		southPanel.add(twentyRodsButton);
		//southPanel.add(hundredRodsButton);
		southPanel.add(simTime);
		southPanel.setBackground(Color.black);
		southPanel.setForeground(Color.lightGray);
		getContentPane().add(southPanel, BorderLayout.SOUTH);
		
		// The East Button Panel
		Icon tweezIcon = new ImageIcon("Pics/fingerpt.gif");
		JButton tweezButton = new JButton (tweezIcon);
		tweezButton.setActionCommand ("Tweezers");
		tweezButton.setToolTipText ("Tweezers!");
		tweezButton.addActionListener (this);
		
		Icon zoomIcon = new ImageIcon("Pics/micro2.gif");		// zoom button
		JButton zoomButton = new JButton (zoomIcon);
		zoomButton.setActionCommand("RocketBugs");
		zoomButton.setToolTipText("RocketBugs In!");
		zoomButton.addActionListener(this);
		
		Icon movieIcon = new ImageIcon(rG.getResource("Pics/mvcam.gif"));		// make movie button
		JButton movieButton = new JButton (movieIcon);
		movieButton.setActionCommand("Make Movie");
		movieButton.setToolTipText("Make A Movie");
		movieButton.addActionListener(this);
		
		Icon snapshotIcon = new ImageIcon(rG.getResource("Pics/camera.gif"));
		JButton snapshotButton = new JButton (snapshotIcon);
		snapshotButton.setActionCommand("Take Snapshot");
		snapshotButton.setToolTipText("Take A Snapshot");
		snapshotButton.addActionListener(this);
		
		Icon stereoIcon = new ImageIcon(rG.getResource("Pics/eyes.gif"));
		JButton stereoButton = new JButton (stereoIcon);
		stereoButton.setActionCommand("Stereo View");
		stereoButton.setToolTipText("Stereo (3D) View");
		stereoButton.addActionListener(this);
		
		Icon startIcon = new ImageIcon(rG.getResource("Pics/grnball.gif"));
		Icon pauseIcon = new ImageIcon(rG.getResource("Pics/redball.gif"));
		JToggleButton runButton = new JToggleButton (pauseIcon);
		this.runButton = runButton;
		runButton.setSelectedIcon (startIcon);
		runButton.setActionCommand ("Start Running");
		runButton.setToolTipText("Start the Simulation");
		runButton.addActionListener(this);
		
		Icon paintIcon = new ImageIcon(rG.getResource("Pics/paint.gif"));
		Icon nopaintIcon = new ImageIcon (rG.getResource("Pics/nopaint.gif"));
		JToggleButton paintButton = new JToggleButton (paintIcon);
		this.paintButton = paintButton;
		paintButton.setSelectedIcon (nopaintIcon);
		paintButton.setActionCommand ("Paint/No Paint");
		paintButton.setToolTipText ("Paint/No Paint");
		paintButton.addActionListener(this);
		
		Icon jiggleIcon = new ImageIcon(rG.getResource("Pics/jiggle.gif"));
		JButton jiggleButton = new JButton (jiggleIcon);
		this.jiggleButton = jiggleButton;
		jiggleButton.setActionCommand ("Jiggle");
		jiggleButton.setToolTipText ("Brownian Motion");
		jiggleButton.addActionListener(this);
		
		GridBagLayout eastLay = new GridBagLayout ();
		GridBagConstraints eastLayCons = new GridBagConstraints();
		eastPanel.setLayout(eastLay);
		eastLayCons.ipadx = 14;
		eastLayCons.ipady = 10;
		eastLayCons.gridx = 0;
		eastLayCons.gridy = 0;
		eastLay.setConstraints(tweezButton,eastLayCons);
		eastPanel.add(tweezButton);
		eastLayCons.ipadx = 2;
		eastLayCons.ipady = -5;
		eastLayCons.gridx = 0;
		eastLayCons.gridy = 1;
		eastLay.setConstraints(zoomButton,eastLayCons);
		eastPanel.add(zoomButton);
		eastLayCons.ipadx = 6;
		eastLayCons.ipady = 12;
		eastLayCons.gridx = 0;
		eastLayCons.gridy = 2;
		eastLay.setConstraints(stereoButton,eastLayCons);
		eastPanel.add(stereoButton);
		eastLayCons.ipadx = -4;
		eastLayCons.ipady = 4;
		eastLayCons.gridx = 0;
		eastLayCons.gridy = 3;
		eastLay.setConstraints(movieButton,eastLayCons);
		eastPanel.add(movieButton);
		eastLayCons.ipadx = -2;
		eastLayCons.ipady = 5;
		eastLayCons.gridx = 0;
		eastLayCons.gridy = 4;
		eastLay.setConstraints(snapshotButton,eastLayCons);
		eastPanel.add(snapshotButton);
		eastLayCons.ipadx = -2;
		eastLayCons.ipady = -2;
		eastLayCons.gridx = 0;
		eastLayCons.gridy = 5;
		eastLay.setConstraints(paintButton,eastLayCons);
		eastPanel.add(paintButton);
		eastLayCons.ipadx = 14;
		eastLayCons.ipady = 10;
		eastLayCons.gridx = 0;
		eastLayCons.gridy = 6;
		eastLay.setConstraints(runButton,eastLayCons);
		eastPanel.add(runButton);
		eastLayCons.ipadx = 10;
		eastLayCons.ipady = 8;
		eastLayCons.gridx = 0;
		eastLayCons.gridy = 7;
		eastLay.setConstraints(jiggleButton,eastLayCons);
		eastPanel.add(jiggleButton);
		
		getContentPane().add(eastPanel,"East");
	}
	
	public void makeMenuPanel () {
		fileOpsMenu = new JPopupMenu();
		fileOpsMenu.setInvoker(this);
		fileOpsMenu.add ("**File Operations**");
		JMenuItem newConfig = new JMenuItem("Clear");
		newConfig.addActionListener(this);
		JMenuItem loadConfig = new JMenuItem("Load Configuration");
		loadConfig.addActionListener(this);
		JMenuItem saveConfig = new JMenuItem("Save Configuration");
		saveConfig.addActionListener(this);
		fileOpsMenu.add(newConfig);
		fileOpsMenu.add(loadConfig);
		fileOpsMenu.add(saveConfig);
	}
	
	private void talkln (String info) {
		System.out.println(info);
	}
}