package rocketBugs;
/* Rdm class.... this class is just storage and creation of a bunch of random numbers.  
Create these arrays once at the beginning of a run and then use them throughout
*/

/*  <RocketBugs - for the simulation of actin-based Listeria motility>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.Math;
import java.io.*;

public class Rdm {
	static double deltaT;
	static int arraySize;
	static double [] uniformRnd;
	static UCircRnd [] unitCircleRnd;
	static int dCtr = 0;
	static int cCtr = 0;
	static int uRdmCt = 0;
	static int cRdmCt = 0;
	
	
	public Rdm (int arraySize, int numCounters, double deltaT) {
		this.deltaT = deltaT;
		this.arraySize = arraySize;
		uniformRnd = new double [arraySize];
		unitCircleRnd = new UCircRnd [arraySize];
		mkUniformArray();
		mkUnitCircleArray();
	}
	
	public static void ckURdmRefresh() {
		if (uRdmCt > Env.rdmNumUse*Env.numOfRdmNumbers) {
			mkUniformArray();
			uRdmCt = 0;
		}
	}
	
	public static void ckCRdmRefresh() {
		if (cRdmCt > Env.rdmNumUse*Env.numOfRdmNumbers) {
			refreshUnitCircleArray();
			cRdmCt = 0;
		}
	}
			
	
	private static void mkUniformArray () {
		talk ("Creating random number array of size " + String.valueOf(arraySize) + "......... ");
		for (int i = 0; i < arraySize; i++) { uniformRnd[i] = Env.mtRNG.raw(); }
		talkln ("  Done.");
	}
	
	private static void mkUnitCircleArray() {
		talk ("Creating random pairs in unit circle, size " + String.valueOf(arraySize) + "........");
		for (int i = 0; i < arraySize; i++) { 
			unitCircleRnd[i] = new UCircRnd(deltaT); 
		}
		talkln (" Done.");
	}
	
	private static void refreshUnitCircleArray() {
		talk ("Creating random pairs in unit circle, size " + String.valueOf(arraySize) + "........");
		for (int i = 0; i < arraySize; i++) { 
			unitCircleRnd[i].newValue(deltaT);
		}
		talkln (" Done.");
	}
	
	public static double uRdm() {
		if (dCtr > arraySize -5) { dCtr = 0; } else { dCtr ++; }
		uRdmCt++;
		return uniformRnd[dCtr];
	}
	
	public static UCircRnd cRdm() {
		if (cCtr > arraySize -5) { cCtr = 0; } else { cCtr ++; }
		cRdmCt++;
		return unitCircleRnd [cCtr];
	}
	
	public static double uRdm(Player curP) {
		curP.uRdmIndex++;
		if (curP.uRdmIndex >= Env.numOfRdmNumbers) {
			curP.uRdmIndex = (int)(Math.floor(Env.mtRNG.raw()*(Env.numOfRdmNumbers-1)));
		}
		uRdmCt++;
		return uniformRnd[curP.uRdmIndex];
	}
	
	public static double uRdm(Linker curL) {
		curL.uRdmIndex++;
		if (curL.uRdmIndex >= Env.numOfRdmNumbers) {
			curL.uRdmIndex = (int)(Math.floor(Env.mtRNG.raw()*(Env.numOfRdmNumbers-1)));
		}
		uRdmCt++;
		return uniformRnd[curL.uRdmIndex];
	}
	
	public static UCircRnd cRdm(Player curP) {
		curP.cRdmIndex++;
		if (curP.cRdmIndex >= Env.numOfRdmNumbers) {
			curP.cRdmIndex = (int)(Math.floor(Env.mtRNG.raw()*(Env.numOfRdmNumbers-1)));
		}
		cRdmCt++;
		return unitCircleRnd[curP.cRdmIndex];
	}
	
	public static UCircRnd cRdm(Linker curL) {
		curL.cRdmIndex++;
		if (curL.cRdmIndex >= Env.numOfRdmNumbers) {
			curL.cRdmIndex = (int)(Math.floor(Env.mtRNG.raw()*(Env.numOfRdmNumbers-1)));
		}
		cRdmCt++;
		return unitCircleRnd[curL.cRdmIndex];
	}
		
	
	private static void talkln (String info) {
		FileOps.talkln(info);
	}
	
	private static void talk (String info) {
		FileOps.talk(info);
	}
}