package rocketBugs;
/* SphImpPlayers : Implicit Grid in Spherical Coordinates... to keep track of diffusion/convection
*/

/*  <RocketBugs - for the simulation of actin-based Listeria motility>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.Math;
import java.awt.*;

public class SphImpPlayers {
	RocketBugs MC;		// link back to main file
	Listeria LM;			// link to listeria object
	Rdm rdm;			// link back to MC.rdm
	static DiffThreads diffTs = new DiffThreads();
	static int [] reacDims = new int [Env.numberOfDiffReacThreads+1];
	static double deltaT;
	static double rMax = Env.rDiffusion;	// radius, in micrometers, to which we carry out diffusion
	static int firstCourse = 0;			//  r-index of first grid point to consider (outside of the Listeria body or 0 if no listeria)
	static double [][][][][] mG;			// array to store concentrations of implicit players
	static int readA = 0;				// this int tells us which part of the array to read from....toggles between 0 and 1
	static int writeA = 1;				// this int tells us which part of the array to write diffusion/reaction results to... either 0 or 1
	static int numDiffusers;				// number of implicit players that we need to calculate diffusion on
	static int [] diffusers;
	static int diffCounter = 0;			// counter for assigning diffusers to threads
	static ImplicitPlayer [] iPlayer = new ImplicitPlayer [Env.numEls];
	static double [] ambientC = new double [Env.numEls];		// ambient concentrations of everything
	static double [] alphaList = new double [Env.numEls];			// the array to store difusivities.
	static double [][] moleIncList;			// an array to hold the molarity increments as functions of r and psi
	static Pt3D [] activeArpPts = new Pt3D[8000];			// an array of Pt3D... points whose grid has active Arp2/3
	static GridPt [] activeArpGrids = new GridPt [8000]; 	// an array of GridPts... those that are Arp2/3 active
	static int activeArpPtCt = 0;								// number of points in activeArpPts
	static int activeArpCt = 0;								// counter for activeArpGrids
	static double rInc, thetaInc, psiInc;
	static int rDim,thetaDim,psiDim;
	static short curTime;
	static Pt3D gridCenter;
	static boolean setForListeria = false; 	// flag to indicate that the grid is already set up for Listeria
	
	public SphImpPlayers (RocketBugs MC, int requestedRDim) {
		this.MC = MC;
		this.rdm = MC.rdm;
		if (Env.listeriaPresent) { 
			LM = Player.listeria; 
			gridCenter = LM.rec1;
			setForListeria = true;
		} else {
			LM = null;
			gridCenter = new Pt3D (Env.dimensions.x/2, Env.dimensions.y/2, Env.dimensions.z/2);	
			setForListeria = false;
		}
		figureRDimAndInc(requestedRDim);
		thetaDim = 36;
		psiDim = 36;
		psiInc = 2*Math.PI/psiDim;
		thetaInc = 2*Math.PI/thetaDim;
		// reacDims
		for (int i = 1; i <= Env.numberOfDiffReacThreads; i++) { reacDims[i] = i*rDim/Env.numberOfDiffReacThreads; }
		reacDims[0] = firstCourse;
		//countDiffusers();
		mG = new double [2][rDim][psiDim][thetaDim][Env.numEls];
		moleIncList = new double [rDim][psiDim];
		calcMoleIncList();
		diffusers = new int [] {};
		
		// **** define diffusivities
		alphaList[Env.ACTIN_ATP] = Env.alphaACTIN;
		alphaList[Env.ACTIN_ADP] = Env.alphaACTIN;
		alphaList[Env.DIMER] = Env.alphaDIMER;
		alphaList[Env.TRIMER] = Env.alphaTRIMER;
		alphaList[Env.WASP] = Env.alphaWASP;
		alphaList[Env.ARP] = Env.alphaARP;
		alphaList[Env.ARPa] = Env.alphaARPa;
		alphaList[Env.ADF] = Env.alphaADF;
		alphaList[Env.ADF_ATP_ACTIN] = Env.alphaADF_ACTIN;
		alphaList[Env.ADF_ADP_ACTIN] = Env.alphaADF_ACTIN;
		alphaList[Env.CAP] = Env.alphaCap;
		alphaList[Env.PROFILIN] = Env.alphaPROFILIN;
		alphaList[Env.PROFILIN_ATP_ACTIN] = Env.alphaPROFILIN_ACTIN;
		alphaList[Env.PROFILIN_ADP_ACTIN] = Env.alphaPROFILIN_ACTIN;
		alphaList[Env.ACTA] = Env.alphaActA;
		alphaList[Env.aACTININ] = Env.alphaActinin;
		alphaList[Env.VASP] = Env.alphaVASP;
		
		// **** define ambient environment
		ambientC[Env.ACTIN_ATP] = Env.ACTIN_ATPConc;
		ambientC[Env.ACTIN_ADP] = Env.ACTIN_ADPConc;
		ambientC[Env.DIMER] = Env.DIMERConc;
		ambientC[Env.TRIMER] = Env.TRIMERConc;
		ambientC[Env.WASP] = Env.WASPConc;
		ambientC[Env.ARP] = Env.ARPConc;
		ambientC[Env.ARPa] = Env.ARPaConc;
		ambientC[Env.ADF] = Env.ADFConc;
		ambientC[Env.ADF_ATP_ACTIN] = Env.ADF_ATP_ACTINConc;
		ambientC[Env.ADF_ADP_ACTIN] = Env.ADF_ADP_ACTINConc;
		ambientC[Env.CAP] = Env.CAPConc;
		ambientC[Env.PROFILIN] = Env.PROFILINConc;
		ambientC[Env.PROFILIN_ATP_ACTIN] = Env.PROFILIN_ATP_ACTINConc;
		ambientC[Env.PROFILIN_ADP_ACTIN] = Env.PROFILIN_ADP_ACTINConc;
		ambientC[Env.ACTA] = Env.ACTAConc;
		ambientC[Env.aACTININ] = Env.aACTININConc;
		ambientC[Env.VASP] = Env.VASPConc;
		
		initializeMolarGrid();
		//mkActiveArpPts();
	}
	
	static class DiffThreads extends ThreadSet {
		DiffThreads () {
			super(Env.numberOfDiffReacThreads, "Diffusion Threads");
		}
		
		public void divideAndConquer (int jobId) {
			this.jobId = jobId;
			switch (jobId) {
				case Env.diffStart:
					resetDiffusibles();	// reset counter that farms out diffusible elements
					spawn(); break;
				case Env.reacStart:
					toggleArray();	// switch read and write functions for array
					spawn(); break;
			}
		}
		
		public void regroup (int jobId) {
			switch (jobId) {
				case Env.diffStop:
					gather(); break;
				case Env.reacStop:
					gather(); break;
			}
		}
		
		public void execute (int threadId) {
			switch (jobId) {
				case Env.diffStart:
					int diffuser = getDiffusible();
					while (diffuser != -1) {
						diffusionCalc (diffuser);
						diffuser = getDiffusible();		// keep getting elements from diffuser list until all are done
					}
					break;
				case Env.reacStart:
					proteinReactions(reacDims[threadId], reacDims[threadId+1]);
					break;
			}
		}
	}
	
	public class GridPt {
		// this is the object used to store grid point coordinates
		int r, theta, psi;
		
		public GridPt (int r, int psi, int theta) {
			this.r = r;
			this.psi = psi;
			this.theta = theta;
			
		}
		
		public GridPt () {
			r = 0;  psi = 0; theta = 0;
		}
	}
	
	public void resetGrid() {	// everything necessary to recreate our spherical grid after loading ICs in which Listeria may or may not be present
		if (Env.listeriaPresent) { 
			LM = Player.listeria; 
			gridCenter = LM.rec1;
		} else {
			LM = null;
			gridCenter = new Pt3D (Env.dimensions.x/2, Env.dimensions.y/2, Env.dimensions.z/2);	
		}
		if (((setForListeria) & (Env.listeriaPresent)) | ((!setForListeria) & (!Env.listeriaPresent))) {
			talk ("...spherical grid is OK as is...");
		} else {
			figureRDimAndInc(rDim);
			mG = new double [2][rDim][psiDim][thetaDim][Env.numEls];
			moleIncList = new double [rDim][psiDim];
			calcMoleIncList();
			initializeMolarGrid();
		}
		if (Env.listeriaPresent) { setForListeria = true; } else { setForListeria = false; }
	}
	
	private void figureRDimAndInc (int requestedRDim) {
		rInc = rMax/requestedRDim;
		if (Env.listeriaPresent) {
			talk ("Fitting spherical diffusion grid to Listeria...");
			firstCourse = 0;
			while (firstCourse*rInc < LM.capRad) { firstCourse ++; }
			rInc = LM.capRad/firstCourse;	// redefine rInc such that start of r-index corresponds to Listeria surface
		}
		rDim = (int) (rMax/rInc);		// redefine rDim such that we go out at least to Env.rDiffusion with rInc defined above
		rMax = rDim*rInc;				// redefine rMax based on the new rInc and rDim
		// report these changes
		talk ("Requested diffusion distance of " + sV(Env.rDiffusion) + " microns in " + sV(requestedRDim) + " radial grid points");
		talkln (" will use " + sV(rMax) + " microns in " + sV(rDim) + " grid points.");
	}
	
	private void countDiffusers () {
		for (int i = 0; i < Env.numEls; i++) {
			if (iPlayer[i].diffuser) { numDiffusers ++; }
		}
		diffusers = new int [numDiffusers];
		int diffNum = 0;
		for (int i = 0; i < Env.numEls; i++) {
			if (iPlayer[i].diffuser) { 
				diffusers[diffNum] = iPlayer[i].idNum; 
				diffNum++;
			}
		}
	}
					
	
	private void calcMoleIncList () {
		for (int i = 0; i < rDim; i++) {
			double rVal = (i+.5)*rInc;
			for (int j = 0; j < psiDim; j++) {
				double psiVal = (j+.5)*psiInc;
				double dV = Math.abs(rVal*rVal*Math.sin(psiVal)*rInc*psiInc*thetaInc);
				moleIncList[i][j] = 1/(dV*Env.microMolarPerOne);
				//if (i == 10) { talkln ("moleInc = " + String.valueOf(moleIncList[i][j]) + " for r = " + String.valueOf(rVal) + " and psi = " + String.valueOf(psiVal));}
			}	
		}
	}
	
	private void initializeMolarGrid () {
		// all molarities are in uM
		for (int i = 0; i < rDim; i++) {
			for (int j = 0; j < psiDim; j++) {
				for (int k = 0; k < thetaDim; k++) {
					for (int m = 0; m < Env.numEls; m++) {
						mG[readA][i][j][k][m] = ambientC[m];
						mG[writeA][i][j][k][m] = ambientC[m];
					}
				}
			}
		}
	}	
		
		
	private boolean inBounds(Pt3D pt) {
		// checks to see if the Pt3D is within the spherical grid region
		double dist = Pt3D.ptDist(pt, gridCenter);
		if (dist < rMax) { return true; } else { return false; }
	}
	
	private GridPt ptToSphereIndex (Pt3D pt) {
		Pt3D vec = Pt3D.Sub(pt, gridCenter);
		double r = Pt3D.vecMag(vec);
		double psi = Math.atan2(vec.y, vec.x);
		double theta = Math.acos(vec.z/r);
		if (psi < 0) { psi += 2*Math.PI; }
		return new GridPt ((int) (r/rInc), (int) (psi/psiInc), (int) (theta/thetaInc));
	}
	
	private Pt3D sphereIndexToMidPt (int R, int Psi, int Theta) {
		double r = (R+0.5)*rInc;
		double theta = (Theta+0.5)*thetaInc;
		double psi = (Psi+0.5)*psiInc;
		double x = r*Math.sin(theta)*Math.cos(psi);
		double y = r*Math.sin(theta)*Math.sin(psi);
		double z = r*Math.cos(theta);
		Pt3D vec = new Pt3D(x,y,z);
		vec.inc(gridCenter);
		return vec;
	}
	
	private Pt3D sphereIndexToRdmPt (int R, int Psi, int Theta) {
		double r = (R+rdm.uRdm())*rInc;
		double theta = (Theta+rdm.uRdm())*thetaInc;
		double psi = (Psi+rdm.uRdm())*psiInc;
		double x = r*Math.sin(theta)*Math.cos(psi);
		double y = r*Math.sin(theta)*Math.sin(psi);
		double z = r*Math.cos(theta);
		Pt3D vec = new Pt3D(x,y,z);
		vec.inc(gridCenter);
		return vec;
	}
	
	// ***** Methods to get values from implicit grid *****
	// core getValue method if point is a Pt3D
	public double getValue (Pt3D pt, int index) {
		if (!Env.useOnlyAmbient) {
			if (inBounds (pt)) { 
				GridPt gPt = ptToSphereIndex (pt);
				return mG [readA][gPt.r][gPt.psi][gPt.theta][index]; 
			} else { 
				return ambientC[index];
			}
		} else {
			return ambientC[index];
		}
	}
	
	// get value methods for points given as Pt3D
	public double getActin_ATPValue (Pt3D pt) { return getValue (pt, Env.ACTIN_ATP); }
	public double getActin_ADPValue (Pt3D pt) { return getValue (pt, Env.ACTIN_ADP); }
	public double getDimerValue (Pt3D pt) { return getValue (pt, Env.DIMER); }
	public double getTrimerValue (Pt3D pt) { return getValue (pt, Env.TRIMER); }
	public double getWASPValue (Pt3D pt) {return getValue (pt, Env.WASP); }
	public double getARPValue (Pt3D pt) {return getValue (pt, Env.ARP); }
	public double getARPaValue (Pt3D pt) {return getValue (pt, Env.ARPa); }
	public double getADFValue (Pt3D pt) {return getValue (pt, Env.ADF); }
	public double getADF_ATP_ACTINValue (Pt3D pt) { return getValue (pt, Env.ADF_ATP_ACTIN); }
	public double getADF_ADP_ACTINValue (Pt3D pt) { return getValue (pt, Env.ADF_ADP_ACTIN); }
	public double getCapValue (Pt3D pt) {return getValue (pt, Env.CAP); }
	public double getProfilinValue (Pt3D pt) {return getValue (pt, Env.PROFILIN); }
	public double getProfilin_ATP_ACTINValue (Pt3D pt) { return getValue (pt, Env.PROFILIN_ATP_ACTIN); }
	public double getProfilin_ADP_ACTINValue (Pt3D pt) { return getValue (pt, Env.PROFILIN_ADP_ACTIN); }
	public double getActAValue (Pt3D pt) {return getValue (pt, Env.ACTA); }
	public double getAlphaActininValue (Pt3D pt) {return getValue (pt, Env.aACTININ); }
	public double getVASPValue (Pt3D pt) {return getValue (pt, Env.VASP); }
	
	// ******** Seed Methods... change mG values *******
	public void addQ (Pt3D pt, int Q, int index) {
		if (Env.reactionsChangeGridConc) {
			if (inBounds (pt)) { 
				GridPt gPt = ptToSphereIndex (pt);
				mG [readA][gPt.r][gPt.psi][gPt.theta][index] += Q*moleIncList[gPt.r][gPt.psi]; 
			} else {
				// do nothing... ambient environment is unchanged by seeding
			}
		}
	}
	
	public void setQ (Pt3D pt, double Q, int index) {
		if (inBounds (pt)) { 
			GridPt gPt = ptToSphereIndex (pt);
			mG [readA][gPt.r][gPt.psi][gPt.theta][index] = Q; 
		} else {
			talkln ("Setting out of bounds");
		}
	}
	
	public void setQ (GridPt gPt, double Q, int index) {
		mG [readA][gPt.r][gPt.psi][gPt.theta][index] = Q;
	}
	
	public void seedActin_ATP (Pt3D pt, int Q) { addQ (pt, Q, Env.ACTIN_ATP); }
	public void seedActin_ADP (Pt3D pt, int Q) { addQ (pt, Q, Env.ACTIN_ADP); }
	public void seedDimer (Pt3D pt, int Q) { addQ (pt, Q, Env.DIMER); }
	public void seedTrimer (Pt3D pt, int Q) { addQ (pt, Q, Env.TRIMER); }
	public void seedWASP (Pt3D pt, int Q) { addQ (pt, Q, Env.WASP); }
	public void seedARP (Pt3D pt, int Q) { addQ (pt, Q, Env.ARP); }
	public void seedARPa (Pt3D pt, int Q) { addQ (pt, Q, Env.ARPa); }
	public void seedADF (Pt3D pt, int Q) { addQ (pt, Q, Env.ADF); }
	public void seedADF_ATP_ACTIN (Pt3D pt, int Q) { addQ (pt, Q, Env.ADF_ATP_ACTIN); }
	public void seedADF_ADP_ACTIN (Pt3D pt, int Q) { addQ (pt, Q, Env.ADF_ADP_ACTIN); }
	public void seedCap (Pt3D pt, int Q) { addQ (pt, Q, Env.CAP); }
	public void seedProfilin (Pt3D pt, int Q) { addQ (pt, Q, Env.PROFILIN); }
	public void seedProfilin_ATP_ACTIN (Pt3D pt, int Q) { addQ (pt, Q, Env.PROFILIN_ATP_ACTIN); }
	public void seedProfilin_ADP_ACTIN (Pt3D pt, int Q) { addQ (pt, Q, Env.PROFILIN_ADP_ACTIN); }
	public void seedAlphaActinin (Pt3D pt, int Q) { addQ (pt, Q, Env.aACTININ); }
	public void seedVASP (Pt3D pt, int Q) { addQ (pt, Q, Env.VASP); }
	public void seedActA (Pt3D pt, int Q) {
		if (inBounds (pt)) {
			GridPt gPt = ptToSphereIndex (pt);
		} else {
			talkln ("Seeding ActA out of bounds");
		}
	} 
	
	public static void resetDiffusibles () {
		diffCounter = 0;
	}
	
	public static void toggleArray () {
		int temp = writeA;
		writeA = readA;
		readA = temp;
	}
	
	public static synchronized int getDiffusible () {
		if (diffCounter < diffusers.length) {
			int el = diffusers[diffCounter];
			diffCounter ++;
			return el;
		} else {
			return -1;
		}
	}
				
	public static void diffusionCalc(int el) {
		double deltaC;
		double cr, cth, cr2, cth2, cpsi2;
		int PsiP1, PsiM1, ThP1, ThM1;
		double r;	
		double th;
		double cosTh, sinTh;
		// calculate the diffusion of the designated element in our molar grid
		double alpha = alphaList[el];
		for (int R = firstCourse; R < rDim; R++) {
			r = (R+.5)*rInc;		// define r as the midpoint radial value for this grid point
			for (int Th = 0; Th < thetaDim; Th++) {
				th = (Th+.5)*thetaInc;
				cosTh = Math.cos(th);
				sinTh = Math.sin(th);
				for (int Psi = 0; Psi < psiDim; Psi++) {
					// *** make Psi access cyclic ***
					if ((Psi == psiDim-1) | (Psi == 0)) {
						if (Psi == psiDim-1) { PsiP1 = 0; PsiM1 = Psi -1; } else { PsiP1 = Psi + 1; PsiM1 = psiDim-1; }
					} else {
						PsiP1 = Psi + 1;
						PsiM1 = Psi - 1;
					}
					// ***************************
					// *** make Th access cyclic ***
					if ((Th == thetaDim-1) | (Th == 0)) {
						if (Th == thetaDim-1) { ThP1 = 0; ThM1 = Th - 1; } else { ThP1 = Th + 1; ThM1 = thetaDim-1; }
					} else {
						ThP1 = Th + 1;
						ThM1 = Th - 1;
					}
					// ***************************
					// *** make R access ambient environment at grid edge *****
					if (R == 0) {
						cr = (mG[readA][R+1][Psi][Th][el] - mG[readA][R][Psi][Th][el])/rInc;
						cr2 = (mG[readA][R+1][Psi][Th][el] - 2*mG[readA][R][Psi][Th][el])/(rInc*rInc);	// CHECK THIS ****
					} else {
						if (R == rDim-1) {
							cr = (ambientC[el] - mG[readA][R][Psi][Th][el])/rInc;
							cr2 = (ambientC[el] - 2*mG[readA][R][Psi][Th][el] + mG[readA][R-1][Psi][Th][el])/(rInc*rInc);
						} else {
							cr = (mG[readA][R+1][Psi][Th][el] - mG[readA][R][Psi][Th][el])/rInc;
							cr2 = (mG[readA][R+1][Psi][Th][el] - 2*mG[readA][R][Psi][Th][el] + mG[readA][R-1][Psi][Th][el])/(rInc*rInc);
						}
					}
					// *******************************************************
					cth = (mG[readA][R][Psi][ThP1][el] - mG[readA][R][Psi][Th][el])/thetaInc;
					cth2 = (mG[readA][R][Psi][ThP1][el] - 2*mG[readA][R][Psi][Th][el] + mG[readA][R][Psi][ThM1][el])/(thetaInc*thetaInc);
					cpsi2 = (mG[readA][R][PsiP1][Th][el] - 2*mG[readA][R][Psi][Th][el] + mG[readA][R][PsiM1][Th][el])/(psiInc*psiInc);
					if (sinTh < 1.0e-5) {
						deltaC = alpha*deltaT*(2*cr/r + cr2 + cth2/(r*r));
					} else {
						deltaC = alpha*deltaT*(2*cr/r + cr2 + cosTh*cth/(r*r*sinTh) + cth2/(r*r) + cpsi2/(r*r*sinTh*sinTh));
					}
					mG[writeA][R][Psi][Th][el] = mG[readA][R][Psi][Th][el] + deltaC;
				}
			}
		}
	}
	
	
	public static void proteinReactions (int startR, int stopR) {
		// each call to this method does only one part of R for all Psi and Theta
		for (int R = startR; R < stopR; R++) {
			for (int Psi = 0; Psi < psiDim; Psi++) {
				for (int Th = 0; Th < thetaDim; Th++) {
					//if (R == firstCourse) { deltaARP(R,Psi,Th);	}	//*** Reactions of ARPa and ARP ***
					actinDimerization(R,Psi,Th);	//*** Take care of new filament creation at each grid pt. ***
					//if (R == firstCourse) { arpFilamentNuc (R,Psi,Th); }	// nucleation of new filaments by active ARPs
					
				}
			}
		}
	}
	
	public void deltaARP (int R, int Psi, int Th) {
		// *** this method takes care of changes to ARPa and ARP
		double amountOfActA = mG[readA][R][Psi][Th][Env.ACTA];	// want this number in M
		double amountOfArp = mG[readA][R][Psi][Th][Env.ARP];		// M
		double amountOfArpa = mG[readA][R][Psi][Th][Env.ARPa];	// M
		double activateFromArp = Env.kActAActinArp_On*amountOfArp*amountOfActA*deltaT;	// M
		double degradeToArp = Env.kActAActinArp_Off*amountOfArpa;
		mG[readA][R][Psi][Th][Env.ARPa] += activateFromArp - degradeToArp;
		mG[readA][R][Psi][Th][Env.ARP] += -activateFromArp;
	}
	
	public static void actinDimerization (int R, int Psi, int Th) {
		double dimerCreation = Env.kDimer_On*mG[readA][R][Psi][Th][Env.ACTIN_ATP]*mG[readA][R][Psi][Th][Env.ACTIN_ATP];
		double dimerDissolution = Env.kDimer_Off*mG[readA][R][Psi][Th][Env.DIMER];
		mG[readA][R][Psi][Th][Env.DIMER] += dimerCreation - dimerDissolution;
		mG[readA][R][Psi][Th][Env.ACTIN_ATP] += -2*dimerCreation + 2*dimerDissolution;
	}
	
	public void arpFilamentNuc (int R, int Psi, int Th) {
		if ((Psi > 0) & (Psi < 8) & (Th > 0) & (Th < 8)) {
			double amountOfArpa = mG[readA][R][Psi][Th][Env.ARPa];	// M
			double amountOfActin_ATP = mG[readA][R][Psi][Th][Env.ACTIN_ATP];	
			double amountOfProfilinATPActin = mG[readA][R][Psi][Th][Env.PROFILIN_ATP_ACTIN];
			double nucProb = Env.kArpFilamentNuc*amountOfArpa*(amountOfActin_ATP+amountOfProfilinATPActin)*deltaT/moleIncList[R][Psi];	// *** Check.. a shortcut to filament nuc
			if (Rdm.uRdm() < nucProb) { // nucleate a new filament in this grid space
				Pt3D newPos = sphereIndexToRdmPt(R,Psi,Th);
				Pt3D newUVec = Pt3D.RandomUnitVec();
				A_Rod rod = A_Rod.makeNewRod (newPos, newUVec, 1);
				rod.minusCapped = true;
			}
		}
	}
	
	
	public void showElementConc (Graphics g, double scale, PinCam cam, int elNum) {
		Color concColor;
		for (int R = firstCourse; R < rDim; R++) {
			for (int Psi = 3; Psi < 7; Psi++) {
				for (int Th = 3; Th < 7; Th++) {
					if (mG[readA][R][Psi][Th][elNum] > ambientC[elNum]) {
						int bNum = (int) (255*(1-(mG[readA][R][Psi][Th][elNum] -ambientC[elNum])));
						if (bNum > 255) { bNum = 255; }
						if (bNum < 0) { bNum = 0; }
						concColor = new Color (255-bNum,255-bNum,255-bNum);
						g.setColor(concColor);
						showGridPt (new GridPt(R,Psi,Th), g, scale, cam);
					}
				}
			}
		}
	}
				
	public void showGridPt (int i, int j, int k, Graphics g, double scale, PinCam cam) {
		showGridPt (new GridPt(i,j,k), g, scale, cam);
	}
	
	public void showGridPt (GridPt gPt, Graphics g, double scale, PinCam cam) {
		Pt3D tempPt = new Pt3D();
		Pixel [][][] pix = new Pixel [2][2][2];
		double [] r = new double [] {gPt.r*rInc, (gPt.r+1)*rInc};
		double [] cosTh = new double [] {Math.cos(gPt.theta*thetaInc), Math.cos((gPt.theta+1)*thetaInc)};
		double [] sinTh = new double [] {Math.sin(gPt.theta*thetaInc), Math.sin((gPt.theta+1)*thetaInc)};
		double [] cosPsi = new double [] {Math.cos(gPt.psi*psiInc), Math.cos((gPt.psi+1)*psiInc)};
		double [] sinPsi = new double [] {Math.sin(gPt.psi*psiInc), Math.sin((gPt.psi+1)*psiInc)};
		for (int i = 0; i < 2; i++) {
			for (int j = 0; j < 2; j++) {
				for (int k = 0; k < 2; k++) {
					tempPt = Pt3D.Add(gridCenter, new Pt3D(r[i]*sinTh[j]*cosPsi[k], r[i]*sinTh[j]*sinPsi[k], r[i]*cosTh[j]));
					pix[i][j][k] = Pixel.ptToNewPix(tempPt, scale, cam.offset, cam.e);
		
				}
			}
		}
		for (int i = 0; i < 2; i++) {
			Pixel.drawLine(g, pix[i][0][0], pix[i][0][1]);
			Pixel.drawLine(g, pix[i][0][1], pix[i][1][1]);
			Pixel.drawLine(g, pix[i][1][1], pix[i][1][0]);
			Pixel.drawLine(g, pix[i][1][0], pix[i][0][0]);
		}
		Pixel.drawLine(g, pix[0][0][0], pix[1][0][0]);
		Pixel.drawLine(g, pix[0][1][0], pix[1][1][0]);
		Pixel.drawLine(g, pix[0][0][1], pix[1][0][1]);
		Pixel.drawLine(g, pix[0][1][1], pix[1][1][1]);
		
		
	}
	
	private void talkln (String info) {
		FileOps.talkln(info);
	}
	
	private void talk (String info) {
		FileOps.talk(info);
	}

	private String sV (double dd) {
		return String.valueOf(dd);
	}
	
	private String sV (int ii) {
		return String.valueOf(ii);
	}
	
	
}
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	