package rocketBugs;
/*
	StereoFrame... the frame to hold the stereo vision views
*/

/*  <RocketBugs - for the simulation of actin-based Listeria motility>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.text.*;

public class StereoFrame extends JFrame implements ChangeListener,WindowListener,KeyListener {
	RBFrame theFrame;		// a reference back to the main jframe
	StereoPanel stereoPanel;
	JSlider magSlider, focalSlider, eyeSepSlider, eyeAngSlider;
	JLabel displayAngle, stereoCenter, focalLength, eyeSep, eyeAng, magVal;
	JPanel infoBox = new JPanel();
	int winSize = 250;
	int magSliderSpace = 20;
	int infoBoxSpace = 20;
	boolean chgAng = false;
	boolean chgZ = false;
	boolean firstDrag = false;
	DecimalFormat coordOut = new DecimalFormat ("#0.00#; -#0.00#");
	// some divisors to give appropriate values from integer sliders
	double focalDiv = 100;
	double eyeSepDiv = 100;
	double magDiv = 1;
	double eyeAngDiv = 10.0;
	
	
	public StereoFrame (double [] cenPt ,double scale, RBFrame theFrame) {
		this.theFrame = theFrame;
		setTitle ("Stereo View");
		setSize(2*winSize+magSliderSpace,winSize+infoBoxSpace);
		setResizable(true);
		setLocation(350,0);
		stereoPanel = new StereoPanel(winSize,cenPt,scale,this);
		makeSliders ();
		makeInfoBox ();
		addWindowListener(this);
		addKeyListener(this);
		addKeyListener(theFrame);
		addMouseListener(theFrame);
		addMouseMotionListener(theFrame);
	}
	
	public void initialize () {
		show();
		getContentPane().add(stereoPanel,"Center");
		stereoPanel.initialize();
		getContentPane().add(infoBox,"South");
		show();
	}
	
	public void updateInfo () {
		displayAngle.setText("  X-Z viewing angle: " + String.valueOf(coordOut.format(stereoPanel.dispAng*180/Math.PI)));
		String [] coordStr = new String [3];
		coordStr[0] = String.valueOf(coordOut.format(stereoPanel.stereoCen.x));
		coordStr[1] = String.valueOf(coordOut.format(stereoPanel.stereoCen.y));
		coordStr[2] = String.valueOf(coordOut.format(stereoPanel.stereoCen.z));
		stereoCenter.setText("  Center Coords: (" + coordStr[0] + "," + coordStr[1] + "," + coordStr[2] + ")");
		magVal.setText("  Magnification: " + String.valueOf(magSlider.getValue()/magDiv));
		focalLength.setText("  Focal Length: " + String.valueOf(focalSlider.getValue()/focalDiv));
		eyeSep.setText("  Eye Separation: " + String.valueOf(eyeSepSlider.getValue()/eyeSepDiv));
		eyeAng.setText("  Eye Angle: " + String.valueOf(eyeAngSlider.getValue()/eyeAngDiv) + " degrees");
	}
	
	public void newStereoCenter (double [] point) {
		stereoPanel.newStereoCenter (point);
		updateInfo ();
	}
	
	public void showStereo (Player[] thePlayers, int numOfPlayers, SphImpPlayers sphImp) {
		stereoPanel.clearImage();
		//stereoPanel.showElementConc (sphImp, 3);
		stereoPanel.showPlayers(thePlayers, numOfPlayers);
		stereoPanel.repaint();
	}
	
	public void newFocus (Player player) {
		Pt3D newFoc = new Pt3D (player.coord);
		newFoc.z += -1.0;
		stereoPanel.newStereoCenter (newFoc);
	}
	
	public void showStereoArea (Graphics g, double mainpictscale) {
		int xCen = (int) (stereoPanel.stereoCen.x*mainpictscale);
		int yCen = (int) (stereoPanel.stereoCen.y*mainpictscale);
		int side = (int) (stereoPanel.winSize*mainpictscale/(stereoPanel.winScale));
		int hfside = side/2;
		int crossDim = 5;
		int [] nwC = {xCen-hfside,yCen-hfside};
		int [] neC = {xCen+hfside,yCen-hfside};
		int [] swC = {xCen-hfside,yCen+hfside};
		int [] seC = {xCen+hfside,yCen+hfside};
		g.setColor(Color.blue);
		g.drawLine(xCen-crossDim,yCen,xCen+crossDim,yCen);
		g.drawLine(xCen,yCen-crossDim,xCen,yCen+crossDim);
		g.drawRect(nwC[0],nwC[1],side,side);
		int dX = (int) (hfside*stereoPanel.trigDispAng[0]);
		int dY = (int) (hfside*stereoPanel.trigDispAng[1]);
		g.drawLine(nwC[0],nwC[1],nwC[0]+dX,nwC[1]-dY);
		g.drawLine(neC[0],neC[1],neC[0]+dX,neC[1]-dY);
		g.drawLine(swC[0],swC[1],swC[0]+dX,swC[1]-dY);
		g.drawLine(seC[0],seC[1],seC[0]+dX,seC[1]-dY);
	}
	
	public void stateChanged (ChangeEvent e) {
		stereoPanel.newScale (magSlider.getValue()/magDiv);
		stereoPanel.newFocalLength(focalSlider.getValue()/focalDiv);
		stereoPanel.newEyeSep(eyeSepSlider.getValue()/eyeSepDiv);
		stereoPanel.newEyeAng(eyeAngSlider.getValue()/eyeAngDiv);
		updateInfo();
	}
	
	public void windowActivated (WindowEvent e) {}
	public void windowClosing (WindowEvent e) {
		theFrame.turnStereoOff();
	}
	public void windowDeactivated (WindowEvent e) {}
	public void windowDeiconified (WindowEvent e) {}
	public void windowIconified (WindowEvent e) {}
	public void windowOpened (WindowEvent e) {}
	public void windowClosed (WindowEvent e) {}
	
	public void keyReleased (KeyEvent e) {
		String act = e.getKeyText(e.getKeyCode());
		if (act.equals("Space")) { chgAng = false; }
		if (act.equals("Z")) { chgZ = false; }
	}
	public void keyTyped (KeyEvent e) {}
	public void keyPressed (KeyEvent e) {
		String act = e.getKeyText(e.getKeyCode());
		if (act.equals("Space")) { chgAng = true; firstDrag = true; }
		if (act.equals("Z")) { chgZ = true; firstDrag = true; }
	}
	
	public void makeSliders () {
		magSlider = new JSlider (0,200,(int)(stereoPanel.scale));
		magSlider.setOrientation(JSlider.HORIZONTAL);
		magSlider.addChangeListener(this);

		focalSlider = new JSlider (0,200,80);
		focalSlider.setOrientation(JSlider.HORIZONTAL);
		focalSlider.addChangeListener(this);
		
		eyeSepSlider = new JSlider (-100,100,0);
		eyeSepSlider.setOrientation(JSlider.HORIZONTAL);
		eyeSepSlider.addChangeListener(this);
		
		eyeAngSlider = new JSlider (-200,200,17);
		eyeAngSlider.setOrientation(JSlider.HORIZONTAL);
		eyeAngSlider.addChangeListener(this);
		
	}
	
	public void makeInfoBox () {
		displayAngle = new JLabel();				// JLabel to show current X-Z viewing angle
		displayAngle.setForeground(Color.black);
		displayAngle.setBackground(Color.black);
		
		stereoCenter = new JLabel();
		stereoCenter.setForeground(Color.black);
		stereoCenter.setBackground(Color.black);
		
		magVal = new JLabel();
		magVal.setForeground(Color.black);
		magVal.setBackground(Color.black);
		
		focalLength = new JLabel();
		focalLength.setForeground(Color.black);
		focalLength.setBackground(Color.black);
		
		eyeSep = new JLabel();
		eyeSep.setForeground(Color.black);
		eyeSep.setBackground(Color.black);
		
		eyeAng = new JLabel();
		eyeAng.setForeground(Color.black);
		eyeAng.setBackground(Color.black);
		
		infoBox.setLayout(new GridLayout(3,2));
		infoBox.add(displayAngle);
		infoBox.add(stereoCenter);
		JPanel magPanel = new JPanel(new GridLayout(1,2));
		magPanel.add(magSlider);
		magPanel.add(magVal);
		infoBox.add(magPanel);
		JPanel focalPanel = new JPanel(new GridLayout(1,2));
		focalPanel.add(focalSlider);
		focalPanel.add(focalLength);
		infoBox.add(focalPanel);
		JPanel eyeSepPanel = new JPanel(new GridLayout(1,2));
		eyeSepPanel.add(eyeSepSlider);
		eyeSepPanel.add(eyeSep);
		infoBox.add(eyeSepPanel);
		JPanel eyeAngPanel = new JPanel(new GridLayout(1,2));
		eyeAngPanel.add(eyeAngSlider);
		eyeAngPanel.add(eyeAng);
		infoBox.add(eyeAngPanel);
		updateInfo();
	}
		
	private void talk (String info) {
		System.out.println(info);
	}
		
}
