package rocketBugs;
/*  Class Util... contains methods, such as vector and matrix operations, that I want to access
      from many different class locations
*/

/*  <RocketBugs - for the simulation of actin-based Listeria motility>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.Math;

public class Util {
	
	public static double ptDist (double [] pt1, double [] pt2) {
		// calculates the distance between two points.  The double arrays passed may be different size
		// so long as the first three entries are (x,y,z).
		double distsqrd = 0;
		for (int i=0; i < 3; i++) {
			distsqrd += Math.pow(pt1[i]-pt2[i],2);
		}
		return Math.sqrt(distsqrd);
	}
	
	public static double [] cross (double [] vec1, double [] vec2) {
		// cross product of 3-space vectors
		double [] vecX = new double [3];
		vecX[0] = vec1[1]*vec2[2] - vec1[2]*vec2[1];
		vecX[1] = vec1[2]*vec2[0] - vec1[0]*vec2[2];
		vecX[2] = vec1[0]*vec2[1] - vec1[1]*vec2[0];
		return vecX;
	}
	
	public static double dot (double [] vec1, double [] vec2) {
		// dot product of 3-space vectors
		return vec1[0]*vec2[0] + vec1[1]*vec2[1] + vec1[2]*vec2[2];
	}
	
	public static double [] vecAdd (double [] vec1, double [] vec2) {
		double [] vecSum = new double [3];
		for (int i=0; i < 3; i++) {
			vecSum[i] = vec1[i] + vec2[i];
		}
		return vecSum;
	}
	
	public static double [] vecSub (double [] vec1, double [] vec2) {
		double [] vecSum = new double [3];
		for (int i=0; i < 3; i++) {
			vecSum[i] = vec1[i] - vec2[i];
		}
		return vecSum;
	}
	
	public static double [] vecSqrd (double [] vector) {
		double [] sqrd = new double [3];
		for (int i = 0; i < 3; i ++) { sqrd[i] = Math.pow(vector[i],2); }
		return sqrd;
	}
	
	public static double vecMag (double [] vector) {
		double magsqrd = 0;
		for (int i=0; i < vector.length; i++) {
			magsqrd += Math.pow(vector[i],2);
		}
		return Math.sqrt(magsqrd);
	}
	
	public static double vecMagSqrd (double [] vector) {
		double magsqrd = 0;
		for (int i=0; i < vector.length; i++) {
			magsqrd += Math.pow(vector[i],2);
		}
		return magsqrd;
	}
	
	public static double [] vecScale (double scalar, double [] vec) {
		double [] scaledVec = new double [vec.length];
		for (int i=0; i < vec.length; i++) {
			scaledVec[i] = scalar*vec[i];
		}
		return scaledVec;
	}
	
	public static double [] xToX (Player player, double [] ptInx) {
		double [] ptInX = new double [3];
		for (int i = 0; i < 3; i ++) {
			for (int j = 0; j < 3; j ++) {
				ptInX[i] += player.transxToX[i][j]*ptInx[j];
			}
		}
		return ptInX;
	}
	
	public static void talk (String info) {
		System.out.println(info);
	}
}
