package rocketBugs;
/* ValueTracker...an object that averages some value over time-steps */

/*  <RocketBugs - for the simulation of actin-based Listeria motility>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.Math;
import java.io.*;
import java.text.*;

public class ValueTracker {
	static final int DOUBLE_TYPE = 0;
	static final int PT3D_TYPE = 1;
	private int placeMark = 0;
	int type = DOUBLE_TYPE;
	private int stepsToTrack;
	double [] vals;
	private Pt3D [] ptVals;
	private Pt3D avePt = new Pt3D();
	
	public ValueTracker (int stepsToTrack, int type) { 
		// the type is either 0 (default) meaning track values or 1 to track Pt3Ds
		this.stepsToTrack = stepsToTrack;
		if (type == PT3D_TYPE) {
			ptVals = new Pt3D [stepsToTrack];
			for (int i=0;i<stepsToTrack;i++) { ptVals[i] = new Pt3D(); }
			this.type = PT3D_TYPE;
		} else {
			vals = new double [stepsToTrack];
		}
	}
		
	public ValueTracker (int stepsToTrack) {
		this.stepsToTrack = stepsToTrack;
		vals = new double [stepsToTrack];
	}
	
	public void registerValue (double newVal) {
		if (placeMark == stepsToTrack) { placeMark = 0; }
		vals[placeMark] = newVal;
		placeMark++;
	}
	
	public void registerValue (Pt3D newPt3D) {
		ptVals[placeMark].copy(newPt3D);
		if (placeMark < (stepsToTrack-1)) { placeMark++; } else { placeMark = 0; }
	}
	
	public double averageVal () {
		double ave = 0;
		for (int i=0; i < stepsToTrack; i++) { ave += vals[i]; }
		ave = ave/stepsToTrack;
		return ave;
	}
	
	public Pt3D averagePtVal () {
		avePt.zero();
		for (int i=0; i < stepsToTrack; i++) { avePt.inc(ptVals[i]); }
		avePt.scale(1.0/((double)stepsToTrack));
		return avePt;
	}
	
	public Pt3D weightedAvePtVal () {
		avePt.zero();
		double weightFactor = 0;
		double weightTotal = 0;
		int weightMark = placeMark;
		for (int i=0; i < stepsToTrack; i++) { 
			if (weightMark == stepsToTrack) { weightMark = 0; }
			weightFactor = 1/((double)(stepsToTrack-i));
			weightTotal += weightFactor;
			avePt.inc(weightFactor,ptVals[weightMark]); 
			weightMark++;
		}
		avePt.scale(1/weightTotal);
		return avePt;
	}
			
	
	
}