package rocketBugs;
/*
	RocketBugsFrame... the frame to hold the zoom panel
*/

/*  <RocketBugs - for the simulation of actin-based Listeria motility>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.text.*;
import infoCCD.*;

public class ZoomFrame extends JFrame implements ChangeListener,WindowListener,KeyListener,ActionListener {
	RBFrame theFrame;		// a reference back to the main jframe
	ZoomPanel zoomPanel;
	JSlider magSlider, focalSlider;
	JLabel displayAngle, zoomCenter;
	JToggleButton rotateButton, rotateRButton;
	Icon rotateIcon, noRotateIcon, rotateRIcon, noRotateRIcon;
	JPanel infoBox = new JPanel();
	JPanel controlBox = new JPanel();
	JPanel southPanel = new JPanel();
	static int winSize = Env.zoomWindowSize;
	int magSliderSpace = 20;
	int infoBoxSpace = 20;
	boolean chgAng = false;
	boolean chgZ = false;
	boolean firstDrag = false;
	DecimalFormat coordOut = new DecimalFormat ("#0.00#; -#0.00#");
	int [] gridPt = new int [3];	// test stuff
	
	static ResourceGetter rG = new ResourceGetter(Info.runType);
	
	
	public ZoomFrame (double [] cenPt ,double scale, RBFrame theFrame) {
		this.theFrame = theFrame;
		setTitle ("  RocketBugs: " + String.valueOf((int)(scale*100)) + " % !!!");
		setSize(winSize+magSliderSpace,winSize+infoBoxSpace);
		setResizable(false);
		setLocation(350,0);
		zoomPanel = new ZoomPanel(winSize,cenPt,scale,this);
		makeMagSlider ();
		makeFocalSlider ();
		makeInfoBox ();
		makeControlBox();
		makeSouthPanel();
		addWindowListener(this);
		addKeyListener(this);
		addKeyListener(theFrame);
		addMouseListener(theFrame);
		addMouseMotionListener(theFrame);
	}
	
	public void initialize () {
		show();
		getContentPane().add(zoomPanel,"Center");
		zoomPanel.initialize();
		getContentPane().add(magSlider,"East");
		getContentPane().add(southPanel,"South");
		show();
	}
	
	public void updateInfo () {
		displayAngle.setText("  X-Z viewing angle: " + String.valueOf(coordOut.format(zoomPanel.dispAng*180/Math.PI)));
		String [] coordStr = new String [3];
		coordStr[0] = String.valueOf(coordOut.format(zoomPanel.zoomCen.x));
		coordStr[1] = String.valueOf(coordOut.format(zoomPanel.zoomCen.y));
		coordStr[2] = String.valueOf(coordOut.format(zoomPanel.zoomCen.z));
		zoomCenter.setText("  Coordinates: (" + coordStr[0] + "," + coordStr[1] + "," + coordStr[2] + ")");
	}
	
	public void newRocketBugsCenter (Pt3D point) {
		zoomPanel.newRocketBugsCenter (point);
		updateInfo ();
	}
	
	public void resetRocketBugsAngle () {
		zoomPanel.newDispAng(0);
		updateInfo ();
	}
	
	public void rotateRocketBugsAngle (double angleInc) {
		zoomPanel.incDispAng(angleInc);
		updateInfo();
	}
	
	public void rotateRocketBugsAngleC () {
		rotateRocketBugsAngle (-Env.viewAngleStep);	// default angle increment
	}
	
	public void rotateRocketBugsAngleCC () {
		rotateRocketBugsAngle (Env.viewAngleStep);	// default angle increment, other way
	}
	
	public void zoomIn () {
		zoomPanel.chgZValue(Env.zoomInc);
		updateInfo();
	}
	
	public void zoomOut () {
		zoomPanel.chgZValue(-Env.zoomInc);
		updateInfo();
	}
	
	
	public void newShowGPt (int i, int j, int k) {
		gridPt[0] = i;
		gridPt[1] = j;
		gridPt[2] = k;
	}
	
	public void showRocketBugs (SphImpPlayers sphImp) {
		zoomPanel.clearImage();
		/*for (int i = 2; i < (int)(sphImp.thetaDim/2)-3; i++) { 
			for (int j = 2; j < (int)(sphImp.psiDim/2)-3; j++) {
				zoomPanel.showSphGridElement(sphImp, Env.courseToShow,i,j);
			}
		} */
		//zoomPanel.showSphGridElement(sphImp, gridPt[0], gridPt[1], gridPt[2]);
		if (Env.elementToShow != -1) { zoomPanel.showElementConc(sphImp, Env.elementToShow); }
		zoomPanel.showPlayers();
		if (Env.showLinkers) { zoomPanel.showLinkers(); }
		//zoomPanel.showActiveArpGrids(sphImp);
		zoomPanel.repaint();
	}
	
	public void showRocketBugsArea (Graphics g, double mainpictscale) {
		int xCen = (int) (zoomPanel.zoomCen.x*mainpictscale);
		int yCen = (int) (zoomPanel.zoomCen.y*mainpictscale);
		double boxSize = zoomPanel.winSize/zoomPanel.winScale;
		int side = (int) (boxSize*mainpictscale);
		int hfside = side/2;
		int crossDim = 5;
		Pixel nwC = new Pixel (xCen-hfside,yCen-hfside);
		Pixel neC = new Pixel (xCen+hfside,yCen-hfside);
		Pixel swC = new Pixel (xCen-hfside,yCen+hfside);
		Pixel seC = new Pixel (xCen+hfside,yCen+hfside);
		g.setColor(Color.green);
		g.drawLine(xCen-crossDim,yCen,xCen+crossDim,yCen);
		g.drawLine(xCen,yCen-crossDim,xCen,yCen+crossDim);
		g.drawRect(nwC.x,nwC.y,side,side);
		int dX = (int) (hfside*zoomPanel.trigDispAng[0]);
		int dY = (int) (hfside*zoomPanel.trigDispAng[1]);
		g.drawLine(nwC.x,nwC.y,nwC.x+dX,nwC.y-dY);
		g.drawLine(neC.x,neC.y,neC.x+dX,neC.y-dY);
		g.drawLine(swC.x,swC.y,swC.x+dX,swC.y-dY);
		g.drawLine(seC.x,seC.y,seC.x+dX,seC.y-dY);
		if (Env.drawScaleBar) {
			String scaleString = String.valueOf(Env.scaleBarFormat.format(boxSize)) + " m";
			g.setFont(new Font ("TimesRoman", Font.PLAIN, 10));
			g.drawString (scaleString, swC.x, swC.y+10);
		}
	}
	
	public void actionPerformed (ActionEvent e) {
		String act = e.getActionCommand();
		
		if (act.equals("To LM")) {
			Pt3D curRec1 = new Pt3D(Player.listeria.rec1);
			newRocketBugsCenter(curRec1);
		}
		
		if (act.equals("Rotate C")) {
			if (Env.rotateRocketBugsAngleC == true) { 
				Env.rotateRocketBugsAngleC = false; 
				rotateButton.setSelectedIcon(noRotateIcon);
			} else { 
				// turn off other rotation, which MIGHT be on
				Env.rotateRocketBugsAngleCC = false;
				rotateRButton.setSelectedIcon(noRotateRIcon);
				// set this rotation button
				Env.rotateRocketBugsAngleC = true; 
				rotateButton.setSelectedIcon(rotateIcon);
			}
		}
		
		if (act.equals("Rotate CC")) {
			if (Env.rotateRocketBugsAngleCC == true) { 
				Env.rotateRocketBugsAngleCC = false; 
				rotateRButton.setSelectedIcon(noRotateRIcon);
			} else { 
				// turn off other rotation, which MIGHT be on
				Env.rotateRocketBugsAngleC = false;
				rotateButton.setSelectedIcon(noRotateIcon);
				//  set this rotation button
				Env.rotateRocketBugsAngleCC = true; 
				rotateRButton.setSelectedIcon(rotateRIcon);
			}
		}
		
		if (act.equals("In")) {
			if (Env.chgZIn) {
				Env.chgZIn = false;
			} else {
				Env.chgZOut = false;
				Env.chgZIn = true;
			}
		}
		
		if (act.equals("Out")) {
			if (Env.chgZOut) {
				Env.chgZOut = false; 
			} else {
				Env.chgZIn = false;
				Env.chgZOut = true;
			}
		}
		
		if (act.equals("EPS Grab")) {
			Env.epsOut = true;
		}
	}
	
	public void stateChanged (ChangeEvent e) {
		zoomPanel.newScale ((double)magSlider.getValue());
		zoomPanel.zoomCam.changeFocalLength(focalSlider.getValue());
		setTitle ("  RocketBugs: " + String.valueOf((int) (zoomPanel.scale*100)) + " % !!!");
	}
	
	public void windowActivated (WindowEvent e) {}
	public void windowClosing (WindowEvent e) {
		theFrame.turnRocketBugsOff();
	}
	public void windowDeactivated (WindowEvent e) {}
	public void windowDeiconified (WindowEvent e) {}
	public void windowIconified (WindowEvent e) {}
	public void windowOpened (WindowEvent e) {}
	public void windowClosed (WindowEvent e) {}
	
	public void keyReleased (KeyEvent e) {
		String act = e.getKeyText(e.getKeyCode());
		if (act.equals("A")) { chgAng = false; }
		if (act.equals("Z")) { chgZ = false; }
	}
	public void keyTyped (KeyEvent e) {}
	public void keyPressed (KeyEvent e) {
		String act = e.getKeyText(e.getKeyCode());
		if (act.equals("A")) { chgAng = true; firstDrag = true; }
		if (act.equals("Z")) { chgZ = true; firstDrag = true; }
		if (act.equals("I")) { 
			talk ("pressed I");
			magSlider.setValue(magSlider.getValue()+10); 
		}
		if (act.equals("O")) { 
			magSlider.setValue(magSlider.getValue()-10); 
		}
	}
	
	public void makeMagSlider () {
		magSlider = new JSlider (100,20000,(int)(zoomPanel.scale*100));
		magSlider.setOrientation(JSlider.VERTICAL);
		//magSlider.setMajorTickSpacing(19900);
		//magSlider.setMinorTickSpacing(1000);
		//magSlider.setPaintTicks(true);
		//magSlider.createStandardLabels(19900);
		//magSlider.setPaintLabels(true);
		magSlider.addChangeListener(this);
	}
	
	public void makeFocalSlider () {
		focalSlider = new JSlider (0,30,1);
		focalSlider.setOrientation(JSlider.VERTICAL);
		focalSlider.addChangeListener(this);
	}
	
	public void makeInfoBox () {
		displayAngle = new JLabel();						// JLabel to show current X-Z viewing angle
		displayAngle.setForeground(Color.black);
		displayAngle.setBackground(Color.black);
		
		zoomCenter = new JLabel();
		zoomCenter.setForeground(Color.black);
		zoomCenter.setBackground(Color.black);
		
		infoBox.setLayout(new GridLayout(2,2));
		infoBox.add(displayAngle);
		infoBox.add(zoomCenter);
		updateInfo();
	}
	
	public void makeControlBox () {
		JButton toLisB = new JButton("To LM");
		toLisB.setActionCommand("To LM");
		toLisB.setToolTipText("Move to Listeria Rec1");
		toLisB.addActionListener(this);
		
		rotateIcon = new ImageIcon(rG.getResource("Pics/rotate.gif"));
		noRotateIcon = new ImageIcon(rG.getResource("Pics/norotate.gif"));
		rotateButton = new JToggleButton(noRotateIcon);
		rotateButton.setSelectedIcon (noRotateIcon);
		rotateButton.setActionCommand ("Rotate C");
		rotateButton.setToolTipText ("Rotate Viewing Angle");
		rotateButton.addActionListener(this);
		
		rotateRIcon = new ImageIcon(rG.getResource("Pics/rotateR.gif"));
		noRotateRIcon = new ImageIcon(rG.getResource("Pics/norotateR.gif"));
		rotateRButton = new JToggleButton(noRotateRIcon);
		rotateRButton.setSelectedIcon (noRotateRIcon);
		rotateRButton.setActionCommand ("Rotate CC");
		rotateRButton.setToolTipText ("Rotate Viewing Angle");
		rotateRButton.addActionListener(this);
		
		JButton inButton = new JButton("In");
		inButton.setActionCommand("In");
		inButton.setToolTipText ("Move forward");
		inButton.addActionListener(this);
		
		JButton outButton = new JButton("Out");
		outButton.setActionCommand("Out");
		outButton.setToolTipText ("Move back");
		outButton.addActionListener(this);
		
		JButton epsGrab = new JButton ("EPS Grab");
		//epsGrab.setFont(Env.controlFont);
		epsGrab.setActionCommand("EPS Grab");
		epsGrab.setToolTipText("Grab an EPS file of the current plot");
		epsGrab.addActionListener(this);
		
		controlBox.setLayout(new FlowLayout());
		
		controlBox.add(toLisB);
		controlBox.add(rotateButton);
		controlBox.add(rotateRButton);
		controlBox.add(inButton);
		controlBox.add(outButton);
		controlBox.add(epsGrab);
	}
	
	public void makeSouthPanel () {
		southPanel.setLayout(new GridLayout(1,1));
		southPanel.add(infoBox);
		southPanel.add(controlBox);
	}
		
	private void talk (String info) {
		System.out.println(info);
	}
		
}
