package rocketBugs;
/* 
	RocketBugsPanel.... the class defining a window to show finer detail
*/

/*  <RocketBugs - for the simulation of actin-based Listeria motility>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import javax.swing.*;
import java.io.*;
import org.jibble.epsgraphics.*;

public class ZoomPanel extends JPanel {
	ZoomFrame pFrame;		// a reference back to the parent JFrame of this panel
	static Image theImage;
	static Graphics2D thePalate;
	static EpsGraphics2D epsG;
	static int winSize;
	double scale,windowfact,winScale;
	double halfWindow;		// dimension of half the zoom window width
	double dispAng = 0;
	double [] trigDispAng = {0,1};  	// store {sin(dispAng), cos(dispAng)}
	Pt3D offset = new Pt3D ();	// store {x,y,z} offsets for the zoom window
	Pt3D zoomCen = new Pt3D ();	// store zoom center
	int [] cursorPos = {0,0};	// store initial cursor position for dispAng and Z changes
	PinCam zoomCam;
	
	// BasicStroke definitions for drawing
	BasicStroke normStroke = new BasicStroke (1.0f,BasicStroke.CAP_ROUND,BasicStroke.JOIN_ROUND);
	BasicStroke wideStroke = new BasicStroke (2.0f,BasicStroke.CAP_ROUND,BasicStroke.JOIN_ROUND);

	public ZoomPanel (int winSize, double [] cenPt, double scale, ZoomFrame pFrame) {
		this.pFrame = pFrame;
		this.winSize = winSize;
		this.scale = scale;
		windowfact = winSize/Env.dimensions.x;
		winScale = scale*windowfact;
		halfWindow = winSize/(2*winScale);
		zoomCam = new PinCam (zoomCen, offset, new Pt3D (0,0,1), .8);
		newRocketBugsCenter(cenPt);
		setSize(winSize,winSize);
	}
	
	public void initialize() {
		theImage = createImage(winSize,winSize);
		setThePalateForScreen();
	}
	
	private void setThePalateForScreen () {
		thePalate = (Graphics2D)theImage.getGraphics();
		thePalate.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
		thePalate.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
		thePalate.setStroke(normStroke);
	}
	
	public boolean openEpsFile () {
		try {
			String epsFileName = FileOps.getFileToSave("Save EPS file as...", pFrame);
			if (epsFileName == null) {
				Env.epsOut = false;
				return false;
			}
			File epsFile = new File(epsFileName);
			String title = "EPS grab";
			epsG = new EpsGraphics2D(title,epsFile,0,0,winSize,winSize);
		} catch (IOException ioe) { talkln ("Problem creating EpsGraphics2D in RocketBugsPanel.java"); }
		thePalate = epsG;	// switch reference to eps file
		return true;
	}
	
	public void closeEpsFile () {
		try {
			epsG.close();
		} catch (IOException ioe) { talkln ("Problem closing epsG in RocketBugsPanel.java"); }
		Env.epsOut = false;
		setThePalateForScreen();	// switch reference back to screen image
	}
	
	public void newRocketBugsCenter (Pt3D zoomCen) {
		this.zoomCen = zoomCen;
		offsetsFromRocketBugsCen ();
		updatePinCam ();
	}
	
	public void newRocketBugsCenter (double [] cenPt) {
		zoomCen.x = cenPt[0];
		zoomCen.y = cenPt[1];
		zoomCen.z = cenPt[2];
		offsetsFromRocketBugsCen ();
		updatePinCam ();
	}
	
	public void offsetsFromRocketBugsCen () {
		offset.y = zoomCen.y - halfWindow;
		offset.x = zoomCen.x - halfWindow*trigDispAng[1];
		offset.z = zoomCen.z + halfWindow*trigDispAng[0];
	}
	
	public void newScale (double newS) {
		scale = newS/100;
		winScale = scale*windowfact;
		halfWindow = winSize/(2*winScale);
		offsetsFromRocketBugsCen();
		updatePinCam ();
	}
	
	public void newDispAng (double newAng) {
		dispAng = newAng;
		if (dispAng > 2*Math.PI) { dispAng += -2*Math.PI; }
		if (dispAng < -2*Math.PI) { dispAng += 2*Math.PI; }
		trigDispAng[0] = Math.sin(dispAng);
		trigDispAng[1] = Math.cos(dispAng);
		offsetsFromRocketBugsCen();
		updatePinCam ();
	}
	
	public void incDispAng (double angleInc) {
		dispAng += angleInc;
		newDispAng(dispAng);
	}
	
	public void chgZValue (double deltaZ) {
		zoomCen.x += deltaZ*trigDispAng[0];
		zoomCen.z += deltaZ*trigDispAng[1];
		offsetsFromRocketBugsCen();
		updatePinCam ();
	}
	
	public void setCursorPos (int x, int y) {
		cursorPos[0] = x;
		cursorPos[1] = y;
	}	
	
	public void chgRocketBugsAng (int x, int y) {
		double newAng = dispAng + (Math.PI/2)*(cursorPos[0]-x)/(winSize/2);
		newDispAng (newAng);
		pFrame.updateInfo();	
	}
	
	public void chgRocketBugsZ (int x, int y) {
		double deltaZ = Env.dimensions.z*(cursorPos[1]-y)/(winSize);
		chgZValue (deltaZ);
		pFrame.updateInfo();	
	}
	
	public void updatePinCam () {
		zoomCam.changePos (zoomCen, offset);
		zoomCam.changeDir(dispAng);
	}
	
	public void update(Graphics g) {
		paint(g);
	}
	
	public void paint (Graphics g) {
		g.drawImage(theImage,0,0,null);
	}
	
	public void showPlayers () {
		Player.showAllPlayers(thePalate, winScale, zoomCam);
	}
	
	public void showLinkers () {
		Linker.showAllLinkers(thePalate, winScale, zoomCam);
	}
	
	public void showSphGridElement(SphImpPlayers sphImp, int i, int j, int k) {
		thePalate.setColor(Env.showGridDefaultColor);
		sphImp.showGridPt(i,j,k, thePalate, winScale, zoomCam);
	}
		
	public void showCellInside (SphImpPlayers sphImp) {
		//sphImp.showMemGrid(thePalate,winScale,offset);
	}
	
	public void showElementConc (SphImpPlayers sphImp, int element) {
		sphImp.showElementConc (thePalate, winScale, zoomCam, element);
	}
	
	public Pt3D convertPixelToPoint (int xPix, int yPix) {
		Pt3D point = new Pt3D();
		point.x = offset.x + trigDispAng[1]*(double)(xPix)/winScale;
		point.y = offset.y + (double)(yPix)/winScale;
		point.z = offset.z - trigDispAng[0]*(double)(xPix)/winScale;
		return point;
	}

	public void clearImage() {
		thePalate.setColor(Env.backColor);
		thePalate.fillRect(0,0,winSize,winSize);
		thePalate.setColor(Env.zoomBoxColor);
		thePalate.setStroke(wideStroke);
		thePalate.drawRect(0,0,winSize-1,winSize-1);
		thePalate.setStroke(normStroke);
	}
	
	private void talk (String info) {
		System.out.print(info);
	}
	
	private void talkln (String info) {
		System.out.println(info);
	}
}